@@ -17,6 +17,7 @@
 #include <osgDB/WriteFile>
 #include <osgDB/FileNameUtils>
 #include <osgDB/ReaderWriter>
+#include <osgDB/PluginQuery>
 
 #include <osgUtil/Optimizer>
 #include <osgUtil/Simplifier>
@@ -492,6 +493,17 @@ static void usage( const char *prog, const char *msg )
                               "                         their own color values (--addMissingColours also accepted)."<< std::endl;
     osg::notify(osg::NOTICE)<<"    --overallNormal    - Replace normals with a single overall normal."<< std::endl;
 
+    osg::notify( osg::NOTICE ) << std::endl;
+    osg::notify( osg::NOTICE ) <<
+        "    --formats          - Lists all supported formats and their supported options." << std::endl;
+    osg::notify( osg::NOTICE ) <<
+        "    --format <format>  - Displays information about the spefied <format>, where <format>\n"
+        "                         is the file extension, such as \"flt\"." << std::endl;
+    osg::notify( osg::NOTICE ) <<
+        "    --plugins          - Lists all supported plugin files." << std::endl;
+    osg::notify( osg::NOTICE ) <<
+        "    --plugin >plugin>  - Displays information about the spefied <plugin>, where <plugin>\n"
+        "                         is the plugin's full path and file name." << std::endl;
 }
 
 
@@ -506,6 +518,8 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display command line parameters");
     arguments.getApplicationUsage()->addCommandLineOption("--help-env","Display environmental variables available");
+    //arguments.getApplicationUsage()->addCommandLineOption("--formats","List supported file formats");
+    //arguments.getApplicationUsage()->addCommandLineOption("--plugins","List database olugins");
 
 
     // if user request help write it out to cout.
@@ -522,6 +536,45 @@ int main( int argc, char **argv )
         return 1;
     }
     
+    if (arguments.read("--plugins"))
+    {
+        osgDB::FileNameList plugins = osgDB::listAllAvailablePlugins();
+        for(osgDB::FileNameList::iterator itr = plugins.begin();
+            itr != plugins.end();
+            ++itr)
+        {
+            std::cout<<"Plugin "<<*itr<<std::endl;
+        }
+        return 0;
+    }    
+    
+    std::string plugin;
+    if (arguments.read("--plugin", plugin))
+    {
+        osgDB::outputPluginDetails(std::cout, plugin);
+        return 0;
+    }    
+    
+    std::string ext;
+    if (arguments.read("--format", ext))
+    {
+        plugin = osgDB::Registry::instance()->createLibraryNameForExtension(ext);
+        osgDB::outputPluginDetails(std::cout, plugin);
+        return 0;
+    }    
+    
+    if (arguments.read("--formats"))
+    {
+        osgDB::FileNameList plugins = osgDB::listAllAvailablePlugins();
+        for(osgDB::FileNameList::iterator itr = plugins.begin();
+            itr != plugins.end();
+            ++itr)
+        {
+            osgDB::outputPluginDetails(std::cout,*itr);
+        }
+        return 0;
+    }    
+    
     if (arguments.argc()<=1)
     {
         arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
@@ -540,7 +593,6 @@ int main( int argc, char **argv )
         osgDB::Registry::instance()->setOptions(options);
     }
 
-    std::string ext;
     while (arguments.read("-e",ext))
     {
         std::string libName = osgDB::Registry::instance()->createLibraryNameForExtension(ext);
