@@ -311,6 +311,7 @@ TypoCorrection typoCorrections[] =
     {"Keuhne",         "Kuehne"},
     {"Kheune",         "Kuehne"},
     {"Lagrade",        "Lagarde"},
+    {"Largade",        "Lagarde"},
     {"Larshkari",      "Lashkari"},
     {"Lashakari",      "Lashkari"},
     {"Lashari",        "Lashkari"},
@@ -377,6 +378,7 @@ TypoCorrection typoCorrections[] =
     {"Sylvan",         "Sylvain"},
     {"Takeahei",       "Takahei"},
     {"Takehei",        "Takahei"},
+    {"Tery",           "Terry"},
     {"Tarantilils",    "Tarantilis"},
     {"Trastenjak",     "Trstenjak"},
     {"Urlich",         "Ulrich"},
