@@ -26,6 +26,17 @@
 #include <osgViewer/Viewer>
 #include <osgViewer/CompositeViewer>
 
+#include <osgGA/TerrainManipulator>
+#include <osgGA/StateSetManipulator>
+#include <osgGA/AnimationPathManipulator>
+#include <osgGA/TrackballManipulator>
+#include <osgGA/FlightManipulator>
+#include <osgGA/DriveManipulator>
+#include <osgGA/KeySwitchMatrixManipulator>
+#include <osgGA/StateSetManipulator>
+#include <osgGA/AnimationPathManipulator>
+#include <osgGA/TerrainManipulator>
+
 #include <osg/Material>
 #include <osg/Geode>
 #include <osg/BlendFunc>
@@ -34,6 +45,7 @@
 #include <osg/MatrixTransform>
 #include <osg/Camera>
 #include <osg/io_utils>
+#include <osg/ShapeDrawable>
 
 #include <osgText/Text>
 
@@ -72,6 +84,18 @@ bool PickHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapte
             if (view) pick(view,ea);
             return false;
         }    
+        case(osgGA::GUIEventAdapter::KEYDOWN):
+        {
+            if (ea.getKey()=='c')
+            {        
+                osgViewer::View* view = dynamic_cast<osgViewer::View*>(&aa);
+                osg::ref_ptr<osgGA::GUIEventAdapter> event = new osgGA::GUIEventAdapter(ea);
+                event->setX((ea.getXmin()+ea.getXmax())*0.5);
+                event->setY((ea.getYmin()+ea.getYmax())*0.5);
+                if (view) pick(view,*event);
+            }
+            return false;
+        }    
         default:
             return false;
     }
@@ -84,8 +108,17 @@ void PickHandler::pick(osgViewer::View* view, const osgGA::GUIEventAdapter& ea)
     std::string gdlist="";
     float x = ea.getX();
     float y = ea.getY();
+#if 0
+    osg::ref_ptr< osgUtil::LineSegmentIntersector > picker = new osgUtil::LineSegmentIntersector(osgUtil::Intersector::WINDOW, x, y);
+    osgUtil::IntersectionVisitor iv(picker.get());
+    view->getCamera()->accept(iv);
+    if (picker->containsIntersections())
+    {
+        intersections = picker->getIntersections();
+#else
     if (view->computeIntersections(x,y,intersections))
     {
+#endif
         for(osgUtil::LineSegmentIntersector::Intersections::iterator hitr = intersections.begin();
             hitr != intersections.end();
             ++hitr)
@@ -218,7 +251,36 @@ int main( int argc, char **argv )
 
     // read the scene from the list of file specified commandline args.
     osg::ref_ptr<osg::Node> scene = osgDB::readNodeFiles(arguments);
-     
+
+    if (!scene && arguments.read("--relative-camera-scene"))
+    {
+        // Create a test scene with a camera that has a relative reference frame.
+        osg::Group* group = new osg::Group();
+
+        osg::Geode* sphere = new osg::Geode();
+        sphere->setName("Sphere");
+        sphere->addDrawable(new osg::ShapeDrawable(new osg::Sphere()));
+
+        osg::Geode* cube = new osg::Geode();
+        cube->setName("Cube");
+        cube->addDrawable(new osg::ShapeDrawable(new osg::Box()));
+
+        osg::Camera* camera = new osg::Camera();
+        camera->setRenderOrder(osg::Camera::POST_RENDER);
+        camera->setClearMask(GL_DEPTH_BUFFER_BIT);
+        camera->setReferenceFrame(osg::Transform::RELATIVE_RF);
+        camera->setViewMatrix(osg::Matrix::translate(-2, 0, 0));
+
+        osg::MatrixTransform* xform = new osg::MatrixTransform(osg::Matrix::translate(1, 1, 1));
+        xform->addChild(camera);
+
+        group->addChild(sphere);
+        group->addChild(xform);
+        camera->addChild(cube);
+
+        scene = group;
+    }
+
     // if not loaded assume no arguments passed in, try use default mode instead.
     if (!scene) scene = osgDB::readNodeFile("fountain.osg");
 
@@ -255,12 +317,42 @@ int main( int argc, char **argv )
     {
         osgViewer::Viewer viewer;
 
+
+        // add all the camera manipulators
+        {
+            osg::ref_ptr<osgGA::KeySwitchMatrixManipulator> keyswitchManipulator = new osgGA::KeySwitchMatrixManipulator;
+
+            keyswitchManipulator->addMatrixManipulator( '1', "Trackball", new osgGA::TrackballManipulator() );
+            keyswitchManipulator->addMatrixManipulator( '2', "Flight", new osgGA::FlightManipulator() );
+            keyswitchManipulator->addMatrixManipulator( '3', "Drive", new osgGA::DriveManipulator() );
+
+            unsigned int num = keyswitchManipulator->getNumMatrixManipulators();
+            keyswitchManipulator->addMatrixManipulator( '4', "Terrain", new osgGA::TerrainManipulator() );
+
+            std::string pathfile;
+            char keyForAnimationPath = '5';
+            while (arguments.read("-p",pathfile))
+            {
+                osgGA::AnimationPathManipulator* apm = new osgGA::AnimationPathManipulator(pathfile);
+                if (apm || !apm->valid()) 
+                {
+                    num = keyswitchManipulator->getNumMatrixManipulators();
+                    keyswitchManipulator->addMatrixManipulator( keyForAnimationPath, "Path", apm );
+                    ++keyForAnimationPath;
+                }
+            }
+
+            keyswitchManipulator->selectMatrixManipulator(num);
+
+            viewer.setCameraManipulator( keyswitchManipulator.get() );
+        }
+
         // add the handler for doing the picking
         viewer.addEventHandler(new PickHandler(updateText.get()));
 
         // set the scene to render
         viewer.setSceneData(group.get());
-
+    
         return viewer.run();
     }
 
