@@ -223,7 +223,6 @@ Drawable::Drawable()
 Drawable::Drawable(const Drawable& drawable,const CopyOp& copyop):
     Object(drawable,copyop),
     _parents(), // leave empty as parentList is managed by Geode
-    _stateset(copyop(drawable._stateset.get())),
     _initialBound(drawable._initialBound),
     _computeBoundCallback(drawable._computeBoundCallback),
     _boundingBox(drawable._boundingBox),
@@ -240,6 +239,7 @@ Drawable::Drawable(const Drawable& drawable,const CopyOp& copyop):
     _cullCallback(drawable._cullCallback),
     _drawCallback(drawable._drawCallback)
 {
+    setStateSet(copyop(drawable._stateset.get()));
 }
 
 Drawable::~Drawable()
@@ -250,6 +250,19 @@ Drawable::~Drawable()
     dirtyDisplayList();
 }
 
+osg::MatrixList Drawable::getWorldMatrices(const osg::Node* haltTraversalAtNode) const
+{
+    osg::MatrixList matrices;
+    for(ParentList::const_iterator itr = _parents.begin();
+        itr != _parents.end();
+        ++itr)
+    {
+        osg::MatrixList localMatrices = (*itr)->getWorldMatrices(haltTraversalAtNode);
+        matrices.insert(matrices.end(), localMatrices.begin(), localMatrices.end());
+    }
+    return matrices;
+}
+
 void Drawable::computeDataVariance()
 {
     if (getDataVariance() != UNSPECIFIED) return;
@@ -268,32 +281,17 @@ void Drawable::computeDataVariance()
 
 void Drawable::addParent(osg::Node* node)
 {
-    if (getRefMutex())
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+    OpenThreads::ScopedPointerLock<OpenThreads::Mutex> lock(getRefMutex());
 
-        _parents.push_back(node);
-    }
-    else
-    {
-        _parents.push_back(node);
-    }
+    _parents.push_back(node);
 }
 
 void Drawable::removeParent(osg::Node* node)
 {
-    if (getRefMutex())
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+    OpenThreads::ScopedPointerLock<OpenThreads::Mutex> lock(getRefMutex());
 
-        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
-        if (pitr!=_parents.end()) _parents.erase(pitr);
-    }
-    else
-    {
-        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
-        if (pitr!=_parents.end()) _parents.erase(pitr);
-    }
+    ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
+    if (pitr!=_parents.end()) _parents.erase(pitr);
 }
 
 
@@ -445,6 +443,17 @@ void Drawable::dirtyBound()
 
 void Drawable::compileGLObjects(RenderInfo& renderInfo) const
 {
+    bool useVertexArrays = _supportsVertexBufferObjects && _useVertexBufferObjects && renderInfo.getState()->isVertexBufferObjectSupported();
+    if (useVertexArrays)
+    {
+        if (_drawCallback.valid())
+            _drawCallback->drawImplementation(renderInfo,this);
+        else
+            drawImplementation(renderInfo);
+
+        return;
+    }
+
     if (!_useDisplayList) return;
 
     // get the contextID (user defined ID of 0 upwards) for the 
