@@ -41,7 +41,25 @@ FBOExtensions* FBOExtensions::instance(unsigned contextID, bool createIfNotInita
 #define LOAD_FBO_EXT(name)    setGLExtensionFuncPtr(name, (#name))
 
 FBOExtensions::FBOExtensions(unsigned int contextID)
-:    _supported(false)
+:   glBindRenderbufferEXT(0),
+    glGenRenderbuffersEXT(0),
+    glDeleteRenderbuffersEXT(0),
+    glRenderbufferStorageEXT(0),
+    glRenderbufferStorageMultisampleEXT(0),
+    glRenderbufferStorageMultisampleCoverageNV(0),
+    glBindFramebufferEXT(0),
+    glDeleteFramebuffersEXT(0),
+    glGenFramebuffersEXT(0),
+    glCheckFramebufferStatusEXT(0),
+    glFramebufferTexture1DEXT(0),
+    glFramebufferTexture2DEXT(0),
+    glFramebufferTexture3DEXT(0),
+    glFramebufferTextureLayerEXT(0),
+    glFramebufferRenderbufferEXT(0),
+    glGenerateMipmapEXT(0),
+    glBlitFramebufferEXT(0),
+    _supported(false),
+    _packed_depth_stencil_supported(false)
 {
     if (!isGLExtensionSupported(contextID, "GL_EXT_framebuffer_object"))
         return;
@@ -75,6 +93,26 @@ FBOExtensions::FBOExtensions(unsigned int contextID)
         glFramebufferTexture3DEXT != 0 &&
         glFramebufferRenderbufferEXT != 0 &&
         glGenerateMipmapEXT != 0;
+
+    if (!isGLExtensionSupported(contextID, "GL_EXT_framebuffer_blit"))
+        return;
+
+    LOAD_FBO_EXT(glBlitFramebufferEXT);
+
+    if (isGLExtensionSupported(contextID, "GL_EXT_framebuffer_multisample"))
+    {
+        LOAD_FBO_EXT(glRenderbufferStorageMultisampleEXT);
+    }
+
+    if (isGLExtensionSupported(contextID, "GL_NV_framebuffer_multisample_coverage"))
+    {
+        LOAD_FBO_EXT(glRenderbufferStorageMultisampleCoverageNV);
+    }
+
+    if (isGLExtensionSupported(contextID, "GL_EXT_packed_depth_stencil"))
+    {
+        _packed_depth_stencil_supported = true;
+    }
 }
 
 
@@ -144,15 +182,19 @@ RenderBuffer::RenderBuffer()
 :    Object(),
     _internalFormat(GL_DEPTH_COMPONENT24),
     _width(512),
-    _height(512)
+    _height(512),
+    _samples(0),
+    _colorSamples(0)
 {
 }
 
-RenderBuffer::RenderBuffer(int width, int height, GLenum internalFormat)
+RenderBuffer::RenderBuffer(int width, int height, GLenum internalFormat, int samples, int colorSamples)
 :    Object(),
     _internalFormat(internalFormat),
     _width(width),
-    _height(height)
+    _height(height),
+    _samples(samples),
+    _colorSamples(colorSamples)
 {
 }
 
@@ -160,7 +202,9 @@ RenderBuffer::RenderBuffer(const RenderBuffer &copy, const CopyOp &copyop)
 :    Object(copy, copyop),
     _internalFormat(copy._internalFormat),
     _width(copy._width),
-    _height(copy._height)
+    _height(copy._height),
+    _samples(copy._samples),
+    _colorSamples(copy._colorSamples)
 {
 }
 
@@ -172,6 +216,20 @@ RenderBuffer::~RenderBuffer()
     }
 }
 
+int RenderBuffer::getMaxSamples(unsigned int contextID, const FBOExtensions *ext)
+{
+    static osg::buffered_value<GLint> maxSamplesList;
+
+    GLint& maxSamples = maxSamplesList[contextID];
+
+    if (!maxSamples && ext->isMultisampleSupported())
+    {
+        glGetIntegerv(GL_MAX_SAMPLES_EXT, &maxSamples);
+    }
+
+    return maxSamples;
+}
+
 GLuint RenderBuffer::getObjectID(unsigned int contextID, const FBOExtensions *ext) const
 {
     GLuint &objectID = _objectID[contextID];
@@ -190,7 +248,35 @@ GLuint RenderBuffer::getObjectID(unsigned int contextID, const FBOExtensions *ex
     {
         // bind and configure
         ext->glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, objectID);
-        ext->glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, _internalFormat, _width, _height);
+
+        // framebuffer_multisample_coverage specification requires that coverage
+        // samples must be >= color samples.
+        if (_samples < _colorSamples)
+        {
+            notify(WARN) << "Coverage samples must be greater than or equal to color samples."
+                " Setting coverage samples equal to color samples." << std::endl;
+            const_cast<RenderBuffer*>(this)->setSamples(_colorSamples);
+        }
+
+        if (_samples > 0 && ext->isMultisampleCoverageSupported())
+        {
+            int samples = minimum(_samples, getMaxSamples(contextID, ext));
+            int colorSamples = minimum(_colorSamples, samples);
+
+            ext->glRenderbufferStorageMultisampleCoverageNV(GL_RENDERBUFFER_EXT,
+                samples, colorSamples, _internalFormat, _width, _height);
+        }
+        else if (_samples > 0 && ext->isMultisampleSupported())
+        {
+            int samples = minimum(_samples, getMaxSamples(contextID, ext));
+
+            ext->glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT,
+                samples, _internalFormat, _width, _height);
+        }
+        else
+        {
+            ext->glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, _internalFormat, _width, _height);
+        }
         dirty = 0;
     }
 
@@ -391,6 +477,16 @@ FrameBufferAttachment &FrameBufferAttachment::operator = (const FrameBufferAttac
     return *this;
 }
 
+bool FrameBufferAttachment::isMultisample() const
+{
+    if (_ximpl->renderbufferTarget.valid())
+    {
+        return _ximpl->renderbufferTarget->getSamples() > 0;
+    }
+
+    return false;
+}
+
 void FrameBufferAttachment::createRequiredTexturesAndApplyGenerateMipMap(State &state, const FBOExtensions* ext) const
 {
     unsigned int contextID = state.getContextID();
@@ -422,7 +518,7 @@ void FrameBufferAttachment::createRequiredTexturesAndApplyGenerateMipMap(State &
     }
 }
 
-void FrameBufferAttachment::attach(State &state, GLenum attachment_point, const FBOExtensions* ext) const
+void FrameBufferAttachment::attach(State &state, GLenum target, GLenum attachment_point, const FBOExtensions* ext) const
 {
     unsigned int contextID = state.getContextID();
 
@@ -444,25 +540,25 @@ void FrameBufferAttachment::attach(State &state, GLenum attachment_point, const
     {
     default:
     case Pimpl::RENDERBUFFER:
-        ext->glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, attachment_point, GL_RENDERBUFFER_EXT, _ximpl->renderbufferTarget->getObjectID(contextID, ext));
+        ext->glFramebufferRenderbufferEXT(target, attachment_point, GL_RENDERBUFFER_EXT, _ximpl->renderbufferTarget->getObjectID(contextID, ext));
         break;
     case Pimpl::TEXTURE1D:
-        ext->glFramebufferTexture1DEXT(GL_FRAMEBUFFER_EXT, attachment_point, GL_TEXTURE_1D, tobj->_id, _ximpl->level);
+        ext->glFramebufferTexture1DEXT(target, attachment_point, GL_TEXTURE_1D, tobj->_id, _ximpl->level);
         break;
     case Pimpl::TEXTURE2D:
-        ext->glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment_point, GL_TEXTURE_2D, tobj->_id, _ximpl->level);
+        ext->glFramebufferTexture2DEXT(target, attachment_point, GL_TEXTURE_2D, tobj->_id, _ximpl->level);
         break;
     case Pimpl::TEXTURE3D:
-        ext->glFramebufferTexture3DEXT(GL_FRAMEBUFFER_EXT, attachment_point, GL_TEXTURE_3D, tobj->_id, _ximpl->level, _ximpl->zoffset);
+        ext->glFramebufferTexture3DEXT(target, attachment_point, GL_TEXTURE_3D, tobj->_id, _ximpl->level, _ximpl->zoffset);
         break;
     case Pimpl::TEXTURE2DARRAY:
-        ext->glFramebufferTextureLayerEXT(GL_FRAMEBUFFER_EXT, attachment_point, tobj->_id, _ximpl->level, _ximpl->zoffset);
+        ext->glFramebufferTextureLayerEXT(target, attachment_point, tobj->_id, _ximpl->level, _ximpl->zoffset);
         break;
     case Pimpl::TEXTURERECT:
-        ext->glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment_point, GL_TEXTURE_RECTANGLE, tobj->_id, 0);
+        ext->glFramebufferTexture2DEXT(target, attachment_point, GL_TEXTURE_RECTANGLE, tobj->_id, 0);
         break;
     case Pimpl::TEXTURECUBE:
-        ext->glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment_point, GL_TEXTURE_CUBE_MAP_POSITIVE_X + _ximpl->cubeMapFace, tobj->_id, _ximpl->level);
+        ext->glFramebufferTexture2DEXT(target, attachment_point, GL_TEXTURE_CUBE_MAP_POSITIVE_X + _ximpl->cubeMapFace, tobj->_id, _ximpl->level);
         break;
     }
 }
@@ -485,6 +581,46 @@ int FrameBufferAttachment::compare(const FrameBufferAttachment &fa) const
     return 0;
 }
 
+RenderBuffer* FrameBufferAttachment::getRenderBuffer()
+{
+    return _ximpl->renderbufferTarget.get();
+}
+
+Texture* FrameBufferAttachment::getTexture()
+{
+    return _ximpl->textureTarget.get();
+}
+
+const RenderBuffer* FrameBufferAttachment::getRenderBuffer() const
+{
+    return _ximpl->renderbufferTarget.get();
+}
+
+const Texture* FrameBufferAttachment::getTexture() const
+{
+    return _ximpl->textureTarget.get();
+}
+
+int FrameBufferAttachment::getCubeMapFace() const
+{
+    return _ximpl->cubeMapFace;
+}
+
+int FrameBufferAttachment::getTextureLevel() const
+{
+    return _ximpl->level;
+}
+
+int FrameBufferAttachment::getTexture3DZOffset() const
+{
+    return _ximpl->zoffset;
+}
+
+int FrameBufferAttachment::getTextureArrayLayer() const
+{
+    return _ximpl->zoffset;
+}
+
 /**************************************************************************
  * FrameBufferObject
  **************************************************************************/
@@ -569,14 +705,8 @@ FrameBufferObject::~FrameBufferObject()
     }
 }
 
-void FrameBufferObject::setAttachment(GLenum attachment_point, const FrameBufferAttachment &attachment)
-{
-    setAttachment(convertGLenumToBufferComponent(attachment_point),attachment);
-}
-
 void FrameBufferObject::setAttachment(BufferComponent attachment_point, const FrameBufferAttachment &attachment)
 {
-    GLenum gl_attachment = convertBufferComponentToGLenum(attachment_point);
     _attachments[attachment_point] = attachment;
 
     updateDrawBuffers();
@@ -595,17 +725,6 @@ GLenum FrameBufferObject::convertBufferComponentToGLenum(BufferComponent attachm
     }
 }
 
-FrameBufferObject::BufferComponent FrameBufferObject::convertGLenumToBufferComponent(GLenum attachment_point) const
-{
-    switch(attachment_point)
-    {
-        case(GL_DEPTH_ATTACHMENT_EXT): return Camera::DEPTH_BUFFER;
-        case(GL_STENCIL_ATTACHMENT_EXT): return Camera::STENCIL_BUFFER;
-        case(GL_COLOR_ATTACHMENT0_EXT): return Camera::COLOR_BUFFER;
-        default: return BufferComponent(Camera::COLOR_BUFFER0+(attachment_point-GL_COLOR_ATTACHMENT0_EXT));
-    }
-}
-
 void FrameBufferObject::updateDrawBuffers()
 {
     _drawBuffers.clear();
@@ -613,8 +732,6 @@ void FrameBufferObject::updateDrawBuffers()
     // create textures and mipmaps before we bind the frame buffer object
     for (AttachmentMap::const_iterator i=_attachments.begin(); i!=_attachments.end(); ++i)
     {
-        const FrameBufferAttachment &fa = i->second;
-
         // setup draw buffers based on the attachment definition
         if (i->first >= Camera::COLOR_BUFFER0 && i->first <= Camera::COLOR_BUFFER15)
             _drawBuffers.push_back(convertBufferComponentToGLenum(i->first));
@@ -622,6 +739,11 @@ void FrameBufferObject::updateDrawBuffers()
 }
 
 void FrameBufferObject::apply(State &state) const
+{
+    apply(state, READ_DRAW_FRAMEBUFFER);
+}
+
+void FrameBufferObject::apply(State &state, BindTarget target) const
 {
     unsigned int contextID = state.getContextID();
 
@@ -639,7 +761,7 @@ void FrameBufferObject::apply(State &state) const
 
     if (_attachments.empty())
     {
-        ext->glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+        ext->glBindFramebufferEXT(target, 0);
         return;
     }
 
@@ -678,7 +800,7 @@ void FrameBufferObject::apply(State &state) const
     }
     
    
-    ext->glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fboID);
+    ext->glBindFramebufferEXT(target, fboID);
 
     // enable drawing buffers to render the result to fbo
     if (_drawBuffers.size() > 0)
@@ -695,13 +817,45 @@ void FrameBufferObject::apply(State &state) const
         for (AttachmentMap::const_iterator i=_attachments.begin(); i!=_attachments.end(); ++i)
         {
             const FrameBufferAttachment &fa = i->second;
-            fa.attach(state, convertBufferComponentToGLenum(i->first), ext);
+            switch(i->first)
+            {
+                case(Camera::PACKED_DEPTH_STENCIL_BUFFER):
+                    if (ext->isPackedDepthStencilSupported())
+                    {
+                        fa.attach(state, target, GL_DEPTH_ATTACHMENT_EXT, ext);
+                        fa.attach(state, target, GL_STENCIL_ATTACHMENT_EXT, ext);
+                    }
+                    else
+                    {
+                        notify(WARN) << 
+                            "Warning: FrameBufferObject: could not attach PACKED_DEPTH_STENCIL_BUFFER, "
+                            "EXT_packed_depth_stencil is not supported !" << std::endl;
+                    }
+                    break;
+
+                default:
+                    fa.attach(state, target, convertBufferComponentToGLenum(i->first), ext);
+                    break;
+            }
         }        
         dirtyAttachmentList = 0;
     }
 
 }
 
+bool FrameBufferObject::isMultisample() const
+{
+    if (_attachments.size())
+    {
+        // If the FBO is correctly set up then all attachments will be either
+        // multisampled or single sampled. Therefore we can just return the
+        // result of the first attachment.
+        return _attachments.begin()->second.isMultisample();
+    }
+
+    return false;
+}
+
 int FrameBufferObject::compare(const StateAttribute &sa) const
 {
     COMPARE_StateAttribute_Types(FrameBufferObject, sa);
