@@ -37,6 +37,9 @@ void GraphicsThread::run()
 
     OperationThread::run();
 
+    // release operations before the thread stops working.
+    _operationQueue->releaseAllOperations(); 
+
     if (graphicsContext)
     {    
         graphicsContext->releaseContext();
@@ -61,7 +64,7 @@ void BarrierOperation::release()
     Barrier::release();
 }
 
-void BarrierOperation::operator () (Object* object)
+void BarrierOperation::operator () (Object* /*object*/)
 {
     if (_preBlockOp!=NO_OPERATION)
     {
@@ -86,10 +89,10 @@ void ReleaseContext_Block_MakeCurrentOperation::operator () (GraphicsContext* co
     // reset the block so that it the next call to block() 
     reset();
     
-    // block this thread, untill the block is released externally.
+    // block this thread, until the block is released externally.
     block();
     
-    // re aquire the graphcis context.
+    // re acquire the graphics context.
     context->makeCurrent();
 }
 
@@ -112,7 +115,8 @@ void BlockAndFlushOperation::operator () (GraphicsContext*)
 }
 
 FlushDeletedGLObjectsOperation::FlushDeletedGLObjectsOperation(double availableTime, bool keep):
-    GraphicsOperation("FlushDeletedGLObjectsOperation",keep)
+    GraphicsOperation("FlushDeletedGLObjectsOperation",keep),
+    _availableTime(availableTime)
 {
 }
 
