@@ -33,33 +33,34 @@ using namespace osg;
 using namespace std;
 
 Image::Image()
-    :Object(true)
+    :Object(true),
+    _fileName(""),
+    _writeHint(NO_PREFERENCE),
+    _origin(BOTTOM_LEFT),
+    _s(0), _t(0), _r(0),
+    _internalTextureFormat(0),
+    _pixelFormat(0),
+    _dataType(0),
+    _packing(4),
+    _pixelAspectRatio(1.0),
+    _allocationMode(USE_NEW_DELETE),
+    _data(0L),
+    _modifiedCount(0)
 {
     setDataVariance(STATIC); 
-
-    _fileName               = "";
-    _origin                 = BOTTOM_LEFT;
-    _s = _t = _r            = 0;
-    _internalTextureFormat  = 0;
-    _pixelFormat            = (unsigned int)0;
-    _dataType               = (unsigned int)0;
-    _packing                = 4;
-
-    _allocationMode         = USE_NEW_DELETE;
-    _data                   = (unsigned char *)0L;
-
-    _modifiedCount = 0;
 }
 
 Image::Image(const Image& image,const CopyOp& copyop):
     Object(image,copyop),
     _fileName(image._fileName),
+    _writeHint(image._writeHint),
     _origin(image._origin),
     _s(image._s), _t(image._t), _r(image._r),
     _internalTextureFormat(image._internalTextureFormat),
     _pixelFormat(image._pixelFormat),
     _dataType(image._dataType),
     _packing(image._packing),
+    _pixelAspectRatio(image._pixelAspectRatio),
     _data(0L),
     _modifiedCount(image._modifiedCount),
     _mipmapData(image._mipmapData)
@@ -203,7 +204,7 @@ GLenum Image::computePixelFormat(GLenum format)
         case(GL_LUMINANCE_ALPHA8UI_EXT):
         case(GL_LUMINANCE_ALPHA16UI_EXT):
         case(GL_LUMINANCE_ALPHA32UI_EXT):
-            return GL_LUMINANCE_ALPHA_INTEGER_EXT;;
+            return GL_LUMINANCE_ALPHA_INTEGER_EXT;
         case(GL_RGB32I_EXT):
         case(GL_RGB16I_EXT):
         case(GL_RGB8I_EXT):
@@ -224,6 +225,62 @@ GLenum Image::computePixelFormat(GLenum format)
     }
 }
 
+GLenum Image::computeFormatDataType(GLenum pixelFormat)
+{
+    switch (pixelFormat)
+    {
+        case GL_LUMINANCE32F_ARB:
+        case GL_LUMINANCE16F_ARB: 
+        case GL_LUMINANCE_ALPHA32F_ARB:
+        case GL_LUMINANCE_ALPHA16F_ARB: 
+        case GL_RGB32F_ARB:
+        case GL_RGB16F_ARB: 
+        case GL_RGBA32F_ARB:
+        case GL_RGBA16F_ARB: return GL_FLOAT;
+
+        case GL_RGBA32UI_EXT:
+        case GL_RGB32UI_EXT:
+        case GL_LUMINANCE32UI_EXT:
+        case GL_LUMINANCE_ALPHA32UI_EXT: return GL_UNSIGNED_INT;
+
+        case GL_RGB16UI_EXT:
+        case GL_RGBA16UI_EXT:
+        case GL_LUMINANCE16UI_EXT: 
+        case GL_LUMINANCE_ALPHA16UI_EXT: return GL_UNSIGNED_SHORT;
+
+        case GL_RGBA8UI_EXT:
+        case GL_RGB8UI_EXT:
+        case GL_LUMINANCE8UI_EXT:
+        case GL_LUMINANCE_ALPHA8UI_EXT:  return GL_UNSIGNED_BYTE;
+
+        case GL_RGBA32I_EXT:  
+        case GL_RGB32I_EXT:
+        case GL_LUMINANCE32I_EXT:
+        case GL_LUMINANCE_ALPHA32I_EXT: return GL_INT;
+
+        case GL_RGBA16I_EXT:
+        case GL_RGB16I_EXT:
+        case GL_LUMINANCE16I_EXT:
+        case GL_LUMINANCE_ALPHA16I_EXT: return GL_SHORT;
+
+        case GL_RGB8I_EXT: 
+        case GL_RGBA8I_EXT: 
+        case GL_LUMINANCE8I_EXT: 
+        case GL_LUMINANCE_ALPHA8I_EXT: return GL_BYTE;
+
+        case GL_RGBA:
+        case GL_RGB:
+        case GL_LUMINANCE:
+        case GL_LUMINANCE_ALPHA: return GL_UNSIGNED_BYTE;
+
+        default: 
+        {
+            notify(WARN)<<"error computeFormatType = "<<std::hex<<pixelFormat<<std::dec<<std::endl;
+            return 0;
+        }
+    }
+}
+
 unsigned int Image::computeNumComponents(GLenum pixelFormat)
 {
     switch(pixelFormat)
@@ -261,6 +318,7 @@ unsigned int Image::computeNumComponents(GLenum pixelFormat)
         case(GL_RGBA32F_ARB): return 4;
         case(GL_RGBA): return 4;
         case(GL_BGRA): return 4;
+        case(GL_RGBA8): return 4;
         case(GL_LUMINANCE): return 1;
         case(GL_LUMINANCE4): return 1;
         case(GL_LUMINANCE8): return 1;
@@ -306,9 +364,20 @@ unsigned int Image::computeNumComponents(GLenum pixelFormat)
         case(GL_DSDT_NV): return 2;
         case(GL_DSDT_MAG_NV): return 3;
         case(GL_DSDT_MAG_VIB_NV): return 4;
+        case(GL_RED_INTEGER_EXT): return 1;
+        case(GL_GREEN_INTEGER_EXT): return 1;
+        case(GL_BLUE_INTEGER_EXT): return 1;
+        case(GL_ALPHA_INTEGER_EXT): return 1;
+        case(GL_RGB_INTEGER_EXT): return 3;
+        case(GL_RGBA_INTEGER_EXT): return 4;
+        case(GL_BGR_INTEGER_EXT): return 3;
+        case(GL_BGRA_INTEGER_EXT): return 4;
+        case(GL_LUMINANCE_INTEGER_EXT): return 1;
+        case(GL_LUMINANCE_ALPHA_INTEGER_EXT): return 2;
+
         default:
         {
-            notify(WARN)<<"error pixelFormat = "<<std::hex<<pixelFormat<<std::endl;
+            notify(WARN)<<"error pixelFormat = "<<std::hex<<pixelFormat<<std::dec<<std::endl;
             return 0;
         }
     }        
@@ -553,7 +622,10 @@ void Image::allocateImage(int s,int t,int r,
         _pixelFormat = format;
         _dataType = type;
         _packing = packing;
-        _internalTextureFormat = format;
+        
+        // preserve internalTextureFormat if already set, otherwise
+        // use the pixelFormat as the source for the format.
+        if (_internalTextureFormat==0) _internalTextureFormat = format;
     }
     else
     {
@@ -565,7 +637,10 @@ void Image::allocateImage(int s,int t,int r,
         _pixelFormat = 0;
         _dataType = 0;
         _packing = 0;
-        _internalTextureFormat = 0;
+        
+        // commenting out reset of _internalTextureFormat as we are changing
+        // policy so that allocateImage honours previous settings of _internalTextureFormat.
+        //_internalTextureFormat = 0;
     }
     
     dirty();
@@ -609,6 +684,8 @@ void Image::readPixels(int x,int y,int width,int height,
 
 void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMapsIfAvailable, GLenum type)
 {
+    // osg::notify(osg::NOTICE)<<"Image::readImageFromCurrentTexture()"<<std::endl;
+
     const osg::Texture::Extensions* extensions = osg::Texture::getExtensions(contextID,true);
     const osg::Texture3D::Extensions* extensions3D = osg::Texture3D::getExtensions(contextID,true);
     const osg::Texture2DArray::Extensions* extensions2DArray = osg::Texture2DArray::getExtensions(contextID,true);
@@ -647,6 +724,7 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
             glGetTexLevelParameteriv(textureMode, numMipMaps, GL_TEXTURE_WIDTH, &width);
             glGetTexLevelParameteriv(textureMode, numMipMaps, GL_TEXTURE_HEIGHT, &height);
             glGetTexLevelParameteriv(textureMode, numMipMaps, GL_TEXTURE_DEPTH, &depth);
+            // osg::notify(osg::NOTICE)<<"   numMipMaps="<<numMipMaps<<" width="<<width<<" height="<<height<<" depth="<<depth<<std::endl;
             if (width==0 || height==0 || depth==0) break;
         }
     }
@@ -654,6 +732,8 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
     {
         numMipMaps = 1;
     }
+    
+    // osg::notify(osg::NOTICE)<<"Image::readImageFromCurrentTexture() : numMipMaps = "<<numMipMaps<<std::endl;
 
         
     GLint compressed = 0;
@@ -859,7 +939,7 @@ void Image::scaleImage(int s,int t,int r, GLenum newDataType)
     dirty();
 }
 
-void Image::copySubImage(int s_offset,int t_offset,int r_offset,osg::Image* source)
+void Image::copySubImage(int s_offset, int t_offset, int r_offset, const osg::Image* source)
 {
     if (!source) return;
     if (s_offset<0 || t_offset<0 || r_offset<0) 
@@ -1202,6 +1282,9 @@ Geode* osg::createGeodeForImage(osg::Image* image)
 }
 
 
+#include <osg/TextureRectangle> 
+
+
 Geode* osg::createGeodeForImage(osg::Image* image,float s,float t)
 {
     if (image)
@@ -1213,7 +1296,22 @@ Geode* osg::createGeodeForImage(osg::Image* image,float s,float t)
             float x = y*(s/t);
 
             // set up the texture.
+
+#if 0
+            osg::TextureRectangle* texture = new osg::TextureRectangle;
+            texture->setFilter(osg::Texture::MIN_FILTER,osg::Texture::LINEAR);
+            texture->setFilter(osg::Texture::MAG_FILTER,osg::Texture::LINEAR);
+            //texture->setResizeNonPowerOfTwoHint(false);
+            float texcoord_x = image->s();
+            float texcoord_y = image->t();
+#else
             osg::Texture2D* texture = new osg::Texture2D;
+            texture->setFilter(osg::Texture::MIN_FILTER,osg::Texture::LINEAR);
+            texture->setFilter(osg::Texture::MAG_FILTER,osg::Texture::LINEAR);
+            texture->setResizeNonPowerOfTwoHint(false);
+            float texcoord_x = 1.0f;
+            float texcoord_y = 1.0f;
+#endif
             texture->setImage(image);
 
             // set up the drawstate.
@@ -1234,10 +1332,10 @@ Geode* osg::createGeodeForImage(osg::Image* image,float s,float t)
             geom->setVertexArray(coords);
 
             Vec2Array* tcoords = new Vec2Array(4);
-            (*tcoords)[0].set(0.0f,1.0f);
-            (*tcoords)[1].set(0.0f,0.0f);
-            (*tcoords)[2].set(1.0f,0.0f);
-            (*tcoords)[3].set(1.0f,1.0f);
+            (*tcoords)[0].set(0.0f*texcoord_x,1.0f*texcoord_y);
+            (*tcoords)[1].set(0.0f*texcoord_x,0.0f*texcoord_y);
+            (*tcoords)[2].set(1.0f*texcoord_x,0.0f*texcoord_y);
+            (*tcoords)[3].set(1.0f*texcoord_x,1.0f*texcoord_y);
             geom->setTexCoordArray(0,tcoords);
 
             osg::Vec4Array* colours = new osg::Vec4Array(1);
