@@ -35,6 +35,8 @@
 #include <map>
 #include <vector>
 
+#include <OpenThreads/Thread>
+
 
 typedef osg::buffered_value< osg::ref_ptr< osg::Drawable::Extensions > > OcclusionQueryBufferedExtensions;
 static OcclusionQueryBufferedExtensions s_OQ_bufferedExtensions;
@@ -232,6 +234,23 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
             osg::notify( osg::DEBUG_INFO ) <<
                 "osgOQ: RQCB: Retrieving..." << std::endl;
 
+#ifdef FORCE_QUERY_RESULT_AVAILABLE_BEFORE_RETRIEVAL
+
+            // Should not need to do this, but is required on some platforms to
+            // work aroung issues in the device driver. For example, without this
+            // code, we've seen crashes on 64-bit Mac/Linux NVIDIA systems doing
+            // multithreaded, multipipe rendering (as in a CAVE).
+            // Tried with ATI and verified this workaround is not needed; the
+            //   problem is specific to NVIDIA.
+            GLint ready( 0 );
+            while( !ready )
+            {
+                // Apparently, must actually sleep here to avoid issues w/ NVIDIA Quadro.
+                OpenThreads::Thread::microSleep( 5 );
+                ext->glGetQueryObjectiv( tr->_id, GL_QUERY_RESULT_AVAILABLE, &ready );
+            };
+#endif
+
             ext->glGetQueryObjectiv( tr->_id, GL_QUERY_RESULT, &(tr->_numPixels) );
             if (tr->_numPixels < 0)
                 osg::notify( osg::WARN ) << "osgOQ: RQCB: " <<
@@ -280,11 +299,11 @@ struct ClearQueriesCallback : public osg::Camera::DrawCallback
     ClearQueriesCallback( const ClearQueriesCallback&, const osg::CopyOp& ) {}
     META_Object( osgOQ, ClearQueriesCallback )
 
-    virtual void operator() (const osg::Camera& camera) const
+    virtual void operator() (const osg::Camera&) const
     {
         if (!_rqcb)
         {
-            osg::notify( osg::FATAL ) << "oagOQ: CQCB: Invalid RQCB." << std::endl;
+            osg::notify( osg::FATAL ) << "osgOQ: CQCB: Invalid RQCB." << std::endl;
             return;
         }
         _rqcb->reset();
@@ -365,7 +384,7 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
         RetrieveQueriesCallback* >( cam->getPostDrawCallback() );
     if (!rqcb)
     {
-        osg::notify( osg::FATAL ) << "oagOQ: QG: Invalid RQCB." << std::endl;
+        osg::notify( osg::FATAL ) << "osgOQ: QG: Invalid RQCB." << std::endl;
         return;
     }
     rqcb->add( tr );
@@ -380,10 +399,10 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
     }
 
     osg::notify( osg::DEBUG_INFO ) <<
-        "oagOQ: QG: Querying for: " << _oqnName << std::endl;
+        "osgOQ: QG: Querying for: " << _oqnName << std::endl;
 
     ext->glBeginQuery( GL_SAMPLES_PASSED_ARB, tr->_id );
-    Geometry::drawImplementation( renderInfo );
+    osg::Geometry::drawImplementation( renderInfo );
     ext->glEndQuery( GL_SAMPLES_PASSED_ARB );
     tr->_active = true;
 
@@ -508,8 +527,6 @@ OcclusionQueryNode::OcclusionQueryNode()
     _queryFrameCount( 5 ),
     _debugBB( false )
 {
-    setDataVariance( osg::Object::DYNAMIC );
-
     // OQN has two Geode member variables, one for doing the
     //   query and one for rendering the debug geometry.
     //   Create and initialize them.
@@ -535,29 +552,49 @@ OcclusionQueryNode::OcclusionQueryNode( const OcclusionQueryNode& oqn, const osg
 
 
 bool
-OcclusionQueryNode::getPassed( const osg::Camera* camera, float distanceToEyePoint )
+OcclusionQueryNode::getPassed( const osg::Camera* camera, osg::NodeVisitor& nv )
 {
     if ( !_enabled )
         // Queries are not enabled. The caller should be osgUtil::CullVisitor,
         //   return true to traverse the subgraphs.
         return true;
 
-    // In the future, we could hold a reference directly to the QueryDrawable
-    //   to avoid the dynamic_cast.
-    QueryGeometry* qg = dynamic_cast< QueryGeometry* >( _queryGeode->getDrawable( 0 ) );
-    if (qg == NULL)
+    {
+        // Two situations where we want to simply do a regular traversal:
+        //  1) it's the first frame for this camers
+        //  2) we haven't rendered for an abnormally long time (probably because we're an out-of-range LOD child)
+        // In these cases, assume we're visible to avoid blinking.
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _frameCountMutex );
+        const int& lastQueryFrame( _frameCountMap[ camera ] );
+        if( ( lastQueryFrame == 0 ) ||
+            ( (nv.getTraversalNumber() - lastQueryFrame) >  (_queryFrameCount + 1) ) )
+            return true;
+    }
+
+    if (_queryGeode->getDrawable( 0 ) == NULL)
     {
         osg::notify( osg::FATAL ) <<
             "osgOQ: OcclusionQueryNode: No QueryGeometry." << std::endl;
         // Something's broke. Return true so we at least render correctly.
         return true;
     }
+    QueryGeometry* qg = static_cast< QueryGeometry* >( _queryGeode->getDrawable( 0 ) );
 
-    // If the distance to the bounding sphere shell is positive, retrieve
-    //   the results. Others (we're inside the BS shell) we are considered
+    // Get the near plane for the upcoming distance calculation.
+    float nearPlane;
+    const osg::Matrix& proj( camera->getProjectionMatrix() );
+    if( ( proj(3,3) != 1. ) || ( proj(2,3) != 0. ) || ( proj(1,3) != 0. ) || ( proj(0,3) != 0.) )
+        nearPlane = proj(3,2) / (proj(2,2)-1.);  // frustum / perspective
+    else
+        nearPlane = (proj(3,2)+1.) / proj(2,2);  // ortho
+
+    // If the distance from the near plane to the bounding sphere shell is positive, retrieve
+    //   the results. Otherwise (near plane inside the BS shell) we are considered
     //   to have passed and don't need to retrieve the query.
     const osg::BoundingSphere& bs = getBound();
-    float distance = distanceToEyePoint  - bs._radius;
+    float distanceToEyePoint = nv.getDistanceToEyePoint( bs._center, false );
+
+    float distance = distanceToEyePoint - nearPlane - bs._radius;
     _passed = ( distance <= 0.f );
     if (!_passed)
     {
@@ -577,7 +614,8 @@ OcclusionQueryNode::traverseQuery( const osg::Camera* camera, osg::NodeVisitor&
 
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _frameCountMutex );
         int& lastQueryFrame = _frameCountMap[ camera ];
-        if ( issueQuery = (curFrame - lastQueryFrame >= _queryFrameCount) )
+        issueQuery = (curFrame - lastQueryFrame >= _queryFrameCount);
+        if (issueQuery)
             lastQueryFrame = curFrame;
     }
     if (issueQuery)
@@ -621,10 +659,10 @@ OcclusionQueryNode::computeBound() const
         (*v)[6] = osg::Vec3( bb._min.x(), bb._max.y(), bb._max.z() );
         (*v)[7] = osg::Vec3( bb._max.x(), bb._max.y(), bb._max.z() );
 
-        osg::Geometry* geom = dynamic_cast< osg::Geometry* >( nonConstThis->_queryGeode->getDrawable( 0 ) );
+        osg::Geometry* geom = static_cast< osg::Geometry* >( nonConstThis->_queryGeode->getDrawable( 0 ) );
         geom->setVertexArray( v.get() );
 
-        geom = dynamic_cast< osg::Geometry* >( nonConstThis->_debugGeode->getDrawable( 0 ) );
+        geom = static_cast< osg::Geometry* >( nonConstThis->_debugGeode->getDrawable( 0 ) );
         geom->setVertexArray( v.get() );
     }
 
@@ -778,10 +816,13 @@ OcclusionQueryNode::createSupportNodes()
 void
 OcclusionQueryNode::releaseGLObjects( osg::State* state ) const
 {
-    // Query object discard and deletion is handled by QueryGeometry support class.
-    OcclusionQueryNode* nonConstThis = const_cast< OcclusionQueryNode* >( this );
-    QueryGeometry* qg = dynamic_cast< QueryGeometry* >( nonConstThis->_queryGeode->getDrawable( 0 ) );
-    qg->releaseGLObjects( state );
+    if(_queryGeode->getDrawable( 0 ) != NULL)
+    {
+        // Query object discard and deletion is handled by QueryGeometry support class.
+        OcclusionQueryNode* nonConstThis = const_cast< OcclusionQueryNode* >( this );
+        QueryGeometry* qg = static_cast< QueryGeometry* >( nonConstThis->_queryGeode->getDrawable( 0 ) );
+        qg->releaseGLObjects( state );
+    }
 }
 
 void
