@@ -79,6 +79,8 @@ State::State():
     _glVertexAttribPointer = 0;
     _glEnableVertexAttribArray = 0;
     _glDisableVertexAttribArray = 0;
+    _glDrawArraysInstanced = 0;
+    _glDrawElementsInstanced = 0;
 
     _dynamicObjectCount  = 0;
 
@@ -88,6 +90,23 @@ State::State():
 
 State::~State()
 {
+    for(AppliedProgramObjectSet::iterator itr = _appliedProgramObjectSet.begin();
+        itr != _appliedProgramObjectSet.end();
+        ++itr)
+    {
+        (*itr)->removeObserver(this);
+    }
+}
+
+void State::objectDeleted(void* object)
+{
+    const Program::PerContextProgram* ppcp = reinterpret_cast<const Program::PerContextProgram*>(object);
+    AppliedProgramObjectSet::iterator itr = _appliedProgramObjectSet.find(ppcp);
+    if (itr != _appliedProgramObjectSet.end()) 
+    {
+        // osg::notify(osg::NOTICE)<<"Removing _appliedProgramObjectSet entry "<<ppcp<<std::endl;
+        _appliedProgramObjectSet.erase(itr);
+    }
 }
 
 void State::reset()
@@ -174,6 +193,7 @@ void State::reset()
         ++apitr)
     {
         (*apitr)->resetAppliedUniforms();
+        (*apitr)->removeObserver(this);
     }
     
     _appliedProgramObjectSet.clear();
@@ -745,14 +765,17 @@ void State::initializeExtensionProcs()
 {
     if (_extensionProcsInitialized) return;
 
-    _glClientActiveTexture = (ActiveTextureProc) osg::getGLExtensionFuncPtr("glClientActiveTexture","glClientActiveTextureARB");
-    _glActiveTexture =  (ActiveTextureProc) osg::getGLExtensionFuncPtr("glActiveTexture","glActiveTextureARB");
-    _glFogCoordPointer = (FogCoordPointerProc) osg::getGLExtensionFuncPtr("glFogCoordPointer","glFogCoordPointerEXT");
-    _glSecondaryColorPointer = (SecondaryColorPointerProc) osg::getGLExtensionFuncPtr("glSecondaryColorPointer","glSecondaryColorPointerEXT");
-    _glVertexAttribPointer =  (VertexAttribPointerProc) osg::getGLExtensionFuncPtr("glVertexAttribPointer","glVertexAttribPointerARB");
-    _glEnableVertexAttribArray =  (EnableVertexAttribProc) osg::getGLExtensionFuncPtr("glEnableVertexAttribArray","glEnableVertexAttribArrayARB");
-    _glDisableVertexAttribArray =  (DisableVertexAttribProc) osg::getGLExtensionFuncPtr("glDisableVertexAttribArray","glDisableVertexAttribArrayARB");
-    _glBindBuffer = (BindBufferProc) osg::getGLExtensionFuncPtr("glBindBuffer","glBindBufferARB");
+    setGLExtensionFuncPtr(_glClientActiveTexture,"glClientActiveTexture","glClientActiveTextureARB");
+    setGLExtensionFuncPtr(_glActiveTexture, "glActiveTexture","glActiveTextureARB");
+    setGLExtensionFuncPtr(_glFogCoordPointer, "glFogCoordPointer","glFogCoordPointerEXT");
+    setGLExtensionFuncPtr(_glSecondaryColorPointer, "glSecondaryColorPointer","glSecondaryColorPointerEXT");
+    setGLExtensionFuncPtr(_glVertexAttribPointer, "glVertexAttribPointer","glVertexAttribPointerARB");
+    setGLExtensionFuncPtr(_glEnableVertexAttribArray, "glEnableVertexAttribArray","glEnableVertexAttribArrayARB");
+    setGLExtensionFuncPtr(_glDisableVertexAttribArray, "glDisableVertexAttribArray","glDisableVertexAttribArrayARB");
+    setGLExtensionFuncPtr(_glBindBuffer, "glBindBuffer","glBindBufferARB");
+
+    setGLExtensionFuncPtr(_glDrawArraysInstanced, "glDrawArraysInstanced","glDrawArraysInstancedARB","glDrawArraysInstancedEXT");
+    setGLExtensionFuncPtr(_glDrawElementsInstanced, "glDrawElementsInstanced","glDrawElementsInstancedARB","glDrawElementsInstancedEXT");
 
     if ( osg::getGLVersionNumber() >= 2.0 || osg::isGLExtensionSupported(_contextID,"GL_ARB_vertex_shader") )
     {
@@ -943,7 +966,7 @@ bool State::checkGLErrors(const char* str) const
     {
         const char* error = (char*)gluErrorString(errorNo);
         if (error) osg::notify(WARN)<<"Warning: detected OpenGL error '" << error<<"'";
-        else       osg::notify(WARN)<<"Warning: detected OpenGL error number 0x" << std::hex << errorNo;
+        else       osg::notify(WARN)<<"Warning: detected OpenGL error number 0x" << std::hex << errorNo << std::dec;
 
         if (str) osg::notify(WARN)<<" at "<<str<< std::endl;
         else     osg::notify(WARN)<<" in osg::State."<< std::endl;
@@ -974,7 +997,7 @@ bool State::checkGLErrors(const StateAttribute* attribute) const
     {
         const char* error = (char*)gluErrorString(errorNo);
         if (error) osg::notify(WARN)<<"Warning: detected OpenGL error '"<< error <<"' after applying attribute "<<attribute->className()<<" "<<attribute<< std::endl;
-        else       osg::notify(WARN)<<"Warning: detected OpenGL error number 0x"<< std::hex << errorNo <<" after applying attribute "<<attribute->className()<<" "<<attribute<< std::endl;
+        else       osg::notify(WARN)<<"Warning: detected OpenGL error number 0x"<< std::hex << errorNo <<" after applying attribute "<<attribute->className()<<" "<<attribute<< std::dec << std::endl;
 
         return true;
     }
