@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2008 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -81,7 +81,8 @@ bool osg::isTextureMode(StateAttribute::GLMode mode)
 }
 
 StateSet::StateSet():
-    Object(true)
+    Object(true),
+    _nestRenderBins(true)
 {
     _renderingHint = DEFAULT_BIN;
     
@@ -91,7 +92,8 @@ StateSet::StateSet():
     setRenderBinToInherit();
 }
 
-StateSet::StateSet(const StateSet& rhs,const CopyOp& copyop):Object(rhs,copyop)
+StateSet::StateSet(const StateSet& rhs,const CopyOp& copyop):Object(rhs,copyop),
+    _nestRenderBins(rhs._nestRenderBins)
 {
     _modeList = rhs._modeList;
 
@@ -249,33 +251,18 @@ void StateSet::computeDataVariance()
 
 void StateSet::addParent(osg::Object* object)
 {
-    // osg::notify(osg::INFO)<<"Adding parent"<<std::endl;
-    if (getRefMutex())
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+    // osg::notify(osg::DEBUG_FP)<<"Adding parent"<<std::endl;
+    OpenThreads::ScopedPointerLock<OpenThreads::Mutex> lock(getRefMutex());
 
-        _parents.push_back(object);
-    }
-    else
-    {
-        _parents.push_back(object);
-    }
+    _parents.push_back(object);
 }
 
 void StateSet::removeParent(osg::Object* object)
 {
-    if (getRefMutex())
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+    OpenThreads::ScopedPointerLock<OpenThreads::Mutex> lock(getRefMutex());
 
-        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),object);
-        if (pitr!=_parents.end()) _parents.erase(pitr);
-    }
-    else
-    {
-        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),object);
-        if (pitr!=_parents.end()) _parents.erase(pitr);
-    }
+    ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),object);
+    if (pitr!=_parents.end()) _parents.erase(pitr);
 }
 
 int StateSet::compare(const StateSet& rhs,bool compareAttributeContents) const
@@ -774,20 +761,21 @@ void StateSet::merge(const StateSet& rhs)
         }
     }
 
-    // need to merge rendering hints
-    // but will need to think how best to do this first
-    // RO, Nov. 2001.
-
+    // Merge RenderBin state from rhs into this.
+    // Only do so if this's RenderBinMode is INHERIT.
+    if (getRenderBinMode() == INHERIT_RENDERBIN_DETAILS)
+    {
+        setRenderingHint( rhs.getRenderingHint() );
+        setRenderBinMode( rhs.getRenderBinMode() );
+        setBinNumber( rhs.getBinNumber() );
+        setBinName( rhs.getBinName() );
+    }
 }
 
 
 void StateSet::setMode(StateAttribute::GLMode mode, StateAttribute::GLModeValue value)
 {
-    if (!getTextureGLModeSet().isTextureMode(mode))
-    {
-        setMode(_modeList,mode,value);
-    }
-    else
+    if (getTextureGLModeSet().isTextureMode(mode))
     {
         notify(NOTICE)<<"Warning: texture mode '"<<mode<<"'passed to setMode(mode,value), "<<std::endl;
         notify(NOTICE)<<"         assuming setTextureMode(unit=0,mode,value) instead."<<std::endl;
@@ -795,15 +783,21 @@ void StateSet::setMode(StateAttribute::GLMode mode, StateAttribute::GLModeValue
 
         setTextureMode(0,mode,value);
     }
+    else if (mode == GL_COLOR_MATERIAL)
+    {
+        notify(NOTICE)<<"Error: Setting mode 'GL_COLOR_MATERIAL' via osg::StateSet::setMode(mode,value) ignored.\n";
+        notify(NOTICE)<<"       The mode 'GL_COLOR_MATERIAL' is set by the osg::Material StateAttribute.\n";
+        notify(NOTICE)<<"       Setting this as a mode fools osg's State tracking."<<std::endl;
+    }
+    else
+    {
+        setMode(_modeList,mode,value);
+    }
 }
 
 void StateSet::removeMode(StateAttribute::GLMode mode)
 {
-    if (!getTextureGLModeSet().isTextureMode(mode))
-    {
-        setModeToInherit(_modeList,mode);
-    }
-    else
+    if (getTextureGLModeSet().isTextureMode(mode))
     {
         notify(NOTICE)<<"Warning: texture mode '"<<mode<<"'passed to setModeToInherit(mode), "<<std::endl;
         notify(NOTICE)<<"         assuming setTextureModeToInherit(unit=0,mode) instead."<<std::endl;
@@ -811,6 +805,16 @@ void StateSet::removeMode(StateAttribute::GLMode mode)
 
         removeTextureMode(0,mode);
     }
+    else if (mode == GL_COLOR_MATERIAL)
+    {
+        notify(NOTICE)<<"Error: Setting mode 'GL_COLOR_MATERIAL' via osg::StateSet::removeMode(mode) ignored.\n";
+        notify(NOTICE)<<"       The mode 'GL_COLOR_MATERIAL' is set by the osg::Material StateAttribute.\n";
+        notify(NOTICE)<<"       Setting this as a mode fools osg's State tracking."<<std::endl;
+    }
+    else
+    {
+        setModeToInherit(_modeList,mode);
+    }
 
 }
 
@@ -1667,7 +1671,7 @@ void StateSet::runUpdateCallbacks(osg::NodeVisitor* nv)
             itr!=_attributeList.end();
             ++itr)
         {
-            StateAttribute::Callback* callback = itr->second.first->getUpdateCallback();
+            StateAttributeCallback* callback = itr->second.first->getUpdateCallback();
             if (callback) (*callback)(itr->second.first.get(),nv);
         }
 
@@ -1680,7 +1684,7 @@ void StateSet::runUpdateCallbacks(osg::NodeVisitor* nv)
                 itr!=attributeList.end();
                 ++itr)
             {
-                StateAttribute::Callback* callback = itr->second.first->getUpdateCallback();
+                StateAttributeCallback* callback = itr->second.first->getUpdateCallback();
                 if (callback) (*callback)(itr->second.first.get(),nv);
             }
         }
@@ -1741,7 +1745,7 @@ void StateSet::runEventCallbacks(osg::NodeVisitor* nv)
             itr!=_attributeList.end();
             ++itr)
         {
-            StateAttribute::Callback* callback = itr->second.first->getEventCallback();
+            StateAttributeCallback* callback = itr->second.first->getEventCallback();
             if (callback) (*callback)(itr->second.first.get(),nv);
         }
 
@@ -1754,7 +1758,7 @@ void StateSet::runEventCallbacks(osg::NodeVisitor* nv)
                 itr!=attributeList.end();
                 ++itr)
             {
-                StateAttribute::Callback* callback = itr->second.first->getEventCallback();
+                StateAttributeCallback* callback = itr->second.first->getEventCallback();
                 if (callback) (*callback)(itr->second.first.get(),nv);
             }
         }
