@@ -42,7 +42,7 @@ using namespace osg;
 #define GL_STORAGE_SHARED_APPLE           0x85BF
 #endif
 
-// #define DO_TIMING
+//#define DO_TIMING
 
 ApplicationUsageProxy Texture_e0(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_MAX_TEXTURE_SIZE","Set the maximum size of textures.");
 
@@ -368,14 +368,14 @@ Texture::Texture():
             _borderColor(0.0, 0.0, 0.0, 0.0),
             _borderWidth(0),
             _internalFormatMode(USE_IMAGE_DATA_FORMAT),
+            _internalFormatType(NORMALIZED),
             _internalFormat(0),
             _sourceFormat(0),
             _sourceType(0),
             _use_shadow_comparison(false),
             _shadow_compare_func(LEQUAL),
             _shadow_texture_mode(LUMINANCE),
-            _shadow_ambient(0),
-            _internalFormatType(NORMALIZED)
+            _shadow_ambient(0)
 {
 }
 
@@ -394,14 +394,14 @@ Texture::Texture(const Texture& text,const CopyOp& copyop):
             _borderColor(text._borderColor),
             _borderWidth(text._borderWidth),
             _internalFormatMode(text._internalFormatMode),
+            _internalFormatType(text._internalFormatType),
             _internalFormat(text._internalFormat),
             _sourceFormat(text._sourceFormat),
             _sourceType(text._sourceType),
             _use_shadow_comparison(text._use_shadow_comparison),
             _shadow_compare_func(text._shadow_compare_func),
             _shadow_texture_mode(text._shadow_texture_mode),
-            _shadow_ambient(text._shadow_ambient),
-            _internalFormatType(text._internalFormatType)
+            _shadow_ambient(text._shadow_ambient)
 {
 }
 
@@ -603,7 +603,7 @@ void Texture::computeInternalFormatWithImage(const osg::Image& image) const
                 switch(image.getPixelFormat())
                 {
                     case(3):
-                    case(GL_RGB):   internalFormat = GL_COMPRESSED_RGB_S3TC_DXT1_EXT; break;
+                    case(GL_RGB):   internalFormat = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT; break;
                     case(4):
                     case(GL_RGBA):  internalFormat = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT; break;
                     default:        internalFormat = image.getInternalTextureFormat(); break;
@@ -618,7 +618,7 @@ void Texture::computeInternalFormatWithImage(const osg::Image& image) const
                 switch(image.getPixelFormat())
                 {
                     case(3):
-                    case(GL_RGB):   internalFormat = GL_COMPRESSED_RGB_S3TC_DXT1_EXT; break;
+                    case(GL_RGB):   internalFormat = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT; break;
                     case(4):
                     case(GL_RGBA):  internalFormat = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT; break;
                     default:        internalFormat = image.getInternalTextureFormat(); break;
@@ -881,7 +881,7 @@ void Texture::computeRequiredTextureDimensions(State& state, const osg::Image& i
     inwidth = width;
     inheight = height;
     
-    bool useHardwareMipMapGeneration = !image.isMipmap() && _useHardwareMipMapGeneration && extensions->isGenerateMipMapSupported();
+    bool useHardwareMipMapGeneration = !image.isMipmap() && isHardwareMipmapGenerationEnabled(state);
 
     if( _min_filter == LINEAR || _min_filter == NEAREST || useHardwareMipMapGeneration )
     {
@@ -906,6 +906,9 @@ void Texture::computeRequiredTextureDimensions(State& state, const osg::Image& i
             height >>= 1;
         }    
     }
+    
+    // osg::notify(osg::NOTICE)<<"Texture::computeRequiredTextureDimensions() image.s() "<<image.s()<<" image.t()="<<image.t()<<" width="<<width<<" height="<<height<<" numMipmapLevels="<<numMipmapLevels<<std::endl; 
+    // osg::notify(osg::NOTICE)<<"  _resizeNonPowerOfTwoHint="<<_resizeNonPowerOfTwoHint<<" extensions->isNonPowerOfTwoTextureSupported(_min_filter)="<<extensions->isNonPowerOfTwoTextureSupported(_min_filter) <<std::endl; 
 }
 
 bool Texture::areAllTextureObjectsLoaded() const
@@ -933,8 +936,6 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
     // current OpenGL context.
     const unsigned int contextID = state.getContextID();
     const Extensions* extensions = getExtensions(contextID,true);
-    
-    bool generateMipMapSupported = extensions->isGenerateMipMapSupported();
 
     // select the internalFormat required for the texture.
     bool compressed_image = isCompressedInternalFormat((GLenum)image->getPixelFormat());
@@ -979,6 +980,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
     
     unsigned char* data = (unsigned char*)image->data();
  
+    // osg::notify(osg::NOTICE)<<"inwidth="<<inwidth<<" inheight="<<inheight<<" image->getFileName()"<<image->getFileName()<<std::endl;
 
     bool needImageRescale = inwidth!=image->s() || inheight!=image->t();
     if (needImageRescale)
@@ -1017,11 +1019,15 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
         
     }    
 
+    bool mipmappingRequired = _min_filter != LINEAR && _min_filter != NEAREST;
+    bool useHardwareMipMapGeneration = mipmappingRequired && (!image->isMipmap() && isHardwareMipmapGenerationEnabled(state));
+    bool useGluBuildMipMaps = mipmappingRequired && (!useHardwareMipMapGeneration && !image->isMipmap());
+
     unsigned char* dataMinusOffset = 0;
     unsigned char* dataPlusOffset = 0;
 
     const PixelBufferObject* pbo = image->getPixelBufferObject();
-    if (pbo && pbo->isPBOSupported(contextID) && !needImageRescale)
+    if (pbo && pbo->isPBOSupported(contextID) && !needImageRescale && !useGluBuildMipMaps)
     {
         state.bindPixelBufferObject(pbo);
         dataMinusOffset = data;
@@ -1035,16 +1041,10 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
         pbo = 0;
     }
 
-    bool useHardwareMipMapGeneration = !image->isMipmap() && _useHardwareMipMapGeneration && generateMipMapSupported;
-    
-    if( _min_filter == LINEAR || _min_filter == NEAREST || useHardwareMipMapGeneration)
+    if( !mipmappingRequired || useHardwareMipMapGeneration)
     {
-        bool hardwareMipMapOn = false;
-        if (_min_filter != LINEAR && _min_filter != NEAREST) 
-        {
-            if (useHardwareMipMapGeneration) glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS,GL_TRUE);
-            hardwareMipMapOn = true;
-        }
+
+        GenerateMipmapMode mipmapResult = mipmapBeforeTexImage(state, useHardwareMipMapGeneration);
 
         if ( !compressed_image)
         {
@@ -1070,7 +1070,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
                 data-dataMinusOffset+dataPlusOffset);                
         }
 
-        if (hardwareMipMapOn) glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS,GL_FALSE);
+        mipmapAfterTexImage(state, mipmapResult);
     }
     else
     {
@@ -1137,7 +1137,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
                 gluBuild2DMipmaps( target, _internalFormat,
                     inwidth,inheight,
                     (GLenum)image->getPixelFormat(), (GLenum)image->getDataType(),
-                    data -dataMinusOffset+dataPlusOffset);
+                    data);
 
                 int width  = image->s();
                 int height = image->t();
@@ -1219,8 +1219,6 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
     // current OpenGL context.
     const unsigned int contextID = state.getContextID();
     const Extensions* extensions = getExtensions(contextID,true);
-    
-    bool generateMipMapSupported = extensions->isGenerateMipMapSupported();
 
     // select the internalFormat required for the texture.
     bool compressed_image = isCompressedInternalFormat((GLenum)image->getPixelFormat());
@@ -1267,13 +1265,16 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
         
     }    
 
-    bool useHardwareMipMapGeneration = !image->isMipmap() && _useHardwareMipMapGeneration && generateMipMapSupported;
-    
-    unsigned char* dataMinusOffset=0;
-    unsigned char* dataPlusOffset=0;
 
+    bool mipmappingRequired = _min_filter != LINEAR && _min_filter != NEAREST;
+    bool useHardwareMipMapGeneration = mipmappingRequired && (!image->isMipmap() && isHardwareMipmapGenerationEnabled(state));
+    bool useGluBuildMipMaps = mipmappingRequired && (!useHardwareMipMapGeneration && !image->isMipmap());
+
+    unsigned char* dataMinusOffset = 0;
+    unsigned char* dataPlusOffset = 0;
+    
     const PixelBufferObject* pbo = image->getPixelBufferObject();
-    if (pbo && pbo->isPBOSupported(contextID) && !needImageRescale)
+    if (pbo && pbo->isPBOSupported(contextID) && !needImageRescale && !useGluBuildMipMaps)
     {
         state.bindPixelBufferObject(pbo);
         dataMinusOffset = data;
@@ -1287,16 +1288,10 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
         pbo = 0;
     }
 
-    if( _min_filter == LINEAR || _min_filter == NEAREST || useHardwareMipMapGeneration)
+    if( !mipmappingRequired || useHardwareMipMapGeneration)
     {
-    
-        bool hardwareMipMapOn = false;
-        if (_min_filter != LINEAR && _min_filter != NEAREST) 
-        {
-            if (useHardwareMipMapGeneration) glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS,GL_TRUE);
-            hardwareMipMapOn = true;
-        }
-        
+
+        GenerateMipmapMode mipmapResult = mipmapBeforeTexImage(state, useHardwareMipMapGeneration);
 
         if (!compressed_image)
         {
@@ -1321,7 +1316,7 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
                 data - dataMinusOffset + dataPlusOffset );                
         }
 
-        if (hardwareMipMapOn) glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS,GL_FALSE);
+        mipmapAfterTexImage(state, mipmapResult);
     }
     else
     {
@@ -1405,6 +1400,79 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
     }
 }
 
+bool Texture::isHardwareMipmapGenerationEnabled(const State& state) const
+{
+    if (_useHardwareMipMapGeneration)
+    {
+        unsigned int contextID = state.getContextID();
+        const Extensions* extensions = getExtensions(contextID,true);
+
+        if (extensions->isGenerateMipMapSupported())
+        {
+            return true;
+        }
+
+        const FBOExtensions* fbo_ext = FBOExtensions::instance(contextID,true);
+
+        if (fbo_ext->glGenerateMipmapEXT)
+        {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+Texture::GenerateMipmapMode Texture::mipmapBeforeTexImage(const State& state, bool hardwareMipmapOn) const
+{
+    if (hardwareMipmapOn)
+    {
+        int width = getTextureWidth();
+        int height = getTextureHeight();
+
+        //quick bithack to determine whether width or height are non-power-of-two
+        if ((width & (width - 1)) || (height & (height - 1)))
+        {
+            //GL_GENERATE_MIPMAP_SGIS with non-power-of-two textures on NVIDIA hardware
+            //is extremely slow. Use glGenerateMipmapEXT() instead if supported.
+            if (_internalFormatType != SIGNED_INTEGER &&
+                _internalFormatType != UNSIGNED_INTEGER)
+            {
+                if (FBOExtensions::instance(state.getContextID(), true)->glGenerateMipmapEXT)
+                {
+                    return GENERATE_MIPMAP;
+                }
+            }
+        }
+
+        glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, GL_TRUE);
+        return GENERATE_MIPMAP_TEX_PARAMETER;
+    }
+    return GENERATE_MIPMAP_NONE;
+}
+
+void Texture::mipmapAfterTexImage(State& state, GenerateMipmapMode beforeResult) const
+{
+    switch (beforeResult)
+    {
+    case GENERATE_MIPMAP:
+        {
+            unsigned int contextID = state.getContextID();
+            TextureObject* textureObject = getTextureObject(contextID);
+            if (textureObject)
+            {
+                osg::FBOExtensions* fbo_ext = osg::FBOExtensions::instance(contextID, true);
+                fbo_ext->glGenerateMipmapEXT(textureObject->_target);
+            }
+        }
+        break;
+    case GENERATE_MIPMAP_TEX_PARAMETER:
+        glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, GL_FALSE);
+        break;
+    case GENERATE_MIPMAP_NONE:
+        break;
+    }
+}
 
 void Texture::generateMipmap(State& state) const
 {
@@ -1566,35 +1634,35 @@ void Texture::Extensions::setupGLExtensions(unsigned int contextID)
         return;
     }
     
-
-    float glVersion = atof( (const char *)version );
     const char* renderer = (const char*) glGetString(GL_RENDERER);
     std::string rendererString(renderer ? renderer : "");
     
-    _isMultiTexturingSupported = ( glVersion >= 1.3 ) ||
-                                 isGLExtensionSupported(contextID,"GL_ARB_multitexture") ||
-                                 isGLExtensionSupported(contextID,"GL_EXT_multitexture");
+    _isMultiTexturingSupported = isGLExtensionOrVersionSupported( contextID,"GL_ARB_multitexture", 1.3f) ||
+                                 isGLExtensionOrVersionSupported(contextID,"GL_EXT_multitexture", 1.3f);
+                                 
     _isTextureFilterAnisotropicSupported = isGLExtensionSupported(contextID,"GL_EXT_texture_filter_anisotropic");
-    _isTextureCompressionARBSupported = ( glVersion >= 1.3 ) ||
-                                        isGLExtensionSupported(contextID,"GL_ARB_texture_compression");
+    
+    _isTextureCompressionARBSupported = isGLExtensionOrVersionSupported(contextID,"GL_ARB_texture_compression", 1.3f);
+    
     _isTextureCompressionS3TCSupported = isGLExtensionSupported(contextID,"GL_EXT_texture_compression_s3tc");
-    _isTextureMirroredRepeatSupported = ( glVersion >= 1.4 ) || 
-                                    isGLExtensionSupported(contextID,"GL_IBM_texture_mirrored_repeat") ||
-                                    isGLExtensionSupported(contextID,"GL_ARB_texture_mirrored_repeat");
-    _isTextureEdgeClampSupported = ( glVersion >= 1.2 ) ||
-                                    isGLExtensionSupported(contextID,"GL_EXT_texture_edge_clamp") || 
-                                    isGLExtensionSupported(contextID,"GL_SGIS_texture_edge_clamp");
-    _isTextureBorderClampSupported = ( glVersion >= 1.3 ) ||
-                                    isGLExtensionSupported(contextID,"GL_ARB_texture_border_clamp");
-    _isGenerateMipMapSupported = (strncmp((const char*)glGetString(GL_VERSION),"1.4",3)>=0) ||
-                                  isGLExtensionSupported(contextID,"GL_SGIS_generate_mipmap");
+    
+    _isTextureMirroredRepeatSupported = isGLExtensionOrVersionSupported(contextID,"GL_IBM_texture_mirrored_repeat", 1.4f) ||
+                                        isGLExtensionOrVersionSupported(contextID,"GL_ARB_texture_mirrored_repeat", 1.4f);
+                                        
+    _isTextureEdgeClampSupported = isGLExtensionOrVersionSupported(contextID,"GL_EXT_texture_edge_clamp", 1.2f) || 
+                                   isGLExtensionOrVersionSupported(contextID,"GL_SGIS_texture_edge_clamp", 1.2f);
+                                   
+    _isTextureBorderClampSupported = isGLExtensionOrVersionSupported(contextID,"GL_ARB_texture_border_clamp", 1.3f);
+    
+    _isGenerateMipMapSupported = isGLExtensionOrVersionSupported(contextID,"GL_SGIS_generate_mipmap", 1.4f);
+                                  
     _isShadowSupported = isGLExtensionSupported(contextID,"GL_ARB_shadow");
+    
     _isShadowAmbientSupported = isGLExtensionSupported(contextID,"GL_ARB_shadow_ambient");
 
     _isClientStorageSupported = isGLExtensionSupported(contextID,"GL_APPLE_client_storage");
 
-    _isNonPowerOfTwoTextureNonMipMappedSupported = ( glVersion >= 2.0 ) ||
-                                       isGLExtensionSupported(contextID,"GL_ARB_texture_non_power_of_two");
+    _isNonPowerOfTwoTextureNonMipMappedSupported = isGLExtensionOrVersionSupported(contextID,"GL_ARB_texture_non_power_of_two", 2.0);
 
     _isNonPowerOfTwoTextureMipMappedSupported = _isNonPowerOfTwoTextureNonMipMappedSupported;
     
