@@ -12,6 +12,7 @@
 */
 #include <osg/GLExtensions>
 #include <osg/Texture1D>
+#include <osg/ImageSequence>
 #include <osg/State>
 #include <osg/GLU>
 
@@ -25,6 +26,13 @@ Texture1D::Texture1D():
 {
 }
 
+Texture1D::Texture1D(osg::Image* image):
+            _textureWidth(0),
+            _numMipmapLevels(0)
+{
+    setImage(image);
+}
+
 Texture1D::Texture1D(const Texture1D& text,const CopyOp& copyop):
             Texture(text,copyop),
             _image(copyop(text._image.get())),
@@ -87,12 +95,25 @@ int Texture1D::compare(const StateAttribute& sa) const
 
 void Texture1D::setImage(Image* image)
 {
+    if (_image == image) return;
+
+    if (dynamic_cast<osg::ImageSequence*>(_image.get()))
+    {
+        setUpdateCallback(0);
+        setDataVariance(osg::Object::STATIC);
+    }
+
     // delete old texture objects.
     dirtyTextureObject();
 
     _image = image;
     _modifiedCount.setAllElementsTo(0);
-
+    
+    if (dynamic_cast<osg::ImageSequence*>(_image.get()))
+    {
+        setUpdateCallback(new ImageSequence::UpdateCallback());
+        setDataVariance(osg::Object::DYNAMIC);
+    }
 }
 
 
@@ -150,7 +171,7 @@ void Texture1D::apply(State& state) const
     {
 
         // we don't have a applyTexImage1D_subload yet so can't reuse.. so just generate a new texture object.        
-        _textureObjectBuffer[contextID] = textureObject = generateTextureObject(contextID,GL_TEXTURE_1D);
+        textureObject = generateTextureObject(contextID,GL_TEXTURE_1D);
 
         textureObject->bind();
 
@@ -163,6 +184,8 @@ void Texture1D::apply(State& state) const
         // update the modified count to show that it is upto date.
         getModifiedCount(contextID) = _image->getModifiedCount();
     
+        _textureObjectBuffer[contextID] = textureObject;
+    
         if (_unrefImageDataAfterApply && areAllTextureObjectsLoaded() && _image->getDataVariance()==STATIC)
         {
             Texture1D* non_const_this = const_cast<Texture1D*>(this);
@@ -238,7 +261,7 @@ void Texture1D::applyTexImage1D(GLenum target, Image* image, State& state, GLsiz
     glPixelStorei(GL_UNPACK_ALIGNMENT,image->getPacking());
 
     static MyCompressedTexImage1DArbProc glCompressedTexImage1D_ptr = 
-        (MyCompressedTexImage1DArbProc)getGLExtensionFuncPtr("glCompressedTexImage1DARB");
+        convertPointerType<MyCompressedTexImage1DArbProc, void*>(getGLExtensionFuncPtr("glCompressedTexImage1DARB"));
 
     if( _min_filter == LINEAR || _min_filter == NEAREST )
     {
