@@ -66,6 +66,13 @@ std::string osgDB::getFileExtension(const std::string& fileName)
     return std::string(fileName.begin()+dot+1,fileName.end());
 }
 
+std::string osgDB::getFileExtensionIncludingDot(const std::string& fileName)
+{
+    std::string::size_type dot = fileName.find_last_of('.');
+    if (dot==std::string::npos) return std::string("");
+    return std::string(fileName.begin()+dot,fileName.end());
+}
+
 std::string osgDB::convertFileNameToWindowsStyle(const std::string& fileName)
 {
     std::string new_fileName(fileName);
@@ -113,17 +120,21 @@ std::string osgDB::convertFileNameToNativeStyle(const std::string& fileName)
 
 std::string osgDB::getLowerCaseFileExtension(const std::string& filename)
 {
-    std::string ext = osgDB::getFileExtension(filename);
-    for(std::string::iterator itr=ext.begin();
-        itr!=ext.end();
+    return convertToLowerCase(osgDB::getFileExtension(filename));
+}
+
+std::string osgDB::convertToLowerCase(const std::string& str)
+{
+    std::string lowcase_str(str);
+    for(std::string::iterator itr=lowcase_str.begin();
+        itr!=lowcase_str.end();
         ++itr)
     {
         *itr = tolower(*itr);
     }
-    return ext;
+    return lowcase_str;
 }
 
-
 // strip one level of extension from the filename.
 std::string osgDB::getNameLessExtension(const std::string& fileName)
 {
@@ -221,6 +232,10 @@ std::string osgDB::concatPaths(const std::string& left, const std::string& right
     const char delimiterForeign = '\\';
 #endif
 
+    if(left.empty())
+    {
+        return(right);
+    }
     char lastChar = left[left.size() - 1];
 
     if(lastChar == delimiterNative)
