@@ -11,7 +11,16 @@
  * OpenSceneGraph Public License for more details.
 */
 
-// currently this impl is for _all_ platforms, execpt as defined.
+// handle TCHAR type on various platforms
+// #ifndef is inspired by https://svn.apache.org/repos/asf/logging/log4cxx/tags/v0_9_4/include/log4cxx/helpers/tchar.h
+// defining type as plain char is from unzip.h, line 64
+
+#ifndef TCHAR
+typedef char TCHAR;
+#endif
+
+
+// currently this impl is for _all_ platforms, except as defined.
 // the mac version will change soon to reflect the path scheme under osx, but
 // for now, the above include is commented out, and the below code takes precedence.
 
@@ -27,8 +36,9 @@
     #define stat64 _stati64
 
     // set up for windows so acts just like unix access().
+#ifndef F_OK
     #define F_OK 4
-
+#endif
 
 #else // unix
 
@@ -51,6 +61,7 @@
     #define stat64 stat
 #endif
 
+    #include <stdlib.h>
     #include <unistd.h>
     #include <sys/types.h>
     #include <sys/stat.h>
@@ -64,6 +75,8 @@
 #  define S_ISDIR(mode)    (mode&__S_IFDIR)
 #endif
 
+#include <osg/Config>
+#include <osgDB/ConvertUTF>
 #include <osg/Notify>
 
 #include <osgDB/FileUtils>
@@ -71,9 +84,37 @@
 #include <osgDB/Registry>
 
 #include <errno.h>
+#include <string.h>
+
 #include <stack>
 
+namespace osgDB
+{
+#ifdef OSG_USE_UTF8_FILENAME
+#define OSGDB_STRING_TO_FILENAME(s) osgDB::convertUTF8toUTF16(s)
+#define OSGDB_FILENAME_TO_STRING(s) osgDB::convertUTF16toUTF8(s)
+#define OSGDB_FILENAME_TEXT(x) L ## x
+#define OSGDB_WINDOWS_FUNCT(x) x ## W
+typedef wchar_t filenamechar;
+typedef std::wstring filenamestring;
+#else
+#define OSGDB_STRING_TO_FILENAME(s) s
+#define OSGDB_FILENAME_TO_STRING(s) s
+#define OSGDB_FILENAME_TEXT(x) x
+#define OSGDB_WINDOWS_FUNCT(x) x ## A
+typedef char filenamechar;
+typedef std::string filenamestring;
+#endif
+}
 
+FILE* osgDB::fopen(const char* filename, const char* mode)
+{
+#ifdef OSG_USE_UTF8_FILENAME
+    return ::_wfopen(convertUTF8toUTF16(filename).c_str(), convertUTF8toUTF16(mode).c_str());
+#else
+    return ::fopen(filename, mode);
+#endif
+}
 
 bool osgDB::makeDirectory( const std::string &path )
 {
@@ -84,7 +125,11 @@ bool osgDB::makeDirectory( const std::string &path )
     }
     
     struct stat64 stbuf;
+#ifdef OSG_USE_UTF8_FILENAME
+    if( _wstat64( OSGDB_STRING_TO_FILENAME(path).c_str(), &stbuf ) == 0 )
+#else
     if( stat64( path.c_str(), &stbuf ) == 0 )
+#endif
     {
         if( S_ISDIR(stbuf.st_mode))
             return true;
@@ -103,7 +148,11 @@ bool osgDB::makeDirectory( const std::string &path )
         if( dir.empty() )
             break;
  
+#ifdef OSG_USE_UTF8_FILENAME
+        if( _wstat64( OSGDB_STRING_TO_FILENAME(dir).c_str(), &stbuf ) < 0 )
+#else
         if( stat64( dir.c_str(), &stbuf ) < 0 )
+#endif
         {
             switch( errno )
             {
@@ -132,7 +181,11 @@ bool osgDB::makeDirectory( const std::string &path )
             }
         #endif
 
+#ifdef OSG_USE_UTF8_FILENAME
+        if ( _wmkdir(OSGDB_STRING_TO_FILENAME(dir).c_str())< 0 )
+#else
         if( mkdir( dir.c_str(), 0755 )< 0 )
+#endif
         {
             osg::notify(osg::DEBUG_INFO) << "osgDB::makeDirectory(): "  << strerror(errno) << std::endl;
             return false;
@@ -147,6 +200,42 @@ bool osgDB::makeDirectoryForFile( const std::string &path )
     return makeDirectory( getFilePath( path ));
 }
 
+
+std::string osgDB::getCurrentWorkingDirectory( void )
+{
+    // MAX_PATH/cwd inspired by unzip.cpp
+#ifndef MAX_PATH
+    #define MAX_PATH 1024
+#endif
+    TCHAR rootdir[MAX_PATH];
+    if(getcwd(rootdir,MAX_PATH-1))
+    {
+        return(rootdir);
+    }
+    return("");
+}// osgDB::getCurrentWorkingDirectory
+
+
+
+bool osgDB::setCurrentWorkingDirectory( const std::string &newCurrentWorkingDirectory )
+{
+    if (newCurrentWorkingDirectory.empty())
+    {
+        osg::notify(osg::DEBUG_INFO) << "osgDB::setCurrentWorkingDirectory(): called with empty string." << std::endl;
+        return false;
+    }
+    
+#ifdef OSG_USE_UTF8_FILENAME
+    return _wchdir( OSGDB_STRING_TO_FILENAME(newCurrentWorkingDirectory).c_str()) == 0;
+#else
+    return chdir( newCurrentWorkingDirectory.c_str()) == 0;
+#endif
+
+    return true;
+} // osgDB::setCurrentWorkingDirectory
+
+
+
 void osgDB::convertStringPathIntoFilePathList(const std::string& paths,FilePathList& filepath)
 {
 #if defined(WIN32) && !defined(__CYGWIN__)
@@ -174,13 +263,21 @@ void osgDB::convertStringPathIntoFilePathList(const std::string& paths,FilePathL
 
 bool osgDB::fileExists(const std::string& filename)
 {
+#ifdef OSG_USE_UTF8_FILENAME
+    return _waccess( OSGDB_STRING_TO_FILENAME(filename).c_str(), F_OK ) == 0;
+#else
     return access( filename.c_str(), F_OK ) == 0;
+#endif
 }
 
 osgDB::FileType osgDB::fileType(const std::string& filename)
 {
     struct stat64 fileStat;
-    if ( stat64(filename.c_str(), &fileStat) != 0 ) 
+#ifdef OSG_USE_UTF8_FILENAME
+    if ( _wstat64(OSGDB_STRING_TO_FILENAME(filename).c_str(), &fileStat) != 0 )
+#else
+    if ( stat64(filename.c_str(), &fileStat) != 0 )
+#endif
     {
         return FILE_NOT_FOUND;
     } // end if
@@ -327,43 +424,136 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
     bool needDirectoryName = true;
     osgDB::DirectoryContents dc;
 
-    if (dirName.empty())
+    std::string realDirName = dirName;
+    std::string realFileName = fileName;
+
+    // Skip case-insensitive recursion if on Windows
+    #ifdef WIN32
+        bool win32 = true;
+    #else
+        bool win32 = false;
+    #endif
+    
+    // If the fileName contains extra path information, make that part of the
+    // directory name instead
+    if (fileName != getSimpleFileName(fileName))
+    {
+        // See if we need to add a slash between the directory and file
+        if (realDirName.empty())
+        {
+            realDirName = getFilePath(fileName);
+        }
+        else if (realDirName=="." || realDirName=="./" || realDirName==".\\")
+        {
+            realDirName = "./" + getFilePath(fileName);
+        }
+        else
+        {
+            char lastChar = dirName[dirName.size()-1];
+            if ((lastChar == '/') || (lastChar == '\\'))
+                realDirName = dirName + getFilePath(fileName);
+            else
+                realDirName = dirName + "/" + getFilePath(fileName);
+        }
+
+        // Simplify the file name
+        realFileName = getSimpleFileName(fileName);
+    }
+
+    osg::notify(osg::DEBUG_INFO) << "findFileInDirectory() : looking for " << realFileName << " in " << realDirName << "...\n";
+
+    if (realDirName.empty())
     {
         dc = osgDB::getDirectoryContents(".");
         needFollowingBackslash = false;
         needDirectoryName = false;
     }
-    else if (dirName=="." || dirName=="./" || dirName==".\\")
+    else if (realDirName=="." || realDirName=="./" || realDirName==".\\")
     {
         dc = osgDB::getDirectoryContents(".");
         needFollowingBackslash = false;
         needDirectoryName = false;
     }
+    else if (realDirName=="/")
+    {
+        dc = osgDB::getDirectoryContents("/");
+        needFollowingBackslash = false;
+        needDirectoryName = true;
+    }
     else
     {
-        dc = osgDB::getDirectoryContents(dirName);
-        char lastChar = dirName[dirName.size()-1];
-        if (lastChar=='/') needFollowingBackslash = false;
-        else if (lastChar=='\\') needFollowingBackslash = false;
-        else needFollowingBackslash = true;
-        needDirectoryName = true;
+        // See if we're working in case insensitive mode, and that we're not
+        // using Windows (the recursive search is not needed in these
+        // cases)
+        if ((caseSensitivity == CASE_INSENSITIVE) && (!win32))
+        {
+            // Split the last path element from the directory name
+            std::string parentPath = getFilePath(realDirName);
+            std::string lastElement = getSimpleFileName(realDirName);
+
+            // See if we're already at the top level of the filesystem
+            if ((parentPath.empty()) && (!lastElement.empty()))
+            {
+                // Search for the first path element (ignoring case) in
+                // the top-level directory
+                realDirName = findFileInDirectory(lastElement, "/",
+                                                  CASE_INSENSITIVE);
+      
+                dc = osgDB::getDirectoryContents(realDirName);
+                needFollowingBackslash = true;
+                needDirectoryName = true;
+            }
+            else
+            {
+                // Recursively search for the last path element (ignoring case)
+                // in the parent path
+                realDirName = findFileInDirectory(lastElement, parentPath,
+                                                  CASE_INSENSITIVE);
+      
+                dc = osgDB::getDirectoryContents(realDirName);
+                char lastChar = realDirName[realDirName.size()-1];
+                if (lastChar=='/') needFollowingBackslash = false;
+                else if (lastChar=='\\') needFollowingBackslash = false;
+                else needFollowingBackslash = true;
+                needDirectoryName = true;
+            }
+        }
+        else
+        {
+            // No need for recursive search if we're doing an exact comparison
+            dc = osgDB::getDirectoryContents(realDirName);
+            char lastChar = realDirName[realDirName.size()-1];
+            if (lastChar=='/') needFollowingBackslash = false;
+            else if (lastChar=='\\') needFollowingBackslash = false;
+            else needFollowingBackslash = true;
+            needDirectoryName = true;
+        }
     }
 
     for(osgDB::DirectoryContents::iterator itr=dc.begin();
         itr!=dc.end();
         ++itr)
     {
-        if ((caseSensitivity==CASE_INSENSITIVE && osgDB::equalCaseInsensitive(fileName,*itr)) ||
-            (fileName==*itr))
+        if ((caseSensitivity==CASE_INSENSITIVE && osgDB::equalCaseInsensitive(realFileName,*itr)) ||
+            (realFileName==*itr))
         {
             if (!needDirectoryName) return *itr;
-            else if (needFollowingBackslash) return dirName+'/'+*itr;
-            else return dirName+*itr;
+            else if (needFollowingBackslash) return realDirName+'/'+*itr;
+            else return realDirName+*itr;
         }
     }
     return "";
 }
 
+static void appendInstallationLibraryFilePaths(osgDB::FilePathList& filepath)
+{
+#ifdef OSG_DEFAULT_LIBRARY_PATH
+
+    // Append the install prefix path to the library search path if configured
+    filepath.push_back(ADDQUOTES(OSG_DEFAULT_LIBRARY_PATH));
+#endif
+}
+
 #if defined(WIN32) && !defined(__CYGWIN__)
     #include <io.h>
     #include <direct.h>
@@ -372,15 +562,15 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
     {
         osgDB::DirectoryContents contents;
 
-        WIN32_FIND_DATA data;
-        HANDLE handle = FindFirstFile((dirName + "\\*").c_str(), &data);
+        OSGDB_WINDOWS_FUNCT(WIN32_FIND_DATA) data;
+        HANDLE handle = OSGDB_WINDOWS_FUNCT(FindFirstFile)((OSGDB_STRING_TO_FILENAME(dirName) + OSGDB_FILENAME_TEXT("\\*")).c_str(), &data);
         if (handle != INVALID_HANDLE_VALUE)
         {
             do
             {
-                contents.push_back(data.cFileName);
+                contents.push_back(OSGDB_FILENAME_TO_STRING(data.cFileName));
             }
-            while (FindNextFile(handle, &data) != 0);
+            while (OSGDB_WINDOWS_FUNCT(FindNextFile)(handle, &data) != 0);
 
             FindClose(handle);
         }
@@ -450,6 +640,8 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
             convertStringPathIntoFilePathList(ptr,filepath);
         }
         #endif
+
+        appendInstallationLibraryFilePaths(filepath);
     }
 
 
@@ -463,8 +655,9 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
             convertStringPathIntoFilePathList(ptr,filepath);
         }
 
-        convertStringPathIntoFilePathList("/usr/bin/:/usr/local/bin/",filepath);
+        appendInstallationLibraryFilePaths(filepath);
 
+        convertStringPathIntoFilePathList("/usr/bin/:/usr/local/bin/",filepath);
     }
     
 #elif defined(WIN32)
@@ -485,14 +678,14 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
         //   1. The directory from which the application loaded.
         DWORD retval = 0;
         const DWORD size = MAX_PATH;
-        char path[size];
-        retval = GetModuleFileName(NULL, path, size);
+        filenamechar path[size];
+        retval = OSGDB_WINDOWS_FUNCT(GetModuleFileName)(NULL, path, size);
         if (retval != 0 && retval < size)
         {
-            std::string pathstr(path);
-            std::string executableDir(pathstr, 0, 
-                                      pathstr.find_last_of("\\/"));
-            convertStringPathIntoFilePathList(executableDir, filepath);
+            filenamestring pathstr(path);
+            filenamestring executableDir(pathstr, 0, 
+                                      pathstr.find_last_of(OSGDB_FILENAME_TEXT("\\/")));
+            convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(executableDir), filepath);
         }
         else
         {
@@ -502,11 +695,12 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
 
         //   2. The system directory. Use the GetSystemDirectory function to 
         //      get the path of this directory.
-        char systemDir[(UINT)size];
-        retval = GetSystemDirectory(systemDir, (UINT)size);
+        filenamechar systemDir[(UINT)size];
+        retval = OSGDB_WINDOWS_FUNCT(GetSystemDirectory)(systemDir, (UINT)size);
+
         if (retval != 0 && retval < size)
         {
-            convertStringPathIntoFilePathList(std::string(systemDir), 
+            convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(systemDir), 
                                               filepath);
         }
         else
@@ -521,13 +715,13 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
         //      the path of this directory, but it is searched.
         //   4. The Windows directory. Use the GetWindowsDirectory function to 
         //      get the path of this directory.
-        char windowsDir[(UINT)size];
-        retval = GetWindowsDirectory(windowsDir, (UINT)size);
+        filenamechar windowsDir[(UINT)size];
+        retval = OSGDB_WINDOWS_FUNCT(GetWindowsDirectory)(windowsDir, (UINT)size);
         if (retval != 0 && retval < size)
         {
-            convertStringPathIntoFilePathList(std::string(windowsDir) + 
+            convertStringPathIntoFilePathList(std::string(OSGDB_FILENAME_TO_STRING(windowsDir)) + 
                                               "\\System", filepath);
-            convertStringPathIntoFilePathList(std::string(windowsDir), 
+            convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(windowsDir), 
                                               filepath);
         }
         else
@@ -545,15 +739,21 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
         //   6. The directories that are listed in the PATH environment 
         //      variable. Note that this does not include the per-application 
         //      path specified by the App Paths registry key.
-        char* ptr;
-        if ((ptr = getenv( "PATH" )))
+        filenamechar* ptr;
+#ifdef OSG_USE_UTF8_FILENAME
+        if ((ptr = _wgetenv(OSGDB_FILENAME_TEXT("PATH"))))
+#else
+        if ((ptr = getenv("PATH")))
+#endif
         {
             // Note that on any sane Windows system, some of the paths above
             // will also be on the PATH (the values gotten in systemDir and
             // windowsDir), but the DLL search goes sequentially and stops
             // when a DLL is found, so I see no point in removing duplicates.
-            convertStringPathIntoFilePathList(ptr, filepath);
+            convertStringPathIntoFilePathList(OSGDB_FILENAME_TO_STRING(ptr), filepath);
         }
+
+        appendInstallationLibraryFilePaths(filepath);
     }
     
 #elif defined(__APPLE__)
@@ -692,6 +892,8 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
             convertStringPathIntoFilePathList(ptr, filepath);
         }
 
+        appendInstallationLibraryFilePaths(filepath);
+
         // Since this is currently the only Objective-C code in the
         // library, we need an autoreleasepool for obj-c memory management.
         // If more Obj-C is added, we might move this pool to another 
@@ -762,6 +964,8 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
             convertStringPathIntoFilePathList(ptr, filepath);
         }
 
+        appendInstallationLibraryFilePaths(filepath);
+
         const std::string OSG_PLUGIN_PATH("/OpenSceneGraph/PlugIns");
         CFURLRef  url;
         CFBundleRef myBundle;
@@ -865,6 +1069,8 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
         {
             convertStringPathIntoFilePathList(ptr, filepath);
         }
+
+        appendInstallationLibraryFilePaths(filepath);
     }
     #endif
     
@@ -879,6 +1085,7 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
             convertStringPathIntoFilePathList(ptr,filepath);
         }
 
+        appendInstallationLibraryFilePaths(filepath);
 
 #if defined(__ia64__) || defined(__x86_64__)
         convertStringPathIntoFilePathList("/usr/lib/:/usr/lib64/:/usr/local/lib/:/usr/local/lib64/",filepath);
