@@ -87,9 +87,13 @@ osg::Object* Input::readObject(const std::string& fileName)
     return readObjectFile(fileName,_options.get());
 }
 
+osg::Shader*  Input::readShader()
+{
+    return Registry::instance()->readShader(*this);
+}
+
 osg::Image*  Input::readImage(const std::string& fileName)
 {
-
     return readImageFile(fileName,_options.get());
 }
 
@@ -98,6 +102,11 @@ osg::Node* Input::readNode(const std::string& fileName)
     return readNodeFile(fileName,_options.get());
 }
 
+osg::Shader*  Input::readShader(const std::string& fileName)
+{
+    return readShaderFile(fileName,_options.get());
+}
+
 bool Input::read(Parameter value1)
 {
     if (value1.valid((*this)[0].getStr()))
