@@ -183,6 +183,15 @@ Registry::Registry()
         else _buildKdTreesHint = ReaderWriter::Options::BUILD_KDTREES;
     }
 
+    const char* ptr=0;
+
+    _expiryDelay = 10.0;
+    if( (ptr = getenv("OSG_EXPIRY_DELAY")) != 0)
+    {
+        _expiryDelay = osg::asciiToDouble(ptr);
+        osg::notify(osg::INFO)<<"Registry : Expiry delay = "<<_expiryDelay<<std::endl;
+    }
+
     const char* fileCachePath = getenv("OSG_FILE_CACHE");
     if (fileCachePath)
     {
@@ -239,6 +248,19 @@ Registry::Registry()
     addFileExtensionAlias("vert",   "glsl");
     addFileExtensionAlias("frag",   "glsl");
 
+
+#if defined(DARWIN_IMAGEIO)
+    addFileExtensionAlias("jpg",  "imageio");
+    addFileExtensionAlias("jpe",  "imageio");
+    addFileExtensionAlias("jpeg", "imageio");
+    addFileExtensionAlias("tif",  "imageio");
+    addFileExtensionAlias("tiff", "imageio");
+    addFileExtensionAlias("gif",  "imageio");
+    addFileExtensionAlias("png",  "imageio");
+    addFileExtensionAlias("psd",  "imageio");
+    addFileExtensionAlias("tga",  "imageio");
+#endif
+
 #if defined(DARWIN_QUICKTIME)
     addFileExtensionAlias("jpg",  "qt");
     addFileExtensionAlias("jpe",  "qt");
@@ -1968,7 +1990,7 @@ osg::Object* Registry::getFromObjectCache(const std::string& fileName)
     else return 0;
 }
 
-void Registry::updateTimeStampOfObjectsInCacheWithExternalReferences(double currentTime)
+void Registry::updateTimeStampOfObjectsInCacheWithExternalReferences(const osg::FrameStamp& frameStamp)
 {
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_objectCacheMutex);
 
@@ -1981,13 +2003,15 @@ void Registry::updateTimeStampOfObjectsInCacheWithExternalReferences(double curr
         if (itr->second.first->referenceCount()>1)
         {
             // so update it time stamp.
-            itr->second.second = currentTime;
+            itr->second.second = frameStamp.getReferenceTime();
         }
     }
 }
 
-void Registry::removeExpiredObjectsInCache(double expiryTime)
+void Registry::removeExpiredObjectsInCache(const osg::FrameStamp& frameStamp)
 {
+    double expiryTime = frameStamp.getReferenceTime() - _expiryDelay;
+
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_objectCacheMutex);
 
     typedef std::vector<std::string> ObjectsToRemove;
