@@ -116,7 +116,7 @@ void ConnectedParticleSystem::drawImplementation(osg::RenderInfo& renderInfo) co
 {
     osg::State& state = *renderInfo.getState();
 
-    OpenThreads::ScopedReadLock lock(_readWriteMutex);
+    ScopedReadLock lock(_readWriteMutex);
 
     const Particle* particle = (_startParticle != Particle::INVALID_INDEX) ? &_particles[_startParticle] : 0;
     if (!particle) return;
