@@ -31,6 +31,10 @@
 #include <osgSim/MultiSwitch>
 #include <osgSim/GeographicLocation>
 #include <osgSim/ObjectRecordData>
+
+#include <osg/Notify>
+#include <osg/io_utils>
+
 #include "Registry.h"
 #include "Document.h"
 #include "RecordInputStream.h"
@@ -149,7 +153,7 @@ protected:
     }
 };
 
-RegisterRecordProxy<Header> g_Header(HEADER_OP);
+REGISTER_FLTRECORD(Header, HEADER_OP)
 
 
 /** Group
@@ -288,7 +292,8 @@ protected:
     }
 };
 
-RegisterRecordProxy<Group> g_Group(GROUP_OP);
+REGISTER_FLTRECORD(Group, GROUP_OP)
+
 
 
 /** DegreeOfFreedom
@@ -309,7 +314,7 @@ class DegreeOfFreedom : public PrimaryRecord
     struct Range
     {
         float64 min;            // Minimum value with respect to the local coord system
-        float64 max;            // Maximum value with respect to the local coordsystem
+        float64 max;            // Maximum value with respect to the local coord system
         float64 current;        // Current value with respect to the local coord system
         float64 increment;      // Increment
     };
@@ -385,9 +390,21 @@ protected:
         osg::Vec3 yAxis = zAxis ^ xAxis;
 
         // normalize
-        xAxis.normalize();
-        yAxis.normalize();
-        zAxis.normalize();
+        float length_x = xAxis.normalize();
+        float length_y = yAxis.normalize();
+        float length_z = zAxis.normalize();
+        
+        if ((length_x*length_y*length_z)==0.0f)
+        {
+            osg::notify(osg::NOTICE)<<"Warning: OpenFlight DegreeOfFreedom::readRecord() found erroneous axis definition:"<<std::endl;
+            osg::notify(osg::NOTICE)<<"    localOrigin="<<localOrigin<<std::endl;
+            osg::notify(osg::NOTICE)<<"    pointOnXAxis="<<pointOnXAxis<<std::endl;
+            osg::notify(osg::NOTICE)<<"    pointInXYPlane="<<pointInXYPlane<<std::endl;
+            
+            xAxis.set(1.0f,0.0f,0.0f);
+            yAxis.set(0.0f,1.0f,0.0f);
+            zAxis.set(0.0f,0.0f,1.0f);
+        }
 
         // scale origin
         osg::Vec3 origin = localOrigin * document.unitScale();
@@ -398,6 +415,7 @@ protected:
                                zAxis.x(), zAxis.y(), zAxis.z(), 0.0,
                                origin.x(), origin.y(), origin.z(), 1.0);
 
+
         _dof->setInversePutMatrix(inv_putmat);
         _dof->setPutMatrix(osg::Matrix::inverse(inv_putmat));
 
@@ -440,7 +458,8 @@ protected:
     }
 };
 
-RegisterRecordProxy<DegreeOfFreedom> g_DegreeOfFreedom(DOF_OP);
+REGISTER_FLTRECORD(DegreeOfFreedom, DOF_OP)
+
 
 
 /** LevelOfDetail - To recreate the LevelOfDetail record in OSG we have to create a LOD node with one Group node under it.
@@ -496,7 +515,8 @@ protected:
 
 };
 
-RegisterRecordProxy<LevelOfDetail> g_LevelOfDetail(LOD_OP);
+REGISTER_FLTRECORD(LevelOfDetail, LOD_OP)
+
 
 
 /** OldLevelOfDetail
@@ -552,7 +572,8 @@ protected:
 
 };
 
-RegisterRecordProxy<OldLevelOfDetail> g_OldLevelOfDetail(OLD_LOD_OP);
+REGISTER_FLTRECORD(OldLevelOfDetail, OLD_LOD_OP)
+
 
 
 /** Switch
@@ -628,7 +649,8 @@ protected:
     }
 };
 
-RegisterRecordProxy<Switch> g_Switch(SWITCH_OP);
+REGISTER_FLTRECORD(Switch, SWITCH_OP)
+
 
 
 /** ExternalReference
@@ -675,12 +697,12 @@ protected:
         {
             in.forward(4);
 
-            uint32 mask = in.readUInt32(~0);
+            uint32 mask = in.readUInt32(~0u);
 
             // Possible bug in models with version number 15.4.1 ?
             // Symptoms: Black trees in VegaPlayer town.
             if (document.version() == 1541)
-                mask = ~0;
+                mask = ~0u;
 
             ParentPools* parentPools = new ParentPools;
 
@@ -712,7 +734,8 @@ protected:
 
 };
 
-RegisterRecordProxy<ExternalReference> g_ExternalReference(EXTERNAL_REFERENCE_OP);
+REGISTER_FLTRECORD(ExternalReference, EXTERNAL_REFERENCE_OP)
+
 
 
 /** InstanceDefinition
@@ -761,7 +784,8 @@ protected:
 
 };
 
-RegisterRecordProxy<InstanceDefinition> g_InstanceDefinition(INSTANCE_DEFINITION_OP);
+REGISTER_FLTRECORD(InstanceDefinition, INSTANCE_DEFINITION_OP)
+
 
 
 /** InstanceReference
@@ -792,7 +816,8 @@ protected:
     }
 };
 
-RegisterRecordProxy<InstanceReference> g_InstanceReference(INSTANCE_REFERENCE_OP);
+REGISTER_FLTRECORD(InstanceReference, INSTANCE_REFERENCE_OP)
+
 
 
 /** Extension
@@ -831,7 +856,8 @@ protected:
     }
 };
 
-RegisterRecordProxy<Extension> g_Extension(EXTENSION_OP);
+REGISTER_FLTRECORD(Extension, EXTENSION_OP)
+
 
 
 /** Object
@@ -935,7 +961,8 @@ protected:
 
 };
 
-RegisterRecordProxy<Object> g_Object(OBJECT_OP);
+REGISTER_FLTRECORD(Object, OBJECT_OP)
+
 
 
 /** LightSource
@@ -1018,7 +1045,8 @@ protected:
     }
 };
 
-RegisterRecordProxy<LightSource> g_LightSource(LIGHT_SOURCE_OP);
+REGISTER_FLTRECORD(LightSource, LIGHT_SOURCE_OP)
+
 
 
 } // end namespace
