@@ -87,8 +87,8 @@ FltExportVisitor::writeHeader( const std::string& headerName )
     }
 
     static const unsigned int SAVE_VERTEX_NORMALS_BIT = 0x80000000u >> 0;
-    static const unsigned int PACKED_COLOR_MODE_BIT   = 0x80000000u >> 1;
-    static const unsigned int CAD_VIEW_MODE_BIT       = 0x80000000u >> 2;
+    //static const unsigned int PACKED_COLOR_MODE_BIT   = 0x80000000u >> 1;
+    //static const unsigned int CAD_VIEW_MODE_BIT       = 0x80000000u >> 2;
     uint32 flags( SAVE_VERTEX_NORMALS_BIT );
 
     IdHelper id(*this, headerName);
@@ -420,7 +420,8 @@ FltExportVisitor::writeExternalReference( const osg::ProxyNode& proxy )
                           SHADER_PALETTE_OVERRIDE ;
 
     // Selectively turn off overrides for resources we don't need
-    const ParentPools* pp = static_cast<const ParentPools*>(proxy.getUserData() );
+    const ParentPools* pp = dynamic_cast<const ParentPools*>(proxy.getUserData() );
+    
     if (pp && pp->getColorPool() )
       flags &= ~COLOR_PALETTE_OVERRIDE;
 
@@ -661,7 +662,7 @@ FltExportVisitor::writeLightPoint( const osgSim::LightPointNode* lpn )
         return;
 
     // In OSG, each LightPoint within a LightPointNode  can have different appearance
-    // parameters, but in OpenFlight, a Light Point Record contains a list of homogenous
+    // parameters, but in OpenFlight, a Light Point Record contains a list of homogeneous
     // vertices. To be correct, we'd have to look at all LightPoints in the LightPointNode
     // and spew out multiple FLT records for each group that shared common appearance
     // parameters. Instead, we cheat: We take the first LightPoint and use its appearance
@@ -680,7 +681,7 @@ FltExportVisitor::writeLightPoint( const osgSim::LightPointNode* lpn )
         animPhaseDelay = lp0._blinkSequence->getPhaseShift();
     }
 
-    // Note that true bidirectional light points are currently unsupported (they are inavailable
+    // Note that true bidirectional light points are currently unsupported (they are unavailable
     // in OSG, so we never write them out to FLT as BIDIRECTIONAL.
     int32 directionality( OMNIDIRECTIONAL );
     float32 horizLobe( 360.f );
@@ -707,7 +708,7 @@ FltExportVisitor::writeLightPoint( const osgSim::LightPointNode* lpn )
         _records->writeID( id );
         _records->writeInt16( 0 ); // Surface material code
         _records->writeInt16( 0 ); // Feature ID
-        _records->writeUInt32( -1 ); // Back color for bidirectional
+        _records->writeUInt32( ~0u ); // OpenFlight erronously say -1, so will assume ~0u is OK.  Back color for bidirectional
         _records->writeInt32( EITHER ); // Display mode
         _records->writeFloat32( lp0._intensity ); // Intensity
         _records->writeFloat32( 0.f ); // Back intensity TBD
@@ -734,7 +735,7 @@ FltExportVisitor::writeLightPoint( const osgSim::LightPointNode* lpn )
         _records->writeFloat32( 0.f ); // Directional falloff exponent
         _records->writeFloat32( 0.f ); // Directional ambient intensity
         _records->writeFloat32( animPeriod ); // Animation period in seconds
-        _records->writeFloat32( animPhaseDelay ); // Animation phase delay in secnds
+        _records->writeFloat32( animPhaseDelay ); // Animation phase delay in seconds
         _records->writeFloat32( animEnabled ); // Animation enabled period in seconds
         _records->writeFloat32( 1.f ); // Significance
         _records->writeInt32( 0 ); // Calligraphic draw order
