@@ -33,9 +33,9 @@ using namespace osgdae;
 void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, const std::string &geoName )
 {
     osg::ref_ptr<osg::StateSet> ssClean = CleanStateSet(ss); // Need to hold a ref to this or the materialMap.find() will delete it
-    domBind_material *bm = daeSafeCast< domBind_material >( ig->createAndPlace( COLLADA_ELEMENT_BIND_MATERIAL ) );
-    domBind_material::domTechnique_common *tc = daeSafeCast< domBind_material::domTechnique_common >( bm->createAndPlace( "technique_common" ) );
-    domInstance_material *im = daeSafeCast< domInstance_material >( tc->createAndPlace( COLLADA_ELEMENT_INSTANCE_MATERIAL ) );
+    domBind_material *bm = daeSafeCast< domBind_material >( ig->add( COLLADA_ELEMENT_BIND_MATERIAL ) );
+    domBind_material::domTechnique_common *tc = daeSafeCast< domBind_material::domTechnique_common >( bm->add( "technique_common" ) );
+    domInstance_material *im = daeSafeCast< domInstance_material >( tc->add( COLLADA_ELEMENT_INSTANCE_MATERIAL ) );
     std::string symbol = geoName + "_material";
     im->setSymbol( symbol.c_str() );
 
@@ -49,10 +49,10 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
 
     if ( lib_mats == NULL )
     {
-        lib_mats = daeSafeCast< domLibrary_materials >( dom->createAndPlace( COLLADA_ELEMENT_LIBRARY_MATERIALS ) );
+        lib_mats = daeSafeCast< domLibrary_materials >( dom->add( COLLADA_ELEMENT_LIBRARY_MATERIALS ) );
     }
 
-    domMaterial *mat = daeSafeCast< domMaterial >( lib_mats->createAndPlace( COLLADA_ELEMENT_MATERIAL ) );
+    domMaterial *mat = daeSafeCast< domMaterial >( lib_mats->add( COLLADA_ELEMENT_MATERIAL ) );
     std::string name = ssClean->getName();
     if ( name.empty() )
     {
@@ -65,13 +65,13 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
     std::string url = "#" + name;
     im->setTarget( url.c_str() );
 
-    domInstance_effect *ie = daeSafeCast<domInstance_effect>( mat->createAndPlace( "instance_effect" ) );
+    domInstance_effect *ie = daeSafeCast<domInstance_effect>( mat->add( "instance_effect" ) );
 
     if ( lib_effects == NULL )
     {
-        lib_effects = daeSafeCast< domLibrary_effects >( dom->createAndPlace( COLLADA_ELEMENT_LIBRARY_EFFECTS ) );
+        lib_effects = daeSafeCast< domLibrary_effects >( dom->add( COLLADA_ELEMENT_LIBRARY_EFFECTS ) );
     }
-    domEffect *effect = daeSafeCast< domEffect >( lib_effects->createAndPlace( COLLADA_ELEMENT_EFFECT ) );
+    domEffect *effect = daeSafeCast< domEffect >( lib_effects->add( COLLADA_ELEMENT_EFFECT ) );
     std::string efName = name + "_effect";
     
     effect->setId( efName.c_str() );
@@ -79,10 +79,10 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
     url = "#" + efName;
     ie->setUrl( url.c_str() );
 
-    domProfile_COMMON *pc = daeSafeCast< domProfile_COMMON >( effect->createAndPlace( COLLADA_ELEMENT_PROFILE_COMMON ) );
-    domProfile_COMMON::domTechnique *pc_teq = daeSafeCast< domProfile_COMMON::domTechnique >( pc->createAndPlace( "technique" ) );
+    domProfile_COMMON *pc = daeSafeCast< domProfile_COMMON >( effect->add( COLLADA_ELEMENT_PROFILE_COMMON ) );
+    domProfile_COMMON::domTechnique *pc_teq = daeSafeCast< domProfile_COMMON::domTechnique >( pc->add( "technique" ) );
     pc_teq->setSid( "t0" );
-    domProfile_COMMON::domTechnique::domPhong *phong = daeSafeCast< domProfile_COMMON::domTechnique::domPhong >( pc_teq->createAndPlace( "phong" ) );
+    domProfile_COMMON::domTechnique::domPhong *phong = daeSafeCast< domProfile_COMMON::domTechnique::domPhong >( pc_teq->add( "phong" ) );
 
     osg::Texture *tex = static_cast<osg::Texture*>(ssClean->getTextureAttribute( 0, osg::StateAttribute::TEXTURE ));
     if ( ssClean->getTextureAttribute( 1, osg::StateAttribute::TEXTURE ) != NULL )
@@ -92,44 +92,44 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
     if ( tex != NULL && tex->getImage( 0 ) != NULL )
     {
         //TODO: Export all of the texture Attributes like wrap mode and all that jazz
-        domImage *img = daeSafeCast< domImage >( pc->createAndPlace( COLLADA_ELEMENT_IMAGE ) );
+        domImage *img = daeSafeCast< domImage >( pc->add( COLLADA_ELEMENT_IMAGE ) );
         std::string iName = efName + "-image";
         img->setId( iName.c_str() );
 
         osg::Image *osgimg = tex->getImage( 0 );
-        domImage::domInit_from *imgif = daeSafeCast< domImage::domInit_from >( img->createAndPlace( "init_from" ) );
+        domImage::domInit_from *imgif = daeSafeCast< domImage::domInit_from >( img->add( "init_from" ) );
         std::string fileURI = ReaderWriterDAE::ConvertFilePathToColladaCompatibleURI(osgDB::findDataFile(osgimg->getFileName()));
-        daeURI dd( fileURI.c_str() );
+       daeURI dd(*dae, fileURI);//fileURI.c_str() );
         imgif->setValue( dd );
         // The document URI should contain the canonical path it was created with
         imgif->getValue().makeRelativeTo(doc->getDocumentURI());
 
 #ifndef EARTH_TEX
-        domCommon_newparam_type *np = daeSafeCast< domCommon_newparam_type >( pc->createAndPlace( "newparam" ) );
+        domCommon_newparam_type *np = daeSafeCast< domCommon_newparam_type >( pc->add( "newparam" ) );
         std::string surfName = efName + "-surface";
         np->setSid( surfName.c_str() );
-        domFx_surface_common *surface = daeSafeCast< domFx_surface_common >( np->createAndPlace( "surface" ) );
-        domFx_surface_init_from_common *sif = daeSafeCast< domFx_surface_init_from_common >( surface->createAndPlace("init_from") );
+        domFx_surface_common *surface = daeSafeCast< domFx_surface_common >( np->add( "surface" ) );
+        domFx_surface_init_from_common *sif = daeSafeCast< domFx_surface_init_from_common >( surface->add("init_from") );
         sif->setValue( iName.c_str() );
         surface->setType( FX_SURFACE_TYPE_ENUM_2D );
 
-        np = daeSafeCast< domCommon_newparam_type >( pc->createAndPlace( "newparam" ) );
+        np = daeSafeCast< domCommon_newparam_type >( pc->add( "newparam" ) );
         std::string sampName = efName + "-sampler";
         np->setSid( sampName.c_str() );
-        domFx_sampler2D_common *sampler = daeSafeCast< domFx_sampler2D_common >( np->createAndPlace( "sampler2D" ) );
-        domFx_sampler2D_common_complexType::domSource *source = daeSafeCast< domFx_sampler2D_common_complexType::domSource >( sampler->createAndPlace( "source" ) );
+        domFx_sampler2D_common *sampler = daeSafeCast< domFx_sampler2D_common >( np->add( "sampler2D" ) );
+        domFx_sampler2D_common_complexType::domSource *source = daeSafeCast< domFx_sampler2D_common_complexType::domSource >( sampler->add( "source" ) );
         source->setValue( surfName.c_str() ); 
 
         //set sampler state
-        domFx_sampler2D_common_complexType::domWrap_s *wrap_s = daeSafeCast< domFx_sampler2D_common_complexType::domWrap_s >( sampler->createAndPlace( "wrap_s" ) );
+        domFx_sampler2D_common_complexType::domWrap_s *wrap_s = daeSafeCast< domFx_sampler2D_common_complexType::domWrap_s >( sampler->add( "wrap_s" ) );
         osg::Texture::WrapMode wrap = tex->getWrap( osg::Texture::WRAP_S );
         switch( wrap ) 
         {
         case osg::Texture::CLAMP:
+        case osg::Texture::CLAMP_TO_EDGE:
             wrap_s->setValue( FX_SAMPLER_WRAP_COMMON_CLAMP );
             break;
         case osg::Texture::CLAMP_TO_BORDER:
-        case osg::Texture::CLAMP_TO_EDGE:
             wrap_s->setValue( FX_SAMPLER_WRAP_COMMON_BORDER );
             break;
         case osg::Texture::REPEAT:
@@ -143,15 +143,15 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
             break;
         }
 
-        domFx_sampler2D_common_complexType::domWrap_t *wrap_t = daeSafeCast< domFx_sampler2D_common_complexType::domWrap_t >( sampler->createAndPlace( "wrap_t" ) );
+        domFx_sampler2D_common_complexType::domWrap_t *wrap_t = daeSafeCast< domFx_sampler2D_common_complexType::domWrap_t >( sampler->add( "wrap_t" ) );
         wrap = tex->getWrap( osg::Texture::WRAP_T );
         switch( wrap ) 
         {
         case osg::Texture::CLAMP:
+        case osg::Texture::CLAMP_TO_EDGE:
             wrap_t->setValue( FX_SAMPLER_WRAP_COMMON_CLAMP );
             break;
         case osg::Texture::CLAMP_TO_BORDER:
-        case osg::Texture::CLAMP_TO_EDGE:
             wrap_t->setValue( FX_SAMPLER_WRAP_COMMON_BORDER );
             break;
         case osg::Texture::REPEAT:
@@ -166,13 +166,13 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
         }
 
         const osg::Vec4 &bcol = tex->getBorderColor();
-        domFx_sampler2D_common_complexType::domBorder_color *dbcol = daeSafeCast< domFx_sampler2D_common_complexType::domBorder_color >( sampler->createAndPlace( "border_color" ) );
+        domFx_sampler2D_common_complexType::domBorder_color *dbcol = daeSafeCast< domFx_sampler2D_common_complexType::domBorder_color >( sampler->add( "border_color" ) );
         dbcol->getValue().append( bcol.r() );
         dbcol->getValue().append( bcol.g() );
         dbcol->getValue().append( bcol.b() );
         dbcol->getValue().append( bcol.a() );
 
-        domFx_sampler2D_common_complexType::domMinfilter *minfilter = daeSafeCast< domFx_sampler2D_common_complexType::domMinfilter >( sampler->createAndPlace( "minfilter" ) );
+        domFx_sampler2D_common_complexType::domMinfilter *minfilter = daeSafeCast< domFx_sampler2D_common_complexType::domMinfilter >( sampler->add( "minfilter" ) );
         osg::Texture::FilterMode mode = tex->getFilter( osg::Texture::MIN_FILTER );
         switch( mode )
         {
@@ -196,7 +196,7 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
             break;
         }
 
-        domFx_sampler2D_common_complexType::domMagfilter *magfilter = daeSafeCast< domFx_sampler2D_common_complexType::domMagfilter >( sampler->createAndPlace( "magfilter" ) );
+        domFx_sampler2D_common_complexType::domMagfilter *magfilter = daeSafeCast< domFx_sampler2D_common_complexType::domMagfilter >( sampler->add( "magfilter" ) );
         mode = tex->getFilter( osg::Texture::MAG_FILTER );
         switch( mode )
         {
@@ -221,49 +221,21 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
         }
 
 
-        domCommon_color_or_texture_type *cot = daeSafeCast< domCommon_color_or_texture_type >( phong->createAndPlace( "diffuse" ) );
-        domCommon_color_or_texture_type_complexType::domTexture *dtex = daeSafeCast< domCommon_color_or_texture_type_complexType::domTexture >( cot->createAndPlace( "texture" ) );
+        domCommon_color_or_texture_type *cot = daeSafeCast< domCommon_color_or_texture_type >( phong->add( "diffuse" ) );
+        domCommon_color_or_texture_type_complexType::domTexture *dtex = daeSafeCast< domCommon_color_or_texture_type_complexType::domTexture >( cot->add( "texture" ) );
         dtex->setTexture( sampName.c_str() );
         dtex->setTexcoord( "texcoord0" );
 #else
-        domCommon_color_or_texture_type *cot = daeSafeCast< domCommon_color_or_texture_type >( phong->createAndPlace( "diffuse" ) );
-        domCommon_color_or_texture_type_complexType::domTexture *dtex = daeSafeCast< domCommon_color_or_texture_type_complexType::domTexture >( cot->createAndPlace( "texture" ) );
+        domCommon_color_or_texture_type *cot = daeSafeCast< domCommon_color_or_texture_type >( phong->add( "diffuse" ) );
+        domCommon_color_or_texture_type_complexType::domTexture *dtex = daeSafeCast< domCommon_color_or_texture_type_complexType::domTexture >( cot->add( "texture" ) );
         dtex->setTexture( iName.c_str() );
         dtex->setTexcoord( "texcoord0" );
 #endif
 
-        domInstance_material::domBind_vertex_input *bvi = daeSafeCast< domInstance_material::domBind_vertex_input >( im->createAndPlace( "bind_vertex_input" ) );
+        domInstance_material::domBind_vertex_input *bvi = daeSafeCast< domInstance_material::domBind_vertex_input >( im->add( "bind_vertex_input" ) );
         bvi->setSemantic( "texcoord0" );
         bvi->setInput_semantic( "TEXCOORD" );
         bvi->setInput_set( 0 );
-
-/*  I dont think this belongs here RFJ
-        //take care of blending if any
-        osg::BlendFunc *bf = static_cast< osg::BlendFunc * >( ssClean->getAttribute( osg::StateAttribute::BLENDFUNC ) );
-        osg::BlendColor *bc = static_cast< osg::BlendColor * >( ssClean->getAttribute( osg::StateAttribute::BLENDCOLOR ) );
-        if ( bc != NULL )
-        {
-            domCommon_transparent_type *ctt = daeSafeCast< domCommon_transparent_type >( phong->createAndPlace( "transparent" ) );
-            ctt->setOpaque( FX_OPAQUE_ENUM_RGB_ZERO );
-            domCommon_color_or_texture_type_complexType::domColor *tcol = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( ctt->createAndPlace( "color" ) );
-            tcol->getValue().append( bc->getConstantColor().r() );
-            tcol->getValue().append( bc->getConstantColor().r() );
-            tcol->getValue().append( bc->getConstantColor().r() );
-            tcol->getValue().append( bc->getConstantColor().r() );
-        }
-        else if ( bf != NULL )
-        {
-            domCommon_transparent_type *ctt = daeSafeCast< domCommon_transparent_type >( phong->createAndPlace( "transparent" ) );
-            ctt->setOpaque( FX_OPAQUE_ENUM_A_ONE );
-            dtex = daeSafeCast< domCommon_color_or_texture_type_complexType::domTexture >( ctt->createAndPlace( "texture" ) );
-#ifndef EARTH_TEX
-            dtex->setTexture( sampName.c_str() );
-#else
-            dtex->setTexture( iName.c_str() );
-#endif
-            dtex->setTexcoord( "texcoord0" );           
-        }
-*/       
     }
     osg::Material *osgmat = static_cast<osg::Material*>(ssClean->getAttribute( osg::StateAttribute::MATERIAL ));
     if ( osgmat != NULL )
@@ -274,15 +246,15 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
         const osg::Vec4 &sCol = osgmat->getSpecularFrontAndBack()?osgmat->getSpecular( osg::Material::FRONT_AND_BACK ):osgmat->getSpecular( osg::Material::FRONT );
         float shininess = osgmat->getShininessFrontAndBack()?osgmat->getShininess( osg::Material::FRONT_AND_BACK ):osgmat->getShininess( osg::Material::FRONT );
         
-        domCommon_color_or_texture_type *cot = daeSafeCast< domCommon_color_or_texture_type >( phong->createAndPlace( "emission" ) );
-        domCommon_color_or_texture_type_complexType::domColor *col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->createAndPlace( "color" ) );
+        domCommon_color_or_texture_type *cot = daeSafeCast< domCommon_color_or_texture_type >( phong->add( "emission" ) );
+        domCommon_color_or_texture_type_complexType::domColor *col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->add( "color" ) );
         col->getValue().append( eCol.r() );
         col->getValue().append( eCol.g() );
         col->getValue().append( eCol.b() );
         col->getValue().append( eCol.a() );
 
-        cot = daeSafeCast< domCommon_color_or_texture_type >( phong->createAndPlace( "ambient" ) );
-        col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->createAndPlace( "color" ) );
+        cot = daeSafeCast< domCommon_color_or_texture_type >( phong->add( "ambient" ) );
+        col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->add( "color" ) );
         col->getValue().append( aCol.r() );
         col->getValue().append( aCol.g() );
         col->getValue().append( aCol.b() );
@@ -292,8 +264,8 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
         //### check if we really have a texture
         if ( phong->getDiffuse() == NULL )
         {
-            cot = daeSafeCast< domCommon_color_or_texture_type >( phong->createAndPlace( "diffuse" ) );
-            col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->createAndPlace( "color" ) );
+            cot = daeSafeCast< domCommon_color_or_texture_type >( phong->add( "diffuse" ) );
+            col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->add( "color" ) );
             col->getValue().append( dCol.r() );
             col->getValue().append( dCol.g() );
             col->getValue().append( dCol.b() );
@@ -303,27 +275,38 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
         {
             cot = phong->getDiffuse();
             
-            domCommon_color_or_texture_type_complexType::domTexture *dtex = cot->getTexture();
-            domExtra *extra = daeSafeCast< domExtra >( dtex->createAndPlace( COLLADA_ELEMENT_EXTRA ) );
-            extra->setType( "color" );
-            domTechnique *teq = daeSafeCast< domTechnique >( extra->createAndPlace( COLLADA_ELEMENT_TECHNIQUE ) );
-            teq->setProfile( "SCEI" );
-            domAny *any = (domAny*)(daeElement*)teq->createAndPlace( "color" );
+            if (writeExtras)
+            {
+                // Adds the following to a texture element
 
-            std::ostringstream colVal;
-            colVal << dCol.r() << " " << dCol.g() << " " << dCol.b() << " " << dCol.a();
-            any->setValue( colVal.str().c_str() );
+                //<extra type="color">
+                //    <technique profile="SCEI">
+                //        <color>1.0 1.0 1.0 1.0</color>
+                //    </technique>
+                //</extra>
+
+                domCommon_color_or_texture_type_complexType::domTexture *dtex = cot->getTexture();
+                domExtra *extra = daeSafeCast< domExtra >( dtex->add( COLLADA_ELEMENT_EXTRA ) );
+                extra->setType( "color" );
+                domTechnique *teq = daeSafeCast< domTechnique >( extra->add( COLLADA_ELEMENT_TECHNIQUE ) );
+                teq->setProfile( "SCEI" );
+                domAny *any = (domAny*)(daeElement*)teq->add( "color" );
+
+                std::ostringstream colVal;
+                colVal << dCol.r() << " " << dCol.g() << " " << dCol.b() << " " << dCol.a();
+                any->setValue( colVal.str().c_str() );
+            }
         }
 
-        cot = daeSafeCast< domCommon_color_or_texture_type >( phong->createAndPlace( "specular" ) );
-        col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->createAndPlace( "color" ) );
+        cot = daeSafeCast< domCommon_color_or_texture_type >( phong->add( "specular" ) );
+        col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->add( "color" ) );
         col->getValue().append( sCol.r() );
         col->getValue().append( sCol.g() );
         col->getValue().append( sCol.b() );
         col->getValue().append( sCol.a() );
 
-        domCommon_float_or_param_type *fop = daeSafeCast< domCommon_float_or_param_type >( phong->createAndPlace( "shininess" ) );
-        domCommon_float_or_param_type_complexType::domFloat *f = daeSafeCast< domCommon_float_or_param_type_complexType::domFloat >( fop->createAndPlace( "float" ) );
+        domCommon_float_or_param_type *fop = daeSafeCast< domCommon_float_or_param_type >( phong->add( "shininess" ) );
+        domCommon_float_or_param_type_complexType::domFloat *f = daeSafeCast< domCommon_float_or_param_type_complexType::domFloat >( fop->add( "float" ) );
         f->setValue( shininess );
     }
 
@@ -333,65 +316,100 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
     {
         osg::BlendFunc *pBlendFunc = static_cast< osg::BlendFunc * >( ssClean->getAttribute( osg::StateAttribute::BLENDFUNC ) );
         osg::BlendColor *pBlendColor = static_cast< osg::BlendColor * >( ssClean->getAttribute( osg::StateAttribute::BLENDCOLOR ) );
-        if ((NULL != pBlendFunc) && (NULL != pBlendColor))
+        if (pBlendFunc != NULL)
         {
-            if ((GL_CONSTANT_ALPHA == pBlendFunc->getSource()) && (GL_ONE_MINUS_CONSTANT_ALPHA == pBlendFunc->getDestination()))
+            if (pBlendColor != NULL)
             {
-                // A_ONE opaque mode
-                domCommon_transparent_type *pTransparent = daeSafeCast<domCommon_transparent_type>(phong->createAndPlace("transparent"));
-                pTransparent->setOpaque(FX_OPAQUE_ENUM_A_ONE);
-                domCommon_color_or_texture_type_complexType::domColor *pColor = daeSafeCast<domCommon_color_or_texture_type_complexType::domColor>(pTransparent->createAndPlace("color"));
-                domCommon_float_or_param_type *pFop = daeSafeCast<domCommon_float_or_param_type>(phong->createAndPlace( "transparency"));
-                domCommon_float_or_param_type_complexType::domFloat *pTransparency = daeSafeCast<domCommon_float_or_param_type_complexType::domFloat>(pFop->createAndPlace("float"));
-                if (m_GoogleMode)
+                if ((GL_CONSTANT_ALPHA == pBlendFunc->getSource()) && (GL_ONE_MINUS_CONSTANT_ALPHA == pBlendFunc->getDestination()))
                 {
-                    pColor->getValue().append(1.0);
-                    pColor->getValue().append(1.0);
-                    pColor->getValue().append(1.0);
-                    pColor->getValue().append(1.0);
-                    pTransparency->setValue(1.0 - pBlendColor->getConstantColor().a());
+                    // A_ONE opaque mode
+                    domCommon_transparent_type *pTransparent = daeSafeCast<domCommon_transparent_type>(phong->add("transparent"));
+                    pTransparent->setOpaque(FX_OPAQUE_ENUM_A_ONE);
+                    domCommon_color_or_texture_type_complexType::domColor *pColor = daeSafeCast<domCommon_color_or_texture_type_complexType::domColor>(pTransparent->add("color"));
+                    domCommon_float_or_param_type *pFop = daeSafeCast<domCommon_float_or_param_type>(phong->add( "transparency"));
+                    domCommon_float_or_param_type_complexType::domFloat *pTransparency = daeSafeCast<domCommon_float_or_param_type_complexType::domFloat>(pFop->add("float"));
+                    if (m_GoogleMode)
+                    {
+                        pColor->getValue().append(1.0);
+                        pColor->getValue().append(1.0);
+                        pColor->getValue().append(1.0);
+                        pColor->getValue().append(1.0);
+                        pTransparency->setValue(1.0 - pBlendColor->getConstantColor().a());
+                    }
+                    else
+                    {
+                        pColor->getValue().append(pBlendColor->getConstantColor().r());
+                        pColor->getValue().append(pBlendColor->getConstantColor().g());
+                        pColor->getValue().append(pBlendColor->getConstantColor().b());
+                        pColor->getValue().append(pBlendColor->getConstantColor().a());
+                        pTransparency->setValue(1.0);
+                    }
                 }
-                else
+                else if ((GL_ONE_MINUS_CONSTANT_COLOR == pBlendFunc->getSource()) && (GL_CONSTANT_COLOR == pBlendFunc->getDestination()))
                 {
+                    // RGB_ZERO opaque mode
+                    domCommon_transparent_type *pTransparent = daeSafeCast<domCommon_transparent_type>(phong->add("transparent"));
+                    pTransparent->setOpaque(FX_OPAQUE_ENUM_RGB_ZERO);
+                    domCommon_color_or_texture_type_complexType::domColor *pColor = daeSafeCast<domCommon_color_or_texture_type_complexType::domColor>(pTransparent->add("color"));
                     pColor->getValue().append(pBlendColor->getConstantColor().r());
                     pColor->getValue().append(pBlendColor->getConstantColor().g());
                     pColor->getValue().append(pBlendColor->getConstantColor().b());
                     pColor->getValue().append(pBlendColor->getConstantColor().a());
+                    domCommon_float_or_param_type *pFop = daeSafeCast<domCommon_float_or_param_type>(phong->add( "transparency"));
+                    domCommon_float_or_param_type_complexType::domFloat *pTransparency = daeSafeCast<domCommon_float_or_param_type_complexType::domFloat>(pFop->add("float"));
                     pTransparency->setValue(1.0);
                 }
+                else
+                    osg::notify( osg::WARN ) << "Unsupported BlendFunction parameters in transparency processing." << std::endl;
             }
-            else if ((GL_ONE_MINUS_CONSTANT_COLOR == pBlendFunc->getSource()) && (GL_CONSTANT_COLOR == pBlendFunc->getDestination()))
+            else if (tex != NULL && tex->getImage( 0 ) != NULL)
             {
-                // RGB_ZERO opaque mode
-                domCommon_transparent_type *pTransparent = daeSafeCast<domCommon_transparent_type>(phong->createAndPlace("transparent"));
-                pTransparent->setOpaque(FX_OPAQUE_ENUM_RGB_ZERO);
-                domCommon_color_or_texture_type_complexType::domColor *pColor = daeSafeCast<domCommon_color_or_texture_type_complexType::domColor>(pTransparent->createAndPlace("color"));
-                pColor->getValue().append(pBlendColor->getConstantColor().r());
-                pColor->getValue().append(pBlendColor->getConstantColor().g());
-                pColor->getValue().append(pBlendColor->getConstantColor().b());
-                pColor->getValue().append(pBlendColor->getConstantColor().a());
-                domCommon_float_or_param_type *pFop = daeSafeCast<domCommon_float_or_param_type>(phong->createAndPlace( "transparency"));
-                domCommon_float_or_param_type_complexType::domFloat *pTransparency = daeSafeCast<domCommon_float_or_param_type_complexType::domFloat>(pFop->createAndPlace("float"));
-                pTransparency->setValue(1.0);
+                domCommon_transparent_type *ctt = daeSafeCast< domCommon_transparent_type >( phong->add( "transparent" ) );
+                ctt->setOpaque( FX_OPAQUE_ENUM_A_ONE );
+                domCommon_color_or_texture_type_complexType::domTexture * dtex = daeSafeCast< domCommon_color_or_texture_type_complexType::domTexture >( ctt->add( "texture" ) );
+                
+    #ifndef EARTH_TEX
+                std::string sampName = efName + "-sampler";
+                dtex->setTexture( sampName.c_str() );
+    #else
+                std::string iName = efName + "-image";
+                dtex->setTexture( iName.c_str() );
+    #endif
+                dtex->setTexcoord( "texcoord0" );           
             }
             else
-                osg::notify( osg::WARN ) << "Unsupported BlendFunction parameters in transparency processing." << std::endl;
+            {
+                osg::notify( osg::WARN ) << "Transparency processing - No texture or BlendColor." << std::endl;
+            }
         }
         else
-            osg::notify( osg::WARN ) << "Transparency processing - BlendFunction or BlendColor not found." << std::endl;
+        {
+            osg::notify( osg::WARN ) << "Transparency processing - BlendFunction not found." << std::endl;
+        }
     }
 
-    // Process GOOGLE one sided stuff here
-    if (osg::StateAttribute::INHERIT != ssClean->getMode(GL_CULL_FACE))
+    if (writeExtras)
     {
-        domExtra *pExtra = daeSafeCast<domExtra>(pc->createAndPlace(COLLADA_ELEMENT_EXTRA));
-        domTechnique *pTechnique = daeSafeCast<domTechnique>(pExtra->createAndPlace( COLLADA_ELEMENT_TECHNIQUE ) );
-        pTechnique->setProfile("GOOGLEEARTH");
-        domAny *pAny = (domAny*)(daeElement*)pTechnique->createAndPlace("double_sided");
-        if (GL_FALSE == ssClean->getMode(GL_CULL_FACE))
-            pAny->setValue("1");
-        else
-            pAny->setValue("0");
+        // Adds the following to a Profile_COMMON element
+
+        //<extra>
+        //    <technique profile="GOOGLEEARTH">
+        //        <double_sided>0</double_sided>
+        //    </technique>
+        //</extra>
+
+        // Process GOOGLE one sided stuff here
+        if (osg::StateAttribute::INHERIT != ssClean->getMode(GL_CULL_FACE))
+        {
+            domExtra *pExtra = daeSafeCast<domExtra>(pc->add(COLLADA_ELEMENT_EXTRA));
+            domTechnique *pTechnique = daeSafeCast<domTechnique>(pExtra->add( COLLADA_ELEMENT_TECHNIQUE ) );
+            pTechnique->setProfile("GOOGLEEARTH");
+            domAny *pAny = (domAny*)(daeElement*)pTechnique->add("double_sided");
+            if (GL_FALSE == ssClean->getMode(GL_CULL_FACE))
+                pAny->setValue("1");
+            else
+                pAny->setValue("0");
+        }
     }
 
     materialMap.insert( std::make_pair( ssClean, mat ) );
@@ -409,7 +427,7 @@ osg::StateSet* daeWriter::CleanStateSet(osg::StateSet* pStateSet) const
     if (NULL != pStateSet->getAttribute(osg::StateAttribute::MATERIAL))
         pCleanedStateSet->setAttribute(pStateSet->getAttribute(osg::StateAttribute::MATERIAL));
     // pCleanedStateSet->setRenderingHint(pStateSet->getRenderingHint()); does not work at the moment due to stateSet::merge
-    if (osg::StateAttribute::INHERIT != pStateSet->getMode(GL_CULL_FACE))
+    if (osg::StateAttribute::ON != pStateSet->getMode(GL_CULL_FACE))
         pCleanedStateSet->setMode(GL_CULL_FACE, pStateSet->getMode(GL_CULL_FACE));
     return pCleanedStateSet;
 }
