@@ -9,11 +9,19 @@
 *                    ARB_texture_compression.pdf
 *                    Author: Sebastien Domine, NVIDIA Corporation
 *
-*    CREATED BY:        Rune Schmidt Jensen, rsj@uni-dk
+*    CREATED BY:     Rune Schmidt Jensen, rsj@uni-dk
 *
 *    HISTORY:        Created   31.03.2003
 *             Modified  13.05.2004
 *                by George Tarantilis, gtaranti@nps.navy.mil
+*             Modified  22.05.2009 
+*                Wojtek Lewandowski, lewandowski@ai.com.pl 
+*
+*    WARNING: 
+*          Bit Masks in the WrtiteDDS are set for 8 bit components 
+*          write with 4 or 16 bit components will 
+*          probably produce corrupted file 
+*          Wojtek Lewandowski 2009-05-22 
 *
 **********************************************************************/
 
@@ -23,9 +31,11 @@
 #include <osgDB/Registry>
 #include <osgDB/FileNameUtils>
 #include <osgDB/FileUtils>
+#include <osgDB/fstream>
 
 #include <iomanip>
 #include <stdio.h>
+#include <string.h>
 
 
 // NOTICE ON WIN32:
@@ -596,11 +606,11 @@ osg::Image* ReadDDSFile(std::istream& _istream)
         float power2_s = logf((float)s)/logf((float)2);
         float power2_t = logf((float)t)/logf((float)2);
 
-        osg::notify(osg::INFO) << "ReadDDSFile info : ddsd.dwMipMapCount = "<<ddsd.dwMipMapCount<<std::endl;
-        osg::notify(osg::INFO) << "ReadDDSFile info : s = "<<s<<std::endl;
-        osg::notify(osg::INFO) << "ReadDDSFile info : t = "<<t<<std::endl;
-        osg::notify(osg::INFO) << "ReadDDSFile info : power2_s="<<power2_s<<std::endl;
-        osg::notify(osg::INFO) << "ReadDDSFile info : power2_t="<<power2_t<<std::endl;
+        osg::notify(osg::INFO) << "ReadDDSFile INFO : ddsd.dwMipMapCount = "<<ddsd.dwMipMapCount<<std::endl;
+        osg::notify(osg::INFO) << "ReadDDSFile INFO : s = "<<s<<std::endl;
+        osg::notify(osg::INFO) << "ReadDDSFile INFO : t = "<<t<<std::endl;
+        osg::notify(osg::INFO) << "ReadDDSFile INFO : power2_s="<<power2_s<<std::endl;
+        osg::notify(osg::INFO) << "ReadDDSFile INFO : power2_t="<<power2_t<<std::endl;
 
         mipmaps.resize((unsigned int)osg::maximum(power2_s,power2_t),0);
 
@@ -654,7 +664,7 @@ osg::Image* ReadDDSFile(std::istream& _istream)
     if (mipmaps.size()>0)  osgImage->setMipmapLevels(mipmaps);
     unsigned int size = osgImage->getTotalSizeInBytesIncludingMipmaps();
 
-    osg::notify(osg::INFO) << "ReadDDSFile info : size = " << size << std::endl;
+    osg::notify(osg::INFO) << "ReadDDSFile INFO : size = " << size << std::endl;
     
     if(size <= 0)
     {
@@ -714,10 +724,6 @@ printf("CORRECTO**> gtsibim:%d  grsib:%d   mi_size:%d   lPitch%d\n",
  */
 
 
-
-
-
-
 bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
 {
 
@@ -791,8 +797,8 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
         break;
     case GL_LUMINANCE_ALPHA:
         {
-            ddpf.dwRBitMask         = 0x00ff0000;
-            ddpf.dwRGBAlphaBitMask  = 0x000000ff;
+            ddpf.dwRBitMask         = 0x000000ff;
+            ddpf.dwRGBAlphaBitMask  = 0x0000ff00;
             PF_flags |= (DDPF_ALPHAPIXELS | DDPF_LUMINANCE);  
             ddpf.dwRGBBitCount = pixelSize; 
             ddsd.lPitch = img->getRowSizeInBytes();
@@ -812,7 +818,7 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
         break;
     case GL_LUMINANCE:
         {
-            ddpf.dwRBitMask         = 0x00ff0000;
+            ddpf.dwRBitMask         = 0x000000ff;
             PF_flags |= DDPF_LUMINANCE;
             ddpf.dwRGBBitCount = pixelSize;
             ddsd.lPitch = img->getRowSizeInBytes();
@@ -874,6 +880,12 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
         SD_flags   |= DDSD_MIPMAPCOUNT;
         CAPS_flags |= DDSCAPS_COMPLEX | DDSCAPS_MIPMAP;
         ddsd.dwMipMapCount = img->getNumMipmapLevels();
+        
+        osg::notify(osg::INFO)<<"writing out with mipmaps ddsd.dwMipMapCount"<<ddsd.dwMipMapCount<<std::endl;
+    }
+    else
+    {
+        osg::notify(osg::INFO)<<"no mipmaps to write out."<<std::endl;
     }
 
 
@@ -918,16 +930,17 @@ bool WriteDDSFile(const osg::Image *img, std::ostream& fout)
 class ReaderWriterDDS : public osgDB::ReaderWriter
 {
 public:
+
+    ReaderWriterDDS()
+    {
+        supportsExtension("dds","DDS image format");
+    }
+
     virtual const char* className() const
     { 
         return "DDS Image Reader/Writer"; 
     }
 
-    virtual bool acceptsExtension(const std::string& extension) const
-    { 
-        return osgDB::equalCaseInsensitive(extension,"dds"); 
-    }
-
     virtual ReadResult readObject(const std::string& file, const osgDB::ReaderWriter::Options* options) const
     {
         return readImage(file,options);
@@ -947,7 +960,7 @@ public:
     
         if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
         
-        std::ifstream stream(fileName.c_str(), std::ios::in | std::ios::binary);
+        osgDB::ifstream stream(fileName.c_str(), std::ios::in | std::ios::binary);
         if(!stream) return ReadResult::FILE_NOT_HANDLED;
         ReadResult rr = readImage(stream, options);
         if(rr.validImage()) rr.getImage()->setFileName(file);
@@ -989,7 +1002,7 @@ public:
         std::string ext = osgDB::getFileExtension(file);
         if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
 
-        std::ofstream fout(file.c_str(), std::ios::out | std::ios::binary);
+        osgDB::ofstream fout(file.c_str(), std::ios::out | std::ios::binary);
         if(!fout) return WriteResult::ERROR_IN_WRITING_FILE;
 
         return writeImage(image,fout,options);
