@@ -34,6 +34,7 @@
 #include <osgDB/Registry>
 #include <osgDB/FileNameUtils>
 #include <osgDB/FileUtils>
+#include <osgDB/fstream>
 
 #include <stdio.h>
 #include <assert.h>
@@ -47,8 +48,16 @@
 class ReaderWriterHDR : public osgDB::ReaderWriter
 {
 public:
+    ReaderWriterHDR()
+    {
+        supportsExtension("hdr","High Dynamic Range image format");
+        supportsOption("RGBMUL","");
+        supportsOption("RGB8","");
+        supportsOption("RAW","");
+        supportsOption("YFLIP","");
+        supportsOption("NO_YFLIP","");
+    }
     virtual const char* className() { return "HDR Image Reader"; }
-    virtual bool acceptsExtension(const std::string &extension) const { return osgDB::equalCaseInsensitive(extension, "hdr"); }
 
     virtual ReadResult readImage(const std::string &_file, const osgDB::ReaderWriter::Options *_opts) const
     {
@@ -181,7 +190,7 @@ public:
         std::string ext = osgDB::getFileExtension(file);
         if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
 
-        std::ofstream fout(file.c_str(), std::ios::out | std::ios::binary);
+        osgDB::ofstream fout(file.c_str(), std::ios::out | std::ios::binary);
         if(!fout) return WriteResult::ERROR_IN_WRITING_FILE;
 
         return writeImage(image,fout,options);
@@ -251,8 +260,8 @@ public:
         success = HDRWriter::writeHeader(source.get(),fout);
         if(!success)
         {
-            return WriteResult::ERROR_IN_WRITING_FILE;    // early out
             source = 0;    // delete the temporary image
+            return WriteResult::ERROR_IN_WRITING_FILE;    // early out
         }
 
         success = HDRWriter::writeRLE(source.get(), fout);
