@@ -21,6 +21,7 @@
 #include "BlendColor.h"
 #include "Stencil.h"
 #include "BlendFunc.h"
+#include "BlendEquation.h"
 #include "Depth.h"
 #include "Material.h"
 #include "CullFace.h"
@@ -31,6 +32,7 @@
 #include "ShadeModel.h"
 #include "Point.h"
 #include "LineWidth.h"
+#include "LineStipple.h"
 #include "Texture1D.h"
 #include "Texture2D.h"
 #include "Texture3D.h"
@@ -49,8 +51,11 @@
 #include "Viewport.h"
 #include "Scissor.h"
 #include "Image.h"
+#include "ImageSequence.h"
 #include "PointSprite.h"
 #include "Multisample.h"
+#include "Fog.h"
+#include "Light.h"
 
 
 #include "Group.h"
@@ -82,6 +87,13 @@
 #include "VisibilityGroup.h"
 
 #include "MultiTextureControl.h"
+#include "ShapeAttributeList.h"
+#include "Effect.h"
+#include "AnisotropicLighting.h"
+#include "BumpMapping.h"
+#include "Cartoon.h"
+#include "Scribe.h"
+#include "SpecularHighlights.h"
 
 #include "Geometry.h"
 #include "ShapeDrawable.h"
@@ -94,6 +106,10 @@
 #include "ImageLayer.h"
 #include "HeightFieldLayer.h"
 #include "CompositeLayer.h"
+#include "SwitchLayer.h"
+
+#include "FadeText.h"
+#include "Text3D.h"
 
 #include <osg/Endian>
 #include <osg/Notify>
@@ -107,36 +123,34 @@
 using namespace ive;
 using namespace std;
 
-void DataInputStream::setOptions(const osgDB::ReaderWriter::Options* options) 
-{ 
-    _options = options; 
+DataInputStream::DataInputStream(std::istream* istream, const osgDB::ReaderWriter::Options* options)
+{
+    unsigned int endianType ;
+
+    _loadExternalReferenceFiles = false;
+
+    _verboseOutput = false;
+
+    _istream = istream;
+    _owns_istream = false;
+    _peeking = false;
+    _peekValue = 0;
+    _byteswap = 0;
+
+    _options = options;
 
     if (_options.get())
     {
         setLoadExternalReferenceFiles(_options->getOptionString().find("noLoadExternalReferenceFiles")==std::string::npos);
         osg::notify(osg::DEBUG_INFO) << "ive::DataInputStream.setLoadExternalReferenceFiles()=" << getLoadExternalReferenceFiles() << std::endl;
     }
-}
-
-DataInputStream::DataInputStream(std::istream* istream)
-{
-    unsigned int endianType ;
-
-    _loadExternalReferenceFiles = false;
-    
-    _verboseOutput = false;
-
-    _istream = istream;
-    _peeking = false;
-    _peekValue = 0;
-    _byteswap = 0;
 
     if(!istream){
-        throw Exception("DataInputStream::DataInputStream(): null pointer exception in argument.");    
+        throw Exception("DataInputStream::DataInputStream(): null pointer exception in argument.");
     }
 
     endianType = readUInt() ;
-    
+
     if ( endianType != ENDIAN_TYPE) {
       // Make sure the file is simply swapped
       if ( endianType != OPPOSITE_ENDIAN_TYPE ) {
@@ -145,27 +159,132 @@ DataInputStream::DataInputStream(std::istream* istream)
       osg::notify(osg::INFO)<<"DataInputStream::DataInputStream: Reading a byteswapped file" << std::endl ;
       _byteswap = 1 ;
    }
-    
+
     _version = readUInt();
-        
+
     // Are we trying to open a binary .ive file which version are newer than this library.
     if(_version>VERSION){
         throw Exception("DataInputStream::DataInputStream(): The version found in the file is newer than this library can handle.");
     }
+    
+    if (_version>=VERSION_0033)
+    {
+        int compressionLevel = readInt();
         
+        if (compressionLevel>0)
+        {
+            osg::notify(osg::INFO)<<"compressed ive stream"<<std::endl;
+            
+            unsigned int maxSize = readUInt();
+            
+            std::string data;
+            data.reserve(maxSize);
+            
+            if (!uncompress(*istream, data))
+            {
+                throw Exception("Error in uncompressing .ive");
+            }
+            
+            _istream = new std::stringstream(data);
+            _owns_istream = true;
+        }
+        else
+        {
+            osg::notify(osg::INFO)<<"uncompressed ive stream"<<std::endl;
+        }
+    }
 }
 
-DataInputStream::~DataInputStream(){}
+DataInputStream::~DataInputStream()
+{
+    if (_owns_istream) delete _istream;
+}
+
+#ifdef USE_ZLIB
+
+#include <zlib.h>
+
+bool DataInputStream::uncompress(std::istream& fin, std::string& destination) const
+{
+    //#define CHUNK 16384
+    #define CHUNK 32768
+
+    int ret;
+    unsigned have;
+    z_stream strm;
+    unsigned char in[CHUNK];
+    unsigned char out[CHUNK];
+    
+    /* allocate inflate state */
+    strm.zalloc = Z_NULL;
+    strm.zfree = Z_NULL;
+    strm.opaque = Z_NULL;
+    strm.avail_in = 0;
+    strm.next_in = Z_NULL;
+    ret = inflateInit2(&strm,
+                       15 + 32 // autodected zlib or gzip header
+                       );
+    if (ret != Z_OK)
+    {
+        osg::notify(osg::INFO)<<"failed to init"<<std::endl;
+        return ret != 0;
+    }
+    
+    /* decompress until deflate stream ends or end of file */
+    do {
+        //strm.avail_in = fin.readsome((char*)in, CHUNK);
+        fin.read((char *)in, CHUNK);
+        strm.avail_in = fin.gcount();
+        
+        if (strm.avail_in == 0)
+        {
+            break;
+        }
+        strm.next_in = in;
+
+        /* run inflate() on input until output buffer not full */
+        do {
+            strm.avail_out = CHUNK;
+            strm.next_out = out;
+            ret = inflate(&strm, Z_NO_FLUSH);
+
+            switch (ret) {
+            case Z_NEED_DICT:
+            case Z_DATA_ERROR:
+            case Z_MEM_ERROR:
+                (void)inflateEnd(&strm);
+                return false;
+            }
+            have = CHUNK - strm.avail_out;
+
+            destination.append((char*)out, have);
+            
+        } while (strm.avail_out == 0);
+
+        /* done when inflate() says it's done */
+    } while (ret != Z_STREAM_END);
+
+    /* clean up and return */
+    (void)inflateEnd(&strm);
+    
+    return ret == Z_STREAM_END ? true : false;
+}
+#else
+bool DataInputStream::uncompress(std::istream& fin, std::string& destination) const
+{
+    return false;
+}
+#endif
 
 bool DataInputStream::readBool(){
     char c;
     _istream->read(&c, CHARSIZE);
-    
+
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readBool(): Failed to read boolean value.");
 
     if (_verboseOutput) std::cout<<"read/writeBool() ["<<(int)c<<"]"<<std::endl;
-    
+
     return c!=0;
 }
 
@@ -177,7 +296,7 @@ char DataInputStream::readChar(){
         throw Exception("DataInputStream::readChar(): Failed to read char value.");
 
     if (_verboseOutput) std::cout<<"read/writeChar() ["<<(int)c<<"]"<<std::endl;
-    
+
     return c;
 }
 
@@ -189,7 +308,7 @@ unsigned char DataInputStream::readUChar(){
         throw Exception("DataInputStream::readUChar(): Failed to read unsigned char value.");
 
     if (_verboseOutput) std::cout<<"read/writeUChar() ["<<(int)c<<"]"<<std::endl;
-    
+
     return c;
 }
 
@@ -200,9 +319,9 @@ unsigned short DataInputStream::readUShort(){
         throw Exception("DataInputStream::readUShort(): Failed to read unsigned short value.");
 
     if (_verboseOutput) std::cout<<"read/writeUShort() ["<<s<<"]"<<std::endl;
-    
+
     if (_byteswap) osg::swapBytes((char *)&s,SHORTSIZE);
-    
+
     return s;
 }
 
@@ -214,9 +333,9 @@ unsigned int DataInputStream::readUInt(){
         throw Exception("DataInputStream::readUInt(): Failed to read unsigned int value.");
 
     if (_byteswap) osg::swapBytes((char *)&s,INTSIZE) ;
-    
+
     if (_verboseOutput) std::cout<<"read/writeUInt() ["<<s<<"]"<<std::endl;
-    
+
     return s;
 }
 
@@ -234,9 +353,9 @@ int DataInputStream::readInt(){
     // if (_istream->rdstate() & _istream->failbit)
     //    throw Exception("DataInputStream::readInt(): Failed to read int value.");
 
-    
+
     if (_byteswap) osg::swapBytes((char *)&i,INTSIZE) ;
-    
+
     if (_verboseOutput) std::cout<<"read/writeInt() ["<<i<<"]"<<std::endl;
 
     return i;
@@ -289,7 +408,7 @@ unsigned long DataInputStream::readULong(){
     if (_byteswap) osg::swapBytes((char *)&l,LONGSIZE) ;
 
     if (_verboseOutput) std::cout<<"read/writeULong() ["<<l<<"]"<<std::endl;
-    
+
     return l;
 }
 
@@ -318,7 +437,7 @@ std::string DataInputStream::readString()
 
         if (_verboseOutput) std::cout<<"read/writeString() ["<<s<<"]"<<std::endl;
     }
-        
+
     return s;
 }
 
@@ -338,7 +457,7 @@ osg::Vec2 DataInputStream::readVec2()
     v.y()=readFloat();
 
     if (_verboseOutput) std::cout<<"read/writeVec2() ["<<v<<"]"<<std::endl;
-    
+
     return v;
 }
 
@@ -361,7 +480,7 @@ osg::Vec4 DataInputStream::readVec4(){
     v.w()=readFloat();
 
     if (_verboseOutput) std::cout<<"read/writeVec4() ["<<v<<"]"<<std::endl;
-    
+
     return v;
 }
 osg::Vec2d DataInputStream::readVec2d()
@@ -371,7 +490,7 @@ osg::Vec2d DataInputStream::readVec2d()
     v.y()=readDouble();
 
     if (_verboseOutput) std::cout<<"read/writeVec2d() ["<<v<<"]"<<std::endl;
-    
+
     return v;
 }
 
@@ -394,7 +513,7 @@ osg::Vec4d DataInputStream::readVec4d(){
     v.w()=readDouble();
 
     if (_verboseOutput) std::cout<<"read/writeVec4d() ["<<v<<"]"<<std::endl;
-    
+
     return v;
 }
 
@@ -417,9 +536,9 @@ osg::Plane DataInputStream::readPlane(){
         v[2]=readDouble();
         v[3]=readDouble();
     }
-    
+
     if (_verboseOutput) std::cout<<"read/writePlane() ["<<v<<"]"<<std::endl;
-    
+
     return v;
 }
 
@@ -431,7 +550,7 @@ osg::Vec4ub DataInputStream::readVec4ub(){
     v.a()=readChar();
 
     if (_verboseOutput) std::cout<<"read/writeVec4ub() ["<<v<<"]"<<std::endl;
-    
+
     return v;
 }
 
@@ -443,7 +562,7 @@ osg::Quat DataInputStream::readQuat(){
     q.w()=readFloat();
 
     if (_verboseOutput) std::cout<<"read/writeQuat() ["<<q<<"]"<<std::endl;
-    
+
     return q;
 }
 
@@ -454,7 +573,7 @@ osg::Geometry::AttributeBinding DataInputStream::readBinding(){
     char c = readChar();
 
     if (_verboseOutput) std::cout<<"read/writeBinding() ["<<(int)c<<"]"<<std::endl;
-    
+
     switch((int)c){
         case 0:    return osg::Geometry::BIND_OFF;
         case 1: return osg::Geometry::BIND_OVERALL;
@@ -480,12 +599,12 @@ osg::Array* DataInputStream::readArray(){
         case 9:    return readVec2sArray();
         case 10:   return readVec3sArray();
         case 11:   return readVec4sArray();
-        case 12:   return readVec2bArray();        
-        case 13:   return readVec3bArray();        
+        case 12:   return readVec2bArray();
+        case 13:   return readVec3bArray();
         case 14:   return readVec4bArray();
         case 15:   return readVec2dArray();
         case 16:   return readVec3dArray();
-        case 17:   return readVec4dArray();        
+        case 17:   return readVec4dArray();
         default: throw Exception("Unknown array type in DataInputStream::readArray()");
     }
 }
@@ -496,18 +615,18 @@ osg::IntArray* DataInputStream::readIntArray()
     if (size == 0)
         return NULL;
     osg::IntArray* a = new osg::IntArray(size);
-    
+
     _istream->read((char*)&((*a)[0]), INTSIZE*size);
 
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readIntArray(): Failed to read Int array.");
 
-    if (_verboseOutput) std::cout<<"read/writeIntArray() ["<<size<<"]"<<std::endl;  
+    if (_verboseOutput) std::cout<<"read/writeIntArray() ["<<size<<"]"<<std::endl;
 
     if (_byteswap) {
        for (int  i = 0 ; i < size ; i++ ) osg::swapBytes((char *)&((*a)[i]),INTSIZE) ;
     }
-       
+
     return a;
 }
 
@@ -524,7 +643,7 @@ osg::UByteArray* DataInputStream::readUByteArray()
         throw Exception("DataInputStream::readUByteArray(): Failed to read UByte array.");
 
     if (_verboseOutput) std::cout<<"read/writeUByteArray() ["<<size<<"]"<<std::endl;
-    
+
     return a;
 }
 
@@ -541,10 +660,10 @@ osg::UShortArray* DataInputStream::readUShortArray()
         throw Exception("DataInputStream::readUShortArray(): Failed to read UShort array.");
 
     if (_verboseOutput) std::cout<<"read/writeUShortArray() ["<<size<<"]"<<std::endl;
-    
+
     if (_byteswap)
     {
-        for (int i = 0 ; i < size ; i++ ) 
+        for (int i = 0 ; i < size ; i++ )
             osg::swapBytes((char *)&((*a)[i]),SHORTSIZE) ;
     }
     return a;
@@ -563,7 +682,7 @@ osg::UIntArray* DataInputStream::readUIntArray()
         throw Exception("DataInputStream::readUIntArray(): Failed to read UInt array.");
 
     if (_verboseOutput) std::cout<<"read/writeUIntArray() ["<<size<<"]"<<std::endl;
-    
+
     if (_byteswap)
     {
         for (int i = 0 ; i < size ; i++ )
@@ -585,25 +704,95 @@ osg::Vec4ubArray* DataInputStream::readVec4ubArray()
         throw Exception("DataInputStream::readVec4ubArray(): Failed to read Vec4ub array.");
 
     if (_verboseOutput) std::cout<<"read/writeVec4ubArray() ["<<size<<"]"<<std::endl;
-    
+
     return a;
 }
 
+bool DataInputStream::readPackedFloatArray(osg::FloatArray* a)
+{
+    int size = readInt();
+
+    a->resize(size);        
+
+    if (size == 0)
+        return true;
+
+    if (readBool())
+    {
+        float value = readFloat();
+        
+        for(int i=0; i<size; ++i)
+        {
+            (*a)[i] = value;
+        }
+    }
+    else
+    {
+        int packingSize = readInt();
+        
+        if (packingSize==1)
+        {
+            float minValue = readFloat();
+            float maxValue = readFloat();
+
+            float byteMultiplier = 255.0f/(maxValue-minValue);
+            float byteInvMultiplier = 1.0f/byteMultiplier;
+
+            for(int i=0; i<size; ++i)
+            {
+                unsigned char byte_value = readUChar();
+                float value = minValue + float(byte_value)*byteInvMultiplier;
+                (*a)[i] = value;
+            }
+        }
+        else if (packingSize==2)
+        {
+            float minValue = readFloat();
+            float maxValue = readFloat();
+
+            float shortMultiplier = 65535.0f/(maxValue-minValue);
+            float shortInvMultiplier = 1.0f/shortMultiplier;
+
+            for(int i=0; i<size; ++i)
+            {
+                unsigned short short_value = readUShort();
+                float value = minValue + float(short_value)*shortInvMultiplier;
+                (*a)[i] = value;
+            }
+        }
+        else
+        {
+            for(int i=0; i<size; ++i)
+            {
+                (*a)[i] = readFloat();
+            }
+        }        
+    }
+    
+    if (_istream->rdstate() & _istream->failbit)
+        throw Exception("DataInputStream::readFloatArray(): Failed to read float array.");
+
+    if (_verboseOutput) std::cout<<"read/writeFloatArray() ["<<size<<"]"<<std::endl;
+
+    return true;
+}
+
+
 osg::FloatArray* DataInputStream::readFloatArray()
 {
     int size = readInt();
     if (size == 0)
         return NULL;
-    
+
     osg::FloatArray* a = new osg::FloatArray(size);
-    
+
     _istream->read((char*)&((*a)[0]), FLOATSIZE*size);
 
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readFloatArray(): Failed to read float array.");
 
     if (_verboseOutput) std::cout<<"read/writeFloatArray() ["<<size<<"]"<<std::endl;
-    
+
     if (_byteswap)
     {
         for (int i = 0 ; i < size ; i++ )
@@ -619,14 +808,14 @@ osg::Vec2Array* DataInputStream::readVec2Array()
         return NULL;
 
     osg::Vec2Array* a = new osg::Vec2Array(size);
-    
+
     _istream->read((char*)&((*a)[0]), FLOATSIZE*2*size);
 
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readVec2Array(): Failed to read Vec2 array.");
 
     if (_verboseOutput) std::cout<<"read/writeVec2Array() ["<<size<<"]"<<std::endl;
-    
+
     if (_byteswap)
     {
        float *ptr = (float*)&((*a)[0]) ;
@@ -646,12 +835,12 @@ osg::Vec3Array* DataInputStream::readVec3Array()
     osg::Vec3Array* a = new osg::Vec3Array(size);
 
     _istream->read((char*)&((*a)[0]), FLOATSIZE*3*size);
-    
+
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readVec3Array(): Failed to read Vec3 array.");
 
     if (_verboseOutput) std::cout<<"read/writeVec3Array() ["<<size<<"]"<<std::endl;
-    
+
 
     if (_byteswap)
     {
@@ -676,7 +865,7 @@ osg::Vec4Array* DataInputStream::readVec4Array(){
         throw Exception("DataInputStream::readVec4Array(): Failed to read Vec4 array.");
 
     if (_verboseOutput) std::cout<<"read/writeVec4Array() ["<<size<<"]"<<std::endl;
-    
+
     if (_byteswap) {
        float *ptr = (float*)&((*a)[0]) ;
        for (int i = 0 ; i < size*4 ; i++ ) {
@@ -823,14 +1012,14 @@ osg::Vec2dArray* DataInputStream::readVec2dArray()
         return NULL;
 
     osg::Vec2dArray* a = new osg::Vec2dArray(size);
-    
+
     _istream->read((char*)&((*a)[0]), DOUBLESIZE*2*size);
 
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readVec2dArray(): Failed to read Vec2d array.");
 
     if (_verboseOutput) std::cout<<"read/writeVec2dArray() ["<<size<<"]"<<std::endl;
-    
+
     if (_byteswap)
     {
        double *ptr = (double*)&((*a)[0]) ;
@@ -850,12 +1039,12 @@ osg::Vec3dArray* DataInputStream::readVec3dArray()
     osg::Vec3dArray* a = new osg::Vec3dArray(size);
 
     _istream->read((char*)&((*a)[0]), DOUBLESIZE*3*size);
-    
+
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readVec3dArray(): Failed to read Vec3d array.");
 
     if (_verboseOutput) std::cout<<"read/writeVec3dArray() ["<<size<<"]"<<std::endl;
-    
+
 
     if (_byteswap)
     {
@@ -880,7 +1069,7 @@ osg::Vec4dArray* DataInputStream::readVec4dArray(){
         throw Exception("DataInputStream::readVec4dArray(): Failed to read Vec4d array.");
 
     if (_verboseOutput) std::cout<<"read/writeVec4dArray() ["<<size<<"]"<<std::endl;
-    
+
     if (_byteswap) {
        double *ptr = (double*)&((*a)[0]) ;
        for (int i = 0 ; i < size*4 ; i++ ) {
@@ -905,7 +1094,7 @@ osg::Matrixf DataInputStream::readMatrixf()
         throw Exception("DataInputStream::readMatrix(): Failed to read Matrix array.");
 
     if (_verboseOutput) std::cout<<"read/writeMatrix() ["<<mat<<"]"<<std::endl;
-    
+
 
     return mat;
 }
@@ -925,29 +1114,54 @@ osg::Matrixd DataInputStream::readMatrixd()
         throw Exception("DataInputStream::readMatrix(): Failed to read Matrix array.");
 
     if (_verboseOutput) std::cout<<"read/writeMatrix() ["<<mat<<"]"<<std::endl;
-    
+
 
     return mat;
 }
 
 osg::Image* DataInputStream::readImage(std::string filename)
 {
-    // If image is already read and in list 
+    // If image is already read and in list
     // then just return pointer to this.
-    ImageMap::iterator mitr=_imageMap.find(filename);    
+    ImageMap::iterator mitr=_imageMap.find(filename);
     if (mitr!=_imageMap.end()) return mitr->second.get();
-        
-    // Image is not in list. 
-    // Read it from disk, 
-    osg::Image* image = osgDB::readImageFile(filename.c_str(),_options.get());
-        
+
+    // Image is not in list.
+    // Read it from disk,
+    osg::ref_ptr<osg::Image> image = osgDB::readRefImageFile(filename.c_str(),_options.get());
+
     // add it to the imageList,
     _imageMap[filename] = image;
     // and return image pointer.
 
     if (_verboseOutput) std::cout<<"read/writeImage() ["<<image<<"]"<<std::endl;
-    
-    return image;
+
+    return image.release();
+}
+
+osg::Image* DataInputStream::readImage()
+{
+    if ( getVersion() >= VERSION_0029 )
+    {
+        int id = peekInt();
+        if(id == IVEIMAGESEQUENCE)
+        {
+            osg::ImageSequence* image = new osg::ImageSequence();
+            ((ive::ImageSequence*)image)->read(this);
+            return image;
+        }
+        else
+        {
+            readInt();
+            IncludeImageMode includeImg = (IncludeImageMode)readChar();
+            return readImage(includeImg);
+        }
+    }
+    else
+    {
+        IncludeImageMode includeImg = (IncludeImageMode)readChar();
+        return readImage(includeImg);
+    }
 }
 
 osg::Image* DataInputStream::readImage(IncludeImageMode mode)
@@ -966,7 +1180,7 @@ osg::Image* DataInputStream::readImage(IncludeImageMode mode)
             // Only read image name from stream.
             {
                 std::string filename = readString();
-                if(filename.compare("")!=0){
+                if(!filename.empty()){
                     return readImage(filename);
                 }
             }
@@ -1028,13 +1242,13 @@ osg::StateSet* DataInputStream::readStateSet()
 
     // read its properties from stream
     ((ive::StateSet*)(stateset))->read(this);
-        
+
     // and add it to the stateset map,
     _statesetMap[id] = stateset;
-        
+
 
     if (_verboseOutput) std::cout<<"read/writeStateSet() ["<<id<<"]"<<std::endl;
-    
+
     return stateset;
 }
 
@@ -1065,6 +1279,10 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::BlendFunc();
         ((ive::BlendFunc*)(attribute))->read(this);
     }
+    else if(attributeID == IVEBLENDEQUATION){
+        attribute = new osg::BlendEquation();
+        ((ive::BlendEquation*)(attribute))->read(this);
+    }
     else if(attributeID == IVEDEPTH){
         attribute = new osg::Depth();
         ((ive::Depth*)(attribute))->read(this);
@@ -1177,20 +1395,32 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::Multisample();
         ((ive::Multisample*)(attribute))->read(this);
     }
+    else if(attributeID == IVELINESTIPPLE){
+        attribute = new osg::LineStipple();
+        ((ive::LineStipple*)(attribute))->read(this);
+    }
     else if(attributeID == IVESTENCIL){
         attribute = new osg::Stencil();
         ((ive::Stencil*)(attribute))->read(this);
     }
+    else if(attributeID == IVEFOG){
+        attribute = new osg::Fog();
+        ((ive::Fog*)(attribute))->read(this);
+    }
+    else if(attributeID == IVELIGHT){
+        attribute = new osg::Light();
+        ((ive::Light*)(attribute))->read(this);
+    }
     else{
         throw Exception("Unknown StateAttribute in StateSet::read()");
     }
-       
+
     // and add it to the stateattribute map,
     _stateAttributeMap[id] = attribute;
-        
+
 
     if (_verboseOutput) std::cout<<"read/writeStateAttribute() ["<<id<<"]"<<std::endl;
-    
+
     return attribute;
 }
 
@@ -1208,13 +1438,13 @@ osg::Uniform* DataInputStream::readUniform()
 
     // read its properties from stream
     ((ive::Uniform*)(uniform))->read(this);
-        
+
     // and add it to the uniform map,
     _uniformMap[id] = uniform;
-        
+
 
     if (_verboseOutput) std::cout<<"read/writeUniform() ["<<id<<"]"<<std::endl;
-    
+
     return uniform;
 }
 
@@ -1233,13 +1463,13 @@ osg::Shader* DataInputStream::readShader()
 
     // read its properties from stream
     ((ive::Shader*)(shader))->read(this);
-        
+
     // and add it to the shader map,
     _shaderMap[id] = shader;
-        
+
 
     if (_verboseOutput) std::cout<<"read/writeShader() ["<<id<<"]"<<std::endl;
-    
+
     return shader;
 }
 
@@ -1258,28 +1488,36 @@ osg::Drawable* DataInputStream::readDrawable()
     osg::Drawable* drawable;
     if(drawableTypeID == IVEGEOMETRY)
     {
-        drawable = new osg::Geometry();                
+        drawable = new osg::Geometry();
         ((Geometry*)(drawable))->read(this);
     }
     else if(drawableTypeID == IVESHAPEDRAWABLE)
     {
-        drawable = new osg::ShapeDrawable();                
+        drawable = new osg::ShapeDrawable();
         ((ShapeDrawable*)(drawable))->read(this);
     }
     else if(drawableTypeID == IVETEXT){
         drawable = new osgText::Text();
         ((Text*)(drawable))->read(this);
-    }    
+    }
+    else if(drawableTypeID == IVEFADETEXT){
+        drawable = new osgText::FadeText();
+        ((FadeText*)(drawable))->read(this);
+    }
+    else if(drawableTypeID == IVETEXT3D){
+        drawable = new osgText::Text3D();
+        ((Text3D*)(drawable))->read(this);
+    }
     else
         throw Exception("Unknown drawable drawableTypeIDentification in Geode::read()");
 
-       
+
     // and add it to the stateattribute map,
     _drawableMap[id] = drawable;
-        
+
 
     if (_verboseOutput) std::cout<<"read/writeDrawable() ["<<id<<"]"<<std::endl;
-    
+
     return drawable;
 }
 
@@ -1298,7 +1536,7 @@ osg::Shape* DataInputStream::readShape()
     osg::Shape* shape;
     if(shapeTypeID == IVESPHERE)
     {
-        shape = new osg::Sphere();                
+        shape = new osg::Sphere();
         ((Sphere*)(shape))->read(this);
     }
     else if(shapeTypeID == IVEBOX)
@@ -1323,19 +1561,19 @@ osg::Shape* DataInputStream::readShape()
     }
     else if(shapeTypeID == IVEHEIGHTFIELD)
     {
-        shape = new osg::HeightField();                
+        shape = new osg::HeightField();
         ((HeightField*)(shape))->read(this);
     }
     else
         throw Exception("Unknown shape shapeTypeIDentification in Shape::read()");
 
-       
+
     // and add it to the stateattribute map,
     _shapeMap[id] = shape;
-        
+
 
     if (_verboseOutput) std::cout<<"read/writeShape() ["<<id<<"]"<<std::endl;
-    
+
     return shape;
 }
 
@@ -1352,7 +1590,7 @@ osg::Node* DataInputStream::readNode()
 
     osg::Node* node;
     int nodeTypeID= peekInt();
-    
+
     if(nodeTypeID== IVEMATRIXTRANSFORM){
         node = new osg::MatrixTransform();
         ((ive::MatrixTransform*)(node))->read(this);
@@ -1457,6 +1695,28 @@ osg::Node* DataInputStream::readNode()
         node = new osgFX::MultiTextureControl();
         ((ive::MultiTextureControl*)(node))->read(this);
     }
+
+    else if(nodeTypeID== IVEANISOTROPICLIGHTING){
+        node = new osgFX::AnisotropicLighting();
+        ((ive::AnisotropicLighting*)(node))->read(this);
+    }
+    else if(nodeTypeID== IVEBUMPMAPPING){
+        node = new osgFX::BumpMapping();
+        ((ive::BumpMapping*)(node))->read(this);
+    }
+    else if(nodeTypeID== IVECARTOON){
+        node = new osgFX::Cartoon();
+        ((ive::Cartoon*)(node))->read(this);
+    }
+    else if(nodeTypeID== IVESCRIBE){
+        node = new osgFX::Scribe();
+        ((ive::Scribe*)(node))->read(this);
+    }
+    else if(nodeTypeID== IVESPECULARHIGHLIGHTS){
+        node = new osgFX::SpecularHighlights();
+        ((ive::SpecularHighlights*)(node))->read(this);
+    }
+
     else if(nodeTypeID== IVETERRAINTILE){
         node = new osgTerrain::TerrainTile();
         ((ive::TerrainTile*)(node))->read(this);
@@ -1467,10 +1727,10 @@ osg::Node* DataInputStream::readNode()
 
     // and add it to the node map,
     _nodeMap[id] = node;
-        
+
 
     if (_verboseOutput) std::cout<<"read/writeNode() ["<<id<<"]"<<std::endl;
-    
+
     return node;
 }
 
@@ -1479,7 +1739,7 @@ osgTerrain::Layer* DataInputStream::readLayer()
     // Read node unique ID.
     int id = readInt();
     if (id<0) return 0;
-    
+
     // See if layer is already in the list.
     LayerMap::iterator itr= _layerMap.find(id);
     if (itr!=_layerMap.end()) return itr->second.get();
@@ -1489,7 +1749,7 @@ osgTerrain::Layer* DataInputStream::readLayer()
 
     osgTerrain::Layer* layer = 0;
     int layerid = peekInt();
-    
+
     if (layerid==IVEHEIGHTFIELDLAYER)
     {
         layer = new osgTerrain::HeightFieldLayer;
@@ -1500,6 +1760,11 @@ osgTerrain::Layer* DataInputStream::readLayer()
         layer = new osgTerrain::ImageLayer;
         ((ive::ImageLayer*)(layer))->read(this);
     }
+    else if (layerid==IVESWITCHLAYER)
+    {
+        layer = new osgTerrain::SwitchLayer;
+        ((ive::SwitchLayer*)(layer))->read(this);
+    }
     else if (layerid==IVECOMPOSITELAYER)
     {
         layer = new osgTerrain::CompositeLayer;
@@ -1510,19 +1775,19 @@ osgTerrain::Layer* DataInputStream::readLayer()
         std::string filename = readString();
         osg::ref_ptr<osg::Object> object = osgDB::readObjectFile(filename+".gdal");
         osgTerrain::ProxyLayer* proxyLayer = dynamic_cast<osgTerrain::ProxyLayer*>(object.get());
-        
+
         osg::ref_ptr<osgTerrain::Locator> locator = readLocator();
         unsigned int minLevel = readUInt();
         unsigned int maxLevel = readUInt();
-        
+
         if (proxyLayer)
         {
             if (locator.valid()) proxyLayer->setLocator(locator.get());
-            
+
             proxyLayer->setMinLevel(minLevel);
             proxyLayer->setMaxLevel(maxLevel);
         }
-        
+
         layer = proxyLayer;
     }
     else{
@@ -1531,10 +1796,10 @@ osgTerrain::Layer* DataInputStream::readLayer()
 
     // and add it to the node map,
     _layerMap[id] = layer;
-        
+
 
     if (_verboseOutput) std::cout<<"read/writeLayer() ["<<id<<"]"<<std::endl;
-    
+
     return layer;
 }
 
@@ -1544,7 +1809,7 @@ osgTerrain::Locator* DataInputStream::readLocator()
     // Read statesets unique ID.
     int id = readInt();
     if (id<0) return 0;
-    
+
     // See if stateset is already in the list.
     LocatorMap::iterator itr= _locatorMap.find(id);
     if (itr!=_locatorMap.end()) return itr->second.get();
@@ -1555,12 +1820,44 @@ osgTerrain::Locator* DataInputStream::readLocator()
 
     // read its properties from stream
     ((ive::Locator*)(locator))->read(this);
-        
+
     // and add it to the locator map,
     _locatorMap[id] = locator;
 
     if (_verboseOutput) std::cout<<"read/writeLocator() ["<<id<<"]"<<std::endl;
-    
+
     return locator;
 }
 
+osg::Object* DataInputStream::readObject()
+{
+    int id = readInt();
+    if (id<0) return 0;
+    
+    if (id==IVENODE)
+    {
+        return readNode();        
+    }
+    else if (id==IVESTATESET)
+    {
+        return readStateSet();
+    }
+    else if (id==IVESTATEATTRIBUTE)
+    {
+        return readStateAttribute();
+    }
+    else if (id==IVEDRAWABLE)
+    {
+        return readDrawable();
+    }
+    else if (id==IVESHAPEATTRIBUTELIST)
+    {
+        osgSim::ShapeAttributeList* sal = new osgSim::ShapeAttributeList;
+        ((ive::ShapeAttributeList*)sal)->read(this);
+        return sal;
+    }
+    
+    return 0;
+}
+
+
