@@ -20,6 +20,7 @@
     #pragma warning( disable : 4786 )
 #endif
 
+#include <stdlib.h>
 #include <string>
 
 #include <osg/Notify>
@@ -52,12 +53,25 @@
 class ReaderWriterOBJ : public osgDB::ReaderWriter
 {
 public:
-    ReaderWriterOBJ():_fixBlackMaterials(true) {}
+    ReaderWriterOBJ()
+    {
+        supportsExtension("obj","Alias Wavefront OBJ format");
+        supportsOption("noRotation","Do not do the default rotate about X axis");
+        supportsOption("noTesselateLargePolygons","Do not do the default tesselation of large polygons");
+        supportsOption("noTriStripPolygons","Do not do the default tri stripping of polygons");
+
+        supportsOption("DIFFUSE=<unit>", "Set texture unit for diffuse texture");
+        supportsOption("AMBIENT=<unit>", "Set texture unit for ambient texture");
+        supportsOption("SPECULAR=<unit>", "Set texture unit for specular texture");
+        supportsOption("SPECULAR_EXPONENT=<unit>", "Set texture unit for specular exponent texture");
+        supportsOption("OPACITY=<unit>", "Set texture unit for opacity/dissolve texture");
+        supportsOption("BUMP=<unit>", "Set texture unit for bumpmap texture");
+        supportsOption("DISPLACEMENT=<unit>", "Set texture unit for displacement texture");
+        supportsOption("REFLECTION=<unit>", "Set texture unit for reflection texture");
+
+    }
 
     virtual const char* className() const { return "Wavefront OBJ Reader"; }
-    virtual bool acceptsExtension(const std::string& extension) const {
-        return osgDB::equalCaseInsensitive(extension,"obj");
-    }
 
     virtual ReadResult readNode(const std::string& fileName, const osgDB::ReaderWriter::Options* options) const;
 
@@ -77,14 +91,14 @@ public:
         if (!acceptsExtension(osgDB::getFileExtension(fileName)))
             return WriteResult(WriteResult::FILE_NOT_HANDLED); 
             
-        std::ofstream f(fileName.c_str());
+        osgDB::ofstream f(fileName.c_str());
         std::string materialFile = osgDB::getNameLessExtension(fileName) + ".mtl";
         OBJWriterNodeVisitor nv(f, osgDB::getSimpleFileName(materialFile));
         
         // we must cast away constness
         (const_cast<osg::Node*>(&node))->accept(nv);
         
-        std::ofstream mf(materialFile.c_str());
+        osgDB::ofstream mf(materialFile.c_str());
         nv.writeMaterials(mf);
          
         return WriteResult(WriteResult::FILE_SAVED); 
@@ -116,18 +130,30 @@ public:
 
 protected:
 
+     struct ObjOptionsStruct {
+        bool rotate;
+        bool noTesselateLargePolygons;
+        bool noTriStripPolygons;
+        bool fixBlackMaterials;
+        // This is the order in which the materials will be assigned to texture maps, unless
+        // otherwise overriden
+        typedef std::vector< std::pair<int,obj::Material::Map::TextureMapType> > TextureAllocationMap;
+        TextureAllocationMap textureUnitAllocation;
+    };
+
     typedef std::map< std::string, osg::ref_ptr<osg::StateSet> > MaterialToStateSetMap;
     
-    void buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToSetSetMap) const;
+    void buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToSetSetMapObj, ObjOptionsStruct& localOptions) const;
     
     osg::Geometry* convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList, bool& rotate) const;
     
-    osg::Node* convertModelToSceneGraph(obj::Model& model, bool& rotate) const;
+    osg::Node* convertModelToSceneGraph(obj::Model& model, ObjOptionsStruct& localOptions) const;
 
     inline osg::Vec3 transformVertex(const osg::Vec3& vec, const bool rotate) const ;
     inline osg::Vec3 transformNormal(const osg::Vec3& vec, const bool rotate) const ;
-    
-    bool _fixBlackMaterials;
+ 
+    ObjOptionsStruct parseOptions(const Options* options) const;
+
 
 };
 
@@ -159,11 +185,89 @@ inline osg::Vec3 ReaderWriterOBJ::transformNormal(const osg::Vec3& vec, const bo
 // register with Registry to instantiate the above reader/writer.
 REGISTER_OSGPLUGIN(obj, ReaderWriterOBJ)
 
-void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToStateSetMap) const
+static void load_material_texture(    obj::Model &model,
+                                    obj::Material::Map &map,
+                                    osg::StateSet *stateset,
+                                    const unsigned int texture_unit  )
 {
-    osg::Texture::WrapMode textureWrapMode = osg::Texture::REPEAT;
+    std::string filename = map.name;
+    if (!filename.empty())
+    {
+        osg::ref_ptr< osg::Image > image;
+        if ( !model.getDatabasePath().empty() ) 
+        {
+            // first try with database path of parent. 
+            image = osgDB::readRefImageFile(model.getDatabasePath()+'/'+filename);
+        }
+        
+        if ( !image.valid() )
+        {
+            // if not already set then try the filename as is.
+            image = osgDB::readRefImageFile(filename);
+        }
 
-    if (_fixBlackMaterials)
+        if ( image.valid() )
+        {
+            osg::Texture2D* texture = new osg::Texture2D( image.get() );
+            osg::Texture::WrapMode textureWrapMode;
+            if(map.clamp == true)
+            {
+                textureWrapMode = osg::Texture::CLAMP_TO_BORDER;
+                texture->setBorderColor(osg::Vec4(0.0,0.0,0.0,0.0));    // transparent
+                //stateset->setMode(GL_BLEND, osg::StateAttribute::ON);
+                //stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
+            }
+            else
+            {
+                textureWrapMode = osg::Texture::REPEAT;
+            }
+
+            texture->setWrap(osg::Texture2D::WRAP_R, textureWrapMode);
+            texture->setWrap(osg::Texture2D::WRAP_S, textureWrapMode);
+            texture->setWrap(osg::Texture2D::WRAP_T, textureWrapMode);
+            stateset->setTextureAttributeAndModes( texture_unit, texture,osg::StateAttribute::ON );
+            
+            if ( map.type == obj::Material::Map::REFLECTION )
+            {
+                osg::TexGen* texgen = new osg::TexGen;
+                texgen->setMode(osg::TexGen::SPHERE_MAP);
+                stateset->setTextureAttributeAndModes( texture_unit,texgen,osg::StateAttribute::ON );
+            }
+            
+            if  ( image->isImageTranslucent())
+            {
+                osg::notify(osg::INFO)<<"Found transparent image"<<std::endl;
+                stateset->setMode(GL_BLEND, osg::StateAttribute::ON);
+                stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
+            }
+        }
+    }
+
+    if (map.uScale != 1.0f || map.vScale != 1.0f ||
+            map.uOffset != 0.0f || map.vOffset != 0.0f)
+    {
+        osg::Matrix mat;
+        if (map.uScale != 1.0f || map.vScale != 1.0f)
+        {
+            osg::notify(osg::DEBUG_INFO) << "Obj TexMat scale=" << map.uScale << "," << map.vScale << std::endl;
+            mat *= osg::Matrix::scale(map.uScale, map.vScale, 1.0);
+        }
+        if (map.uOffset != 0.0f || map.vOffset != 0.0f)
+        {
+            osg::notify(osg::DEBUG_INFO) << "Obj TexMat offset=" << map.uOffset << "," << map.uOffset << std::endl;
+            mat *= osg::Matrix::translate(map.uOffset, map.vOffset, 0.0);
+        }
+
+        osg::TexMat* texmat = new osg::TexMat;
+        texmat->setMatrix(mat);
+        stateset->setTextureAttributeAndModes( texture_unit,texmat,osg::StateAttribute::ON );
+    }
+}
+
+
+void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToStateSetMap, ObjOptionsStruct& localOptions) const
+{
+    if (localOptions.fixBlackMaterials)
     {
         // hack to fix Maya exported models that contian all black materials.
         int numBlack = 0;
@@ -207,8 +311,8 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
         ++itr)
     {
         obj::Material& material = itr->second;
-        
-        osg::StateSet* stateset = new osg::StateSet;
+       
+        osg::ref_ptr< osg::StateSet > stateset = new osg::StateSet;
 
         bool isTransparent = false;
 
@@ -221,6 +325,8 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
 
             osg_material->setAmbient(osg::Material::FRONT_AND_BACK,material.ambient);
             osg_material->setDiffuse(osg::Material::FRONT_AND_BACK,material.diffuse);
+            osg_material->setEmission(osg::Material::FRONT_AND_BACK,material.emissive);
+
             if (material.illum == 2) {
                 osg_material->setSpecular(osg::Material::FRONT_AND_BACK,material.specular);
             } else {
@@ -230,60 +336,69 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
             
             if (material.ambient[3]!=1.0 ||
                 material.diffuse[3]!=1.0 ||
-                material.specular[3]!=1.0)
+                material.specular[3]!=1.0||
+                material.emissive[3]!=1.0)
             {
                 osg::notify(osg::INFO)<<"Found transparent material"<<std::endl;
                 isTransparent = true;
             }
         }
         
-        // handle textures
-        if (!material.map_Kd.empty())
+        // If the user has explicitly set the required texture type to unit map via the options 
+        // string, then we load ONLY those textures that are in the map.
+        if(localOptions.textureUnitAllocation.size()>0)
         {
-            std::string filename = material.map_Kd;
-            osg::Image* image = 0;
-            if (!model.getDatabasePath().empty()) 
+            for(unsigned int i=0;i<localOptions.textureUnitAllocation.size();i++)
             {
-                // first try with databasr path of parent. 
-                image = osgDB::readImageFile(model.getDatabasePath()+'/'+filename);
-            }
-            
-            if (!image)
-            {
-                // if not already set then try the filename as is.
-                image = osgDB::readImageFile(filename);
-            }
-            if (image)
-            {
-                osg::Texture2D* texture = new osg::Texture2D(image);
-                texture->setWrap(osg::Texture2D::WRAP_R, textureWrapMode);
-                texture->setWrap(osg::Texture2D::WRAP_S, textureWrapMode);
-                texture->setWrap(osg::Texture2D::WRAP_T, textureWrapMode);
-                stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
-                
-                if (material.textureReflection)
+                // firstly, get the option set pair
+                int unit = localOptions.textureUnitAllocation[i].first;
+                obj::Material::Map::TextureMapType type = localOptions.textureUnitAllocation[i].second;
+                // secondly, see if this texture type (e.g. DIFFUSE) is one of those in the material
+                int index = -1;
+                for(unsigned int j=0;j<material.maps.size();j++)
                 {
-                    osg::TexGen* texgen = new osg::TexGen;
-                    texgen->setMode(osg::TexGen::SPHERE_MAP);
-                    stateset->setTextureAttributeAndModes(0,texgen,osg::StateAttribute::ON);
+                    if(material.maps[j].type == type)
+                    {
+                        index = (int) j;
+                        break;
+                    }
                 }
-                
-                if  (!isTransparent && image->isImageTranslucent())
+                if(index>=0) load_material_texture( model, material.maps[index], stateset.get(), unit );
+            }
+        }
+        // If the user has set no options, then we load them up in the order contained in the enum. This
+        // latter method is an attempt not to break user's existing code
+        else
+        {
+            int unit = 0;
+            for(int i=0;i<(int) obj::Material::Map::UNKNOWN;i++) // for each type
+            {
+                obj::Material::Map::TextureMapType type = (obj::Material::Map::TextureMapType) i;
+                // see if this texture type (e.g. DIFFUSE) is one of those in the material
+                int index = -1;
+                for(unsigned int j=0;j<material.maps.size();j++)
                 {
-                    osg::notify(osg::INFO)<<"Found transparent image"<<std::endl;
-                    isTransparent = true;
+                    if(material.maps[j].type == type)
+                    {
+                        index = (int) j;
+                        break;
+                    }
+                }
+                if(index>=0)
+                {
+                    load_material_texture( model, material.maps[index], stateset.get(), unit );
+                    unit++;
                 }
             }
         }
-
+      
         if (isTransparent)
         {
             stateset->setMode(GL_BLEND, osg::StateAttribute::ON);
             stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
         }
-        
-        materialToStateSetMap[material.name] = stateset;
-        
+
+        materialToStateSetMap[material.name] = stateset.get();
     }
 }
 
@@ -540,7 +655,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
     return geometry;
 }
 
-osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, bool& rotate) const
+osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, ObjOptionsStruct& localOptions) const
 {
 
     if (model.elementStateMap.empty()) return 0;
@@ -549,7 +664,7 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, bool& ro
 
     // set up the materials
     MaterialToStateSetMap materialToSetSetMap;
-    buildMaterialToStateSetMap(model, materialToSetSetMap);
+    buildMaterialToStateSetMap(model, materialToSetSetMap, localOptions);
 
     // go through the groups of related elements and build geometry from them.
     for(obj::Model::ElementStateMap::iterator itr=model.elementStateMap.begin();
@@ -560,7 +675,7 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, bool& ro
         const obj::ElementState& es = itr->first;
         obj::Model::ElementList& el = itr->second;
 
-        osg::Geometry* geometry = convertElementListToGeometry(model,el,rotate);
+        osg::Geometry* geometry = convertElementListToGeometry(model,el,localOptions.rotate);
 
         if (geometry)
         {
@@ -569,18 +684,24 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, bool& ro
             geometry->setStateSet(stateset);
         
             // tesseleate any large polygons
-            osgUtil::Tessellator tessellator;
-            tessellator.retessellatePolygons(*geometry);
-
+            if (!localOptions.noTesselateLargePolygons)
+            {
+                osgUtil::Tessellator tessellator;
+                tessellator.retessellatePolygons(*geometry);
+            }
+            
             // tri strip polygons to improve graphics peformance
-            osgUtil::TriStripVisitor tsv;
-            tsv.stripify(*geometry);
-
+            if (!localOptions.noTriStripPolygons)
+            {
+                osgUtil::TriStripVisitor tsv;
+                tsv.stripify(*geometry);
+            }
+            
             // if no normals present add them.
             if (!geometry->getNormalArray() || geometry->getNormalArray()->getNumElements()==0)
             {
-                osgUtil::SmoothingVisitor tsv;
-                tsv.smooth(*geometry);
+                osgUtil::SmoothingVisitor sv;
+                sv.smooth(*geometry);
             }
 
 
@@ -608,6 +729,72 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, bool& ro
     return group;
 }
 
+ReaderWriterOBJ::ObjOptionsStruct ReaderWriterOBJ::parseOptions(const osgDB::ReaderWriter::Options* options) const
+{
+    ObjOptionsStruct localOptions;
+    localOptions.rotate = true;
+    localOptions.noTesselateLargePolygons = false;
+    localOptions.noTriStripPolygons = false;
+    localOptions.fixBlackMaterials = true;
+
+    if (options!=NULL)
+    {
+        std::istringstream iss(options->getOptionString());
+        std::string opt;
+        while (iss >> opt)
+        {
+            // split opt into pre= and post=
+            std::string pre_equals;
+            std::string post_equals;
+
+            size_t found = opt.find("=");
+            if(found!=std::string::npos)
+            {
+                pre_equals = opt.substr(0,found);
+                post_equals = opt.substr(found+1);
+            } 
+            else
+            {
+                pre_equals = opt;
+            }
+
+            if (pre_equals == "noRotation")
+            {
+                localOptions.rotate = false;
+            }                
+            else if (pre_equals == "noTesselateLargePolygons")
+            {
+                localOptions.noTesselateLargePolygons = true;
+            }                
+            else if (pre_equals == "noTriStripPolygons")
+            {
+                localOptions.noTriStripPolygons = true;
+            }
+            else if (post_equals.length()>0)
+            {    
+                obj::Material::Map::TextureMapType type = obj::Material::Map::UNKNOWN;                        
+                // Now we check to see if we have anything forcing a texture allocation
+                if        (pre_equals == "DIFFUSE")            type = obj::Material::Map::DIFFUSE;
+                else if (pre_equals == "AMBIENT")            type = obj::Material::Map::AMBIENT;
+                else if (pre_equals == "SPECULAR")            type = obj::Material::Map::SPECULAR;
+                else if (pre_equals == "SPECULAR_EXPONENT") type = obj::Material::Map::SPECULAR_EXPONENT;
+                else if (pre_equals == "OPACITY")            type = obj::Material::Map::OPACITY;
+                else if (pre_equals == "BUMP")                type = obj::Material::Map::BUMP;
+                else if (pre_equals == "DISPLACEMENT")        type = obj::Material::Map::DISPLACEMENT;
+                else if (pre_equals == "REFLECTION")        type = obj::Material::Map::REFLECTION;
+                
+                if (type!=obj::Material::Map::UNKNOWN)
+                {
+                    int unit = atoi(post_equals.c_str());    // (probably should use istringstream rather than atoi)
+                    localOptions.textureUnitAllocation.push_back(std::make_pair(unit,(obj::Material::Map::TextureMapType) type));
+                    osg::notify(osg::NOTICE)<<"Obj Found map in options, ["<<pre_equals<<"]="<<unit<<std::endl;
+                }
+            }
+        }
+    }
+    return localOptions;
+}
+
 
 // read file and convert to OSG.
 osgDB::ReaderWriter::ReadResult ReaderWriterOBJ::readNode(const std::string& file, const osgDB::ReaderWriter::Options* options) const
@@ -619,7 +806,7 @@ osgDB::ReaderWriter::ReadResult ReaderWriterOBJ::readNode(const std::string& fil
     if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
     
     
-    std::ifstream fin(fileName.c_str());
+    osgDB::ifstream fin(fileName.c_str());
     if (fin)
     {
     
@@ -631,14 +818,10 @@ osgDB::ReaderWriter::ReadResult ReaderWriterOBJ::readNode(const std::string& fil
         model.setDatabasePath(osgDB::getFilePath(fileName.c_str()));
         model.readOBJ(fin, local_opt.get());
         
-        // code for checking the nonRotation
-        bool rotate = true;
-        if ((options!=NULL) && (options->getOptionString() == "noRotation"))
-        {
-            rotate = false;
-        }
-
-        osg::Node* node = convertModelToSceneGraph(model,rotate);
+        ObjOptionsStruct localOptions = parseOptions(options);
+        
+        
+        osg::Node* node = convertModelToSceneGraph(model,localOptions);
         return node;
     }
     
@@ -652,14 +835,9 @@ osgDB::ReaderWriter::ReadResult ReaderWriterOBJ::readNode(std::istream& fin, con
         obj::Model model;
         model.readOBJ(fin, options);
         
-        // code for checking the nonRotation
-        bool rotate = true;
-        if ((options!=NULL) && (options->getOptionString() == "noRotation"))
-        {
-            rotate = false;
-        }
-
-        osg::Node* node = convertModelToSceneGraph(model,rotate);
+        ObjOptionsStruct localOptions = parseOptions(options);
+        
+        osg::Node* node = convertModelToSceneGraph(model, localOptions);
         return node;
     }
     
