@@ -12,7 +12,9 @@
 */
 
 #include <iostream>
+#include <sstream>
 #include <fstream>
+#include <string>
 #include <stdio.h>
 
 #include "obj.h"
@@ -22,8 +24,94 @@
 #include <osgDB/FileUtils>
 #include <osgDB/FileNameUtils>
 
+#include <string.h>
+
 using namespace obj;
 
+static std::string strip( const std::string& ss )
+{
+    std::string result;
+    result.assign( ss.begin() + ss.find_first_not_of( ' ' ), ss.begin() + 1 + ss.find_last_not_of( ' ' ) );
+    return( result );
+}
+
+/*
+ * parse a subset of texture options, following
+ * http://local.wasp.uwa.edu.au/~pbourke/dataformats/mtl/
+ */
+static Material::Map parseTextureMap( const std::string& ss, Material::Map::TextureMapType type)
+{
+    Material::Map map;
+    std::string s(ss);
+    for (;;)
+    {
+        if (s[0] != '-')
+            break;
+ 
+        int n;
+        if (s[1] == 's' || s[1] == 'o')
+        {
+            float x, y, z;
+            if (sscanf(s.c_str(), "%*s %f %f %f%n", &x, &y, &z, &n) != 3)
+            {
+                break;
+            }
+
+            if (s[1] == 's')
+            {
+                // texture scale
+                map.uScale = x;
+                map.vScale = y;
+            }
+            else if (s[1] == 'o')
+            {
+                // texture offset
+                map.uOffset = x;
+                map.vOffset = y;
+            }
+        }
+        else if (s.compare(1,2,"mm")==0)
+        {
+            // texture color offset and gain
+            float base, gain;
+            if (sscanf(s.c_str(), "%*s %f %f%n", &base, &gain, &n) != 2)
+            {
+                break;
+            }
+            // UNUSED
+        }
+        else if (s.compare(1,2,"bm")==0)
+        {
+            // blend multiplier
+            float mult;
+            if (sscanf(s.c_str(), "%*s %f%n", &mult, &n) != 2)
+            {
+                break;
+            }
+            // UNUSED
+        }
+        else if (s.compare(1,5,"clamp")==0)
+        {
+            osg::notify(osg::NOTICE)<<"Got Clamp\n";
+            char c[4];
+            if (sscanf(s.c_str(), "%*s %3s%n", c, &n) != 1)
+            {
+                break;
+            }
+            if(strncmp(c,"on",2)==0) map.clamp = true;
+            else map.clamp = false;    // default behavioud
+        }
+        else
+            break;
+
+        s = strip(s.substr(n));
+    }
+
+    map.name = s;
+    map.type = type;
+    return map;
+}
+
 bool Model::readline(std::istream& fin, char* line, const int LINE_SIZE)
 {
     if (LINE_SIZE<1) return false;
@@ -132,6 +220,7 @@ bool Model::readMTL(std::istream& fin)
     float r = 1.0f, g = 1.0f, b = 1.0f, a = 1.0f;
 
     Material* material = 0;// &(materialMap[""]);
+    std::string filename;
 
     while (fin)
     {
@@ -155,46 +244,136 @@ bool Model::readMTL(std::istream& fin)
                 {
                     unsigned int fieldsRead = sscanf(line+3,"%f %f %f %f", &r, &g, &b, &a);
 
-                    if (fieldsRead==1)      material->ambient.set(r,0.0f,0.0f,1.0f);
-                    else if (fieldsRead==2) material->ambient.set(r,g,0.0f,1.0f);
-                    else if (fieldsRead==3) material->ambient.set(r,g,b,1.0f);
-                    else if (fieldsRead==4) material->ambient.set(r,g,b,a);
+                    if (fieldsRead==1)
+                    {
+                        material->ambient[ 0 ] = r;
+                    }
+                    else if (fieldsRead==2)
+                    {
+                        material->ambient[ 0 ] = r;
+                        material->ambient[ 1 ] = g;
+                    }
+                    else if (fieldsRead==3)
+                    {
+                        material->ambient[ 0 ] = r;
+                        material->ambient[ 1 ] = g;
+                        material->ambient[ 2 ] = b;
+                    }
+                    else if (fieldsRead==4)
+                    {
+                        material->ambient[ 0 ] = r;
+                        material->ambient[ 1 ] = g;
+                        material->ambient[ 2 ] = b;
+                        material->ambient[ 3 ] = a;
+                    }
                 }
                 else if (strncmp(line,"Kd ",3)==0)
                 {
                     unsigned int fieldsRead = sscanf(line+3,"%f %f %f %f", &r, &g, &b, &a);
 
-                    if (fieldsRead==1)      material->diffuse.set(r,0.0f,0.0f,1.0f);
-                    else if (fieldsRead==2) material->diffuse.set(r,g,0.0f,1.0f);
-                    else if (fieldsRead==3) material->diffuse.set(r,g,b,1.0f);
-                    else if (fieldsRead==4) material->diffuse.set(r,g,b,a);
+                    if (fieldsRead==1)
+                    {
+                        material->diffuse[ 0 ] = r;
+                    }
+                    else if (fieldsRead==2)
+                    {
+                        material->diffuse[ 0 ] = r;
+                        material->diffuse[ 1 ] = g;
+                    }
+                    else if (fieldsRead==3)
+                    {
+                        material->diffuse[ 0 ] = r;
+                        material->diffuse[ 1 ] = g;
+                        material->diffuse[ 2 ] = b;
+                    }
+                    else if (fieldsRead==4)
+                    {
+                        material->diffuse[ 0 ] = r;
+                        material->diffuse[ 1 ] = g;
+                        material->diffuse[ 2 ] = b;
+                        material->diffuse[ 3 ] = a;
+                    }
                 }
                 else if (strncmp(line,"Ks ",3)==0)
                 {
                     unsigned int fieldsRead = sscanf(line+3,"%f %f %f %f", &r, &g, &b, &a);
 
-                    if (fieldsRead==1)      material->specular.set(r,0.0f,0.0f,1.0f);
-                    else if (fieldsRead==2) material->specular.set(r,g,0.0f,1.0f);
-                    else if (fieldsRead==3) material->specular.set(r,g,b,1.0f);
-                    else if (fieldsRead==4) material->specular.set(r,g,b,a);
+                    if (fieldsRead==1)
+                    {
+                        material->specular[ 0 ] = r;
+                    }
+                    else if (fieldsRead==2)
+                    {
+                        material->specular[ 0 ] = r;
+                        material->specular[ 1 ] = g;
+                    }
+                    else if (fieldsRead==3)
+                    {
+                        material->specular[ 0 ] = r;
+                        material->specular[ 1 ] = g;
+                        material->specular[ 2 ] = b;
+                    }
+                    else if (fieldsRead==4)
+                    {
+                        material->specular[ 0 ] = r;
+                        material->specular[ 1 ] = g;
+                        material->specular[ 2 ] = b;
+                        material->specular[ 3 ] = a;
+                    }
                 }
                 else if (strncmp(line,"Ke ",3)==0)
                 {
                     unsigned int fieldsRead = sscanf(line+3,"%f %f %f %f", &r, &g, &b, &a);
 
-                    if (fieldsRead==1)      material->emissive.set(r,0.0f,0.0f,1.0f);
-                    else if (fieldsRead==2) material->emissive.set(r,g,0.0f,1.0f);
-                    else if (fieldsRead==3) material->emissive.set(r,g,b,1.0f);
-                    else if (fieldsRead==4) material->emissive.set(r,g,b,a);
+                    if (fieldsRead==1)
+                    {
+                        material->emissive[ 0 ] = r;
+                    }
+                    else if (fieldsRead==2)
+                    {
+                        material->emissive[ 0 ] = r;
+                        material->emissive[ 1 ] = g;
+                    }
+                    else if (fieldsRead==3)
+                    {
+                        material->emissive[ 0 ] = r;
+                        material->emissive[ 1 ] = g;
+                        material->emissive[ 2 ] = b;
+                    }
+                    else if (fieldsRead==4)
+                    {
+                        material->emissive[ 0 ] = r;
+                        material->emissive[ 1 ] = g;
+                        material->emissive[ 2 ] = b;
+                        material->emissive[ 3 ] = a;
+                    }
                 }
                 else if (strncmp(line,"Tf ",3)==0)
                 {
                     unsigned int fieldsRead = sscanf(line+3,"%f %f %f %f", &r, &g, &b, &a);
 
-                    if (fieldsRead==1)      material->Tf.set(r,0.0f,0.0f,1.0f);
-                    else if (fieldsRead==2) material->Tf.set(r,g,0.0f,1.0f);
-                    else if (fieldsRead==3) material->Tf.set(r,g,b,1.0f);
-                    else if (fieldsRead==4) material->Tf.set(r,g,b,a);
+                    if (fieldsRead==1)
+                    {
+                        material->Tf[ 0 ] = r;
+                    }
+                    else if (fieldsRead==2)
+                    {
+                        material->Tf[ 0 ] = r;
+                        material->Tf[ 1 ] = g;
+                    }
+                    else if (fieldsRead==3)
+                    {
+                        material->Tf[ 0 ] = r;
+                        material->Tf[ 1 ] = g;
+                        material->Tf[ 2 ] = b;
+                    }
+                    else if (fieldsRead==4)
+                    {
+                        material->Tf[ 0 ] = r;
+                        material->Tf[ 1 ] = g;
+                        material->Tf[ 2 ] = b;
+                        material->Tf[ 3 ] = a;
+                    }
                 }
                 else if (strncmp(line,"sharpness ",10)==0)
                 {
@@ -252,20 +431,58 @@ bool Model::readMTL(std::istream& fin)
                 }
                 else if (strncmp(line,"map_Ka ",7)==0)
                 {
-                    std::string filename(line+7);
-                    material->map_Ka = filename;
+                    material->maps.push_back(parseTextureMap(strip(line+7),Material::Map::AMBIENT));
                 }
+                // diffuse map
                 else if (strncmp(line,"map_Kd ",7)==0)
                 {
-                    material->map_Kd = lastComponent(line+7);
+                    material->maps.push_back(parseTextureMap(strip(line+7),Material::Map::DIFFUSE));
                 }
+                // specular colour/level map
                 else if (strncmp(line,"map_Ks ",7)==0)
                 {
-                    material->map_Ks = lastComponent(line+7);
+                     material->maps.push_back(parseTextureMap(strip(line+7),Material::Map::SPECULAR));
                 }
-                else if (strcmp(line,"refl")==0 || strncmp(line,"refl ",5)==0)
+                // map_opacity doesn't exist in the spec, but was already in the plugin
+                // so leave it or plugin will break for some users
+                else if (strncmp(line,"map_opacity ",12)==0) 
                 {
-                    material->textureReflection = true;
+                    material->maps.push_back(parseTextureMap(strip(line+12),Material::Map::OPACITY));
+                }
+                // proper dissolve/opacity map
+                else if (strncmp(line,"map_d ",6)==0)
+                {
+                    material->maps.push_back(parseTextureMap(strip(line+6),Material::Map::OPACITY));
+                }
+                // specular exponent map
+                else if (strncmp(line,"map_Ns ",7)==0)
+                {
+                    material->maps.push_back(parseTextureMap(strip(line+7),Material::Map::SPECULAR_EXPONENT));
+                }
+                // modelling tools and convertors variously produce bump, map_bump, and map_Bump so parse them all
+                else if (strncmp(line,"bump ",5)==0)
+                {
+                    material->maps.push_back(parseTextureMap(strip(line+5),Material::Map::BUMP));
+                }
+                else if (strncmp(line,"map_bump ",9)==0)
+                {
+                    material->maps.push_back(parseTextureMap(strip(line+9),Material::Map::BUMP));
+                }
+                else if (strncmp(line,"map_Bump ",9)==0)
+                {
+                    material->maps.push_back(parseTextureMap(strip(line+9),Material::Map::BUMP));
+                }
+                // displacement map
+                else if (strncmp(line,"disp ",5)==0)
+                {
+                    material->maps.push_back(parseTextureMap(strip(line+5),Material::Map::DISPLACEMENT));
+                }
+                // reflection map (the original code had the possibility of a blank "refl" line
+                // which isn't correct according to the spec, so this bit might break for some
+                // modelling packages...
+                else if (strncmp(line,"refl ",5)==0)
+                {
+                    material->maps.push_back(parseTextureMap(strip(line+5),Material::Map::REFLECTION));
                 }
                 else
                 {
@@ -431,7 +648,7 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
                 osg::notify(osg::INFO) << "--" << fullPathFileName << "--" << std::endl;
                 if (!fullPathFileName.empty())
                 {
-                    std::ifstream mfin( fullPathFileName.c_str() );
+                    osgDB::ifstream mfin( fullPathFileName.c_str() );
                     if (mfin)
                     {
                         readMTL(mfin);
