@@ -14,7 +14,7 @@ bool LightModel_writeLocalData(const Object& obj, Output& fw);
 
 
 // register the read and write functions with the osgDB::Registry.
-RegisterDotOsgWrapperProxy g_LightModelProxy
+REGISTER_DOTOSGWRAPPER(LightModel)
 (
     new osg::LightModel,
     "LightModel",
