@@ -13,7 +13,7 @@ bool PositionAttitudeTransform_readLocalData(Object& obj, Input& fr);
 bool PositionAttitudeTransform_writeLocalData(const Object& obj, Output& fw);
 
 // register the read and write functions with the osgDB::Registry.
-RegisterDotOsgWrapperProxy g_PositionAttitudeTransformProxy
+REGISTER_DOTOSGWRAPPER(PositionAttitudeTransform)
 (
     new osg::PositionAttitudeTransform,
     "PositionAttitudeTransform",
