@@ -4,6 +4,8 @@
 #include "osgDB/Input"
 #include "osgDB/Output"
 
+#include <string.h>
+
 using namespace osg;
 using namespace osgDB;
 
@@ -12,7 +14,7 @@ bool Sequence_readLocalData(Object& obj, Input& fr);
 bool Sequence_writeLocalData(const Object& obj, Output& fw);
 
 // register the read and write functions with the osgDB::Registry.
-RegisterDotOsgWrapperProxy g_SequenceProxy
+REGISTER_DOTOSGWRAPPER(Sequence)
 (
     new osg::Sequence,
     "Sequence",
