@@ -7,7 +7,11 @@
 #include <osgDB/Input>
 #include <osgDB/Output>
 
+#include <OpenThreads/Mutex>
+#include <OpenThreads/ScopedLock>
+
 #include <set>
+#include <string.h>
 
 using namespace osg;
 using namespace osgDB;
@@ -26,7 +30,7 @@ bool StateSet_matchRenderBinModeStr(const char* str,StateSet::RenderBinMode& mod
 const char* StateSet_getRenderBinModeStr(StateSet::RenderBinMode mode);
 
 // register the read and write functions with the osgDB::Registry.
-RegisterDotOsgWrapperProxy g_StateSetFuncProxy
+REGISTER_DOTOSGWRAPPER(StateSet)
 (
     new osg::StateSet,
     "StateSet",
@@ -37,7 +41,7 @@ RegisterDotOsgWrapperProxy g_StateSetFuncProxy
 );
 
 // register the read and write functions with the osgDB::Registry.
-RegisterDotOsgWrapperProxy g_GeoStateFuncProxy
+REGISTER_DOTOSGWRAPPER(GeoState)
 (
     new osg::StateSet,
     "GeoState",
@@ -64,6 +68,11 @@ void initGLNames()
 {
     static bool first_time = true;
     if (!first_time) return;
+
+    static OpenThreads::Mutex s_initGLNames;
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_initGLNames);
+
+    if (!first_time) return;
     
     ADD_NAME("GL_ALPHA_TEST",GL_ALPHA_TEST)
     ADD_NAME("GL_BLEND",GL_BLEND)
@@ -333,6 +342,28 @@ bool StateSet_readLocalData(Object& obj, Input& fr)
         stateset.setRenderBinDetails(binNumber,binName,rbmode);
     }
 
+    static ref_ptr<StateSet::Callback> s_callback = new osg::StateSet::Callback;
+    while (fr.matchSequence("UpdateCallback {"))
+    {
+        // int entry = fr[0].getNoNestedBrackets();
+        fr += 2;
+        StateSet::Callback* callback = dynamic_cast<StateSet::Callback*>(fr.readObjectOfType(*s_callback));
+        if (callback) {
+            stateset.setUpdateCallback(callback);
+        }
+        iteratorAdvanced = true;
+    }
+
+    while (fr.matchSequence("EventCallback {"))
+    {
+        //int entry = fr[0].getNoNestedBrackets();
+        fr += 2;
+        StateSet::Callback* callback = dynamic_cast<StateSet::Callback*>(fr.readObjectOfType(*s_callback));
+        if (callback) {
+            stateset.setEventCallback(callback);
+        }
+        iteratorAdvanced = true;
+    }
 
     bool readingMode = true;
     StateAttribute::GLModeValue value;
@@ -599,6 +630,24 @@ bool StateSet_writeLocalData(const Object& obj, Output& fw)
         fw.moveOut();
         fw.indent()<<"}"<< std::endl;
     }
+    
+    if (stateset.getUpdateCallback())
+    {
+        fw.indent() << "UpdateCallback {" << std::endl;
+        fw.moveIn();
+        fw.writeObject(*stateset.getUpdateCallback());
+        fw.moveOut();
+        fw.indent() << "}" << std::endl;
+    }
+
+    if (stateset.getEventCallback())
+    {
+        fw.indent() << "EventCallback {" << std::endl;
+        fw.moveIn();
+        fw.writeObject(*stateset.getEventCallback());
+        fw.moveOut();
+        fw.indent() << "}" << std::endl;
+    }
 
     return true;
 }
