@@ -24,17 +24,17 @@ osgDB::RegisterDotOsgWrapperProxy ShadowTechnique_Proxy
     ShadowTechnique_writeLocalData
 );
 
-bool ShadowTechnique_readLocalData(osg::Object& obj, osgDB::Input &fr)
+bool ShadowTechnique_readLocalData(osg::Object& /*obj*/, osgDB::Input& /*fr*/)
 {
-    osgShadow::ShadowTechnique& ss = static_cast<osgShadow::ShadowTechnique&>(obj);
+    //osgShadow::ShadowTechnique& ss = static_cast<osgShadow::ShadowTechnique&>(obj);
     bool itAdvanced = false;
 
     return itAdvanced;
 }
 
-bool ShadowTechnique_writeLocalData(const osg::Object& obj, osgDB::Output& fw)
+bool ShadowTechnique_writeLocalData(const osg::Object& /*obj*/, osgDB::Output& /*fw*/)
 {
-    const osgShadow::ShadowTechnique& ss = static_cast<const osgShadow::ShadowTechnique &>(obj);
+    //const osgShadow::ShadowTechnique& ss = static_cast<const osgShadow::ShadowTechnique &>(obj);
 
     return true;
 }
