@@ -19,15 +19,12 @@ class ReaderWriterTerrain : public osgDB::ReaderWriter
 
         ReaderWriterTerrain()
         {
+            supportsExtension("osgTerrain","OpenSceneGraph terrain extension to .osg ascii format");
+            supportsExtension("terrain","OpenSceneGraph terrain ascii format");
         }
         
         virtual const char* className() const { return "Terrain ReaderWriter"; }
 
-        virtual bool acceptsExtension(const std::string& extension) const
-        {
-            return osgDB::equalCaseInsensitive( extension, "osgTerrain" ) || osgDB::equalCaseInsensitive(extension,"terrain");
-        }
-
         virtual osgDB::ReaderWriter::ReadResult readNode(const std::string& file, const osgDB::ReaderWriter::Options* opt) const
         {
             std::string ext = osgDB::getLowerCaseFileExtension(file);
@@ -60,7 +57,7 @@ class ReaderWriterTerrain : public osgDB::ReaderWriter
             osg::ref_ptr<Options> local_opt = opt ? static_cast<Options*>(opt->clone(osg::CopyOp::SHALLOW_COPY)) : new Options;
             local_opt->setDatabasePath(osgDB::getFilePath(fileName));
 
-            std::ifstream fin(fileName.c_str());
+            osgDB::ifstream fin(fileName.c_str());
             if (fin)
             {
                 return readNode(fin, local_opt.get());
