@@ -168,6 +168,8 @@ static rawImageRec *RawImageOpen(std::istream& fin)
     }
 
     fin.read((char*)raw,12);
+    if (!fin.good())
+        return NULL;
 
     if (raw->swapFlag)
     {
@@ -441,25 +443,26 @@ static void RawImageGetData(rawImageRec *raw, unsigned char **data )
 class ReaderWriterRGB : public osgDB::ReaderWriter
 {
     public:
+    
+        ReaderWriterRGB()
+        {
+            supportsExtension("rgb","rgb image format");
+            supportsExtension("rgba","rgba image format");
+            supportsExtension("sgi","sgi image format");
+            supportsExtension("int","int image format");
+            supportsExtension("inta","inta image format");
+            supportsExtension("bw","bw image format");
+        }
+    
         virtual const char* className() const { return "RGB Image Reader/Writer"; }
         
-        virtual bool acceptsExtension(const std::string& extension) const
-        {
-            return osgDB::equalCaseInsensitive(extension,"rgb") ||
-                osgDB::equalCaseInsensitive(extension,"sgi") ||
-                osgDB::equalCaseInsensitive(extension,"rgba") ||
-                osgDB::equalCaseInsensitive(extension,"int") || 
-                osgDB::equalCaseInsensitive(extension,"inta") ||
-                osgDB::equalCaseInsensitive(extension,"bw");
-        }
-
         ReadResult readRGBStream(std::istream& fin) const
         {
             rawImageRec *raw;
 
             if( (raw = RawImageOpen(fin)) == NULL )
             {
-                return ReadResult::FILE_NOT_HANDLED;
+                return ReadResult::ERROR_IN_READING_FILE;
             }
 
             int s = raw->sizeX;
@@ -520,7 +523,7 @@ class ReaderWriterRGB : public osgDB::ReaderWriter
             std::string fileName = osgDB::findDataFile( file, options );
             if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
 
-            std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
+            osgDB::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
             if(!istream) return ReadResult::FILE_NOT_HANDLED;
             ReadResult rr = readRGBStream(istream);
             if(rr.validImage()) rr.getImage()->setFileName(file);
@@ -652,7 +655,7 @@ class ReaderWriterRGB : public osgDB::ReaderWriter
             std::string ext = osgDB::getFileExtension(fileName);
             if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
 
-            std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
+            osgDB::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
             if(!fout) return WriteResult::ERROR_IN_WRITING_FILE;
 
             return writeRGBStream(img,fout,fileName);
