@@ -2,14 +2,28 @@
 #if defined(_MSC_VER) || defined(__MINGW32__)
     #include <stdio.h>
     #include <io.h>
+    
+    namespace esri
+    {
+        int read(int fd, void * buf, size_t nbytes) { return _read(fd, buf, nbytes); }
+    }
+    
 #else
     #include <unistd.h>
+
+    namespace esri
+    {
+        int read(int fd, void * buf, size_t nbytes) { return ::read(fd, buf, nbytes); }
+    }
+
 #endif 
 
 #include "ESRIShape.h"
 
 using namespace ESRIShape ;
 
+#define SAFE_DELETE_ARRAY( ptr ) delete[] ptr; ptr = 0L;
+
 template <class T>
 inline void swapBytes(  T &s )
 {
@@ -37,7 +51,7 @@ template <class T>
 inline bool readVal( int fd, T &val, ByteOrder bo = LittleEndian )
 {
     int nbytes = 0;
-    if( (nbytes = ::read( fd, &val, sizeof(T))) <= 0 ) 
+    if( (nbytes = esri::read( fd, &val, sizeof(T))) <= 0 ) 
         return false;
 
     if( getByteOrder() != bo )
@@ -96,7 +110,7 @@ void BoundingBox::print()
 bool ShapeHeader::read(int fd)
 {
     if( readVal<Integer>( fd, fileCode, BigEndian ) == false ) return false;
-    if( ::read( fd, _unused_0, sizeof(_unused_0)) <= 0 ) return false;
+    if( esri::read( fd, _unused_0, sizeof(_unused_0)) <= 0 ) return false;
     if( readVal<Integer>( fd, fileLength, BigEndian ) == false ) return false;
     if( readVal<Integer>( fd, version, LittleEndian ) == false ) return false;
     if( readVal<Integer>( fd, shapeType, LittleEndian ) == false ) return false;
@@ -243,7 +257,7 @@ MultiPoint::MultiPoint( const struct MultiPoint &mpoint ): ShapeObject(ShapeType
 
 MultiPoint::~MultiPoint()
 {
-    delete [] points;
+    delete[] points;
 }
 
 bool MultiPoint::read( int fd )
@@ -251,6 +265,8 @@ bool MultiPoint::read( int fd )
     RecordHeader rh;
     if( rh.read(fd) == false )
         return false;
+    
+    SAFE_DELETE_ARRAY( points );
 
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
@@ -317,6 +333,9 @@ bool PolyLine::read( int fd )
     if( rh.read(fd) == false )
         return false;
 
+    SAFE_DELETE_ARRAY( parts );
+    SAFE_DELETE_ARRAY( points );
+
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
         return false;
@@ -386,6 +405,9 @@ bool Polygon::read( int fd )
     if( rh.read(fd) == false )
         return false;
 
+    SAFE_DELETE_ARRAY( parts );
+    SAFE_DELETE_ARRAY( points );
+
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
         return false;
@@ -501,6 +523,9 @@ bool MultiPointM::read( int fd )
     if( rh.read(fd) == false )
         return false;
 
+    SAFE_DELETE_ARRAY( points );
+    SAFE_DELETE_ARRAY( mArray );
+
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
         return false;
@@ -590,6 +615,10 @@ bool PolyLineM::read( int fd )
     if( rh.read(fd) == false )
         return false;
 
+    SAFE_DELETE_ARRAY( parts );
+    SAFE_DELETE_ARRAY( points );
+    SAFE_DELETE_ARRAY( mArray );
+
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
         return false;
@@ -670,12 +699,24 @@ PolygonM::PolygonM(const PolygonM &p):
     }
 }
 
+PolygonM::~PolygonM()
+{
+    delete[] parts;
+    delete[] points;
+    delete[] mArray;
+}
+
+
 bool PolygonM::read( int fd )
 {
     RecordHeader rh;
     if( rh.read(fd) == false )
         return false;
 
+    SAFE_DELETE_ARRAY( parts );
+    SAFE_DELETE_ARRAY( points );
+    SAFE_DELETE_ARRAY( mArray );
+
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
         return false;
@@ -820,6 +861,10 @@ bool MultiPointZ::read( int fd )
     if( rh.read(fd) == false )
         return false;
 
+    SAFE_DELETE_ARRAY( points );
+    SAFE_DELETE_ARRAY( zArray );
+    SAFE_DELETE_ARRAY( mArray );
+
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
         return false;
@@ -923,8 +968,7 @@ PolyLineZ::~PolyLineZ()
     delete [] parts;
     delete [] points;
     delete [] zArray;
-    if( mArray != 0L )
-        delete [] mArray;
+    delete [] mArray;
 }
 
 bool PolyLineZ::read( int fd )
@@ -933,6 +977,11 @@ bool PolyLineZ::read( int fd )
     if( rh.read(fd) == false )
         return false;
 
+    SAFE_DELETE_ARRAY( parts );
+    SAFE_DELETE_ARRAY( points );
+    SAFE_DELETE_ARRAY( zArray );
+    SAFE_DELETE_ARRAY( mArray );
+
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
         return false;
@@ -976,7 +1025,7 @@ bool PolyLineZ::read( int fd )
     int Y = X + (15 * numPoints);
     int Z = Y + 16 + (8 * numPoints);
 
-    if( rh.contentLength > Z )
+    if( rh.contentLength != Z )
     { 
         mRange.read(fd);
         mArray = new Double[numPoints];
@@ -996,7 +1045,8 @@ PolygonZ::PolygonZ():
     numParts(0), 
     numPoints(0), 
     parts(0L), 
-    points(0L) ,
+    points(0L),
+    zArray(0L),
     mArray(0L)
 {}
 
@@ -1020,6 +1070,8 @@ PolygonZ::PolygonZ(const PolygonZ &p):
     {
         points[i] = p.points[i];
         zArray[i] = p.zArray[i]; // jcm 
+    // M-Array seems to be missing sometimes
+    if(p.mArray)     
         mArray[i] = p.mArray[i];
     }
 }
@@ -1029,8 +1081,7 @@ PolygonZ::~PolygonZ()
     delete [] parts;
     delete [] points;
     delete [] zArray;
-    if( mArray != 0L )
-        delete [] mArray;
+    delete [] mArray;
 }
 
 bool PolygonZ::read( int fd )
@@ -1038,6 +1089,11 @@ bool PolygonZ::read( int fd )
     RecordHeader rh;
     if( rh.read(fd) == false )
         return false;
+    
+    SAFE_DELETE_ARRAY( parts );
+    SAFE_DELETE_ARRAY( points );
+    SAFE_DELETE_ARRAY( zArray );
+    SAFE_DELETE_ARRAY( mArray );
 
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
@@ -1082,7 +1138,7 @@ bool PolygonZ::read( int fd )
     int  X = 44 + (4*numParts);
     int  Y = X + (16*numPoints);
     int  Z = Y + 16 + (8*numPoints);
-    if( rh.contentLength > Z )
+    if( rh.contentLength != Z )
     {
         if( mRange.read(fd) == false )
             return false;
@@ -1165,8 +1221,7 @@ MultiPatch::~MultiPatch()
     delete [] partTypes;
     delete [] points;
     delete [] zArray;
-    if( mArray != 0L )
-        delete [] mArray;
+    delete [] mArray;
 }
 
 bool MultiPatch::read( int fd )
@@ -1175,6 +1230,12 @@ bool MultiPatch::read( int fd )
     if( rh.read(fd) == false )
         return false;
 
+    SAFE_DELETE_ARRAY( parts );
+    SAFE_DELETE_ARRAY( partTypes );
+    SAFE_DELETE_ARRAY( points );
+    SAFE_DELETE_ARRAY( zArray );
+    SAFE_DELETE_ARRAY( mArray );
+
     Integer shapeType;
     if( readVal<Integer>(fd, shapeType, LittleEndian ) == false )
         return false;
