@@ -12,8 +12,9 @@
 
 using namespace ESRIShape;
 
-ESRIShapeParser::ESRIShapeParser( const std::string fileName ):
-    _valid(false)
+ESRIShapeParser::ESRIShapeParser( const std::string fileName, bool useDouble ):
+    _valid(false),
+    _useDouble(useDouble)
 {
     int fd = 0;
     if( !fileName.empty() )
@@ -140,9 +141,9 @@ ESRIShapeParser::ESRIShapeParser( const std::string fileName ):
         case ESRIShape::ShapeTypePointZ      :
             {
                 std::vector<ESRIShape::PointZ> ptzs;
-                ESRIShape::PointZRecord pointZRecord;
-                while( pointZRecord.read(fd) )
-                    ptzs.push_back( pointZRecord.pointZ );
+                ESRIShape::PointZ pointZ;
+                while( pointZ.read( fd ) )
+                    ptzs.push_back( pointZ );
                 _process( ptzs );
             }
             break;
@@ -199,6 +200,12 @@ ESRIShapeParser::ESRIShapeParser( const std::string fileName ):
         default:
             break;
     }
+
+    if(fd)
+    {
+      close(fd);
+      fd = 0;
+    }
 }
 
 osg::Geode *ESRIShapeParser::getGeode() 
@@ -209,8 +216,11 @@ osg::Geode *ESRIShapeParser::getGeode()
 void ESRIShapeParser::_combinePointToMultipoint()
 {
     if( !_valid ) return;
+    
+    osg::notify(osg::NOTICE)<<"_combinePointToMultipoint()"<<std::endl;
+
+    ArrayHelper coords(_useDouble);
 
-    osg::ref_ptr<osg::Vec3Array> coords  = new osg::Vec3Array;
     unsigned int numDrawables = _geode->getNumDrawables();
 
     for( unsigned int i = 0; i < numDrawables; i++ )
@@ -218,9 +228,7 @@ void ESRIShapeParser::_combinePointToMultipoint()
         osg::Geometry *geom = dynamic_cast<osg::Geometry *>(_geode->getDrawable(i));
         if( geom != 0L )
         {
-            osg::Vec3Array *va = dynamic_cast<osg::Vec3Array *>(geom->getVertexArray());
-            if( va != 0L )
-                coords->push_back( va->front() );
+            coords.add( geom->getVertexArray(), 0 );
         }
     }
 
@@ -228,7 +236,7 @@ void ESRIShapeParser::_combinePointToMultipoint()
 
     osg::ref_ptr<osg::Geometry> geometry = new osg::Geometry;
     geometry->setVertexArray(coords.get());
-    geometry->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, coords->size()));
+    geometry->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, coords.size()));
     _geode->addDrawable( geometry.get() );
 }
 
@@ -239,8 +247,9 @@ void ESRIShapeParser::_process( const std::vector<ESRIShape::Point> &pts )
     std::vector<ESRIShape::Point>::const_iterator p;
     for( p = pts.begin(); p != pts.end(); p++ )
     {
-        osg::ref_ptr<osg::Vec3Array> coords  = new osg::Vec3Array;
-        coords->push_back( osg::Vec3( p->x, p->y, 0.0 ));
+        ArrayHelper coords(_useDouble);
+
+        coords.add( p->x, p->y, 0.0 );
         osg::ref_ptr<osg::Geometry> geometry = new osg::Geometry;
         geometry->setVertexArray(coords.get());
         geometry->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, 1));
@@ -258,13 +267,14 @@ void ESRIShapeParser::_process( const std::vector<ESRIShape::MultiPoint> &mpts )
     std::vector<ESRIShape::MultiPoint>::const_iterator p;
     for( p = mpts.begin(); p != mpts.end(); p++ )
     {
-        osg::ref_ptr<osg::Vec3Array> coords  = new osg::Vec3Array;
+        ArrayHelper coords(_useDouble);
+
         for( int i = 0; i < p->numPoints ; i++ )
-            coords->push_back( osg::Vec3( p->points[i].x, p->points[i].y, 0.0 ));
+            coords.add( p->points[i].x, p->points[i].y, 0.0 );
 
         osg::ref_ptr<osg::Geometry> geometry = new osg::Geometry;
         geometry->setVertexArray(coords.get());
-        geometry->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, coords->size()));
+        geometry->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, coords.size()));
 
         _geode->addDrawable( geometry.get() );
     }
@@ -277,11 +287,11 @@ void ESRIShapeParser::_process(const std::vector<ESRIShape::PolyLine> &lines )
     std::vector<ESRIShape::PolyLine>::const_iterator p;
     for( p = lines.begin(); p != lines.end(); p++ )
     {
-        osg::ref_ptr<osg::Vec3Array> coords  = new osg::Vec3Array;
+        ArrayHelper coords(_useDouble);
 
         int i;
         for( i = 0; i < p->numPoints; i++ )
-            coords->push_back( osg::Vec3( p->points[i].x, p->points[i].y, 0.0 ));
+            coords.add( p->points[i].x, p->points[i].y, 0.0 );
 
         osg::ref_ptr<osg::Geometry> geometry = new osg::Geometry;
         geometry->setVertexArray(coords.get());
@@ -307,16 +317,14 @@ void ESRIShapeParser::_process( const std::vector<ESRIShape::Polygon> &polys )
     std::vector<ESRIShape::Polygon>::const_iterator p;
     for( p = polys.begin(); p != polys.end(); p++ )
     {
-        osg::ref_ptr<osg::Vec3Array> coords  = new osg::Vec3Array;
+        ArrayHelper coords(_useDouble);
         int i;
         for( i = 0; i < p->numPoints; i++ )
-            coords->push_back( osg::Vec3( p->points[i].x, p->points[i].y, 0.0 ));
+            coords.add( p->points[i].x, p->points[i].y, 0.0 );
 
         osg::ref_ptr<osg::Geometry> geometry = new osg::Geometry;
         geometry->setVertexArray(coords.get());
 
-
-
         for( i = 0; i < p->numParts; i++ )
         {
             int index = p->parts[i];
@@ -347,8 +355,8 @@ void ESRIShapeParser::_process( const std::vector<ESRIShape::PointM> &ptms )
     std::vector<ESRIShape::PointM>::const_iterator p;
     for( p = ptms.begin(); p != ptms.end(); p++ )
     {
-        osg::ref_ptr<osg::Vec3Array> coords  = new osg::Vec3Array;
-        coords->push_back( osg::Vec3( p->x, p->y, 0.0 ));
+        ArrayHelper coords(_useDouble);
+        coords.add( p->x, p->y, 0.0 );
         osg::ref_ptr<osg::Geometry> geometry = new osg::Geometry;
         geometry->setVertexArray(coords.get());
         geometry->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, 1));
