@@ -7,8 +7,11 @@
 #include <osgDB/Registry>
 #include <osgDB/FileUtils>
 #include <osg/io_utils>
+
 #include <iostream>
 #include <sstream>
+#include <stdlib.h>
+#include <string.h>
 
 #include "ReaderWriterTXP.h"
 #include "TXPNode.h"
@@ -49,21 +52,25 @@ osgDB::ReaderWriter::ReadResult ReaderWriterTXP::local_readNode(const std::strin
             txpNode->setOptions(options->getOptionString());
         }
         
-        if (txpNode->loadArchive())
-        {
-            TXPArchive* archive = txpNode->getArchive();
-            if (archive) 
-            {
-                if (options && options->getOptionString().find("loadMaterialsToStateSet")!=std::string::npos)
-                {
-                   archive->SetMaterialAttributesToStateSetVar(true);
-                }
 
-                int id = _archiveId++;
-                archive->setId(id);
-//                txpNode->setArchive(getArchive(id,osgDB::getFilePath(fileName)));
-                getArchive(id,osgDB::getFilePath(fileName));
+        //modified by Brad Anderegg on May-27-08
+        //calling getArchive will create a new TXPArchive if the specified one does not exist
+        //we will set our osgdb loader options on the archive and set the appropriate archive on 
+        //the txpNode.
+        int id = ++_archiveId;
+        TXPArchive* archive = getArchive(id,osgDB::getFilePath(fileName));
+
+        if (archive != NULL)
+        {
+            archive->setId(id);
+
+            if (options && options->getOptionString().find("loadMaterialsToStateSet")!=std::string::npos)
+            {
+               archive->SetMaterialAttributesToStateSetVar(true);
             }
+
+            txpNode->loadArchive(archive);
+            
             return txpNode.get();
         }
         else
@@ -532,14 +539,14 @@ bool ReaderWriterTXP::extractChildrenLocations(const std::string& name, int pare
         token = strtok(0, "_");
         if(!token)
             break;
-        locs[idx].zmin = (float)atof(token);
+        locs[idx].zmin = osg::asciiToFloat(token);
         nbTokenRead++;
 
         // ZMAX
         token = strtok(0, "_");
         if(!token)
             break;
-        locs[idx].zmax= (float)atof(token);
+        locs[idx].zmax = osg::asciiToFloat(token);
         nbTokenRead++;
 
         locs[idx].lod = parentLod+1;
@@ -645,6 +652,8 @@ public:
 
 protected:
     osg::Node* seamReplacement(osg::Node* node);
+    
+    SeamFinder& operator = (const SeamFinder&) { return *this; }
 
     int _x, _y, _lod;
     const TXPArchive::TileInfo& _info;
