@@ -16,6 +16,7 @@
 #include <osgDB/FileUtils>
 #include <osgDB/FileUtils>
 #include <osgDB/FileNameUtils>
+#include <osgDB/fstream>
 #include <osgDB/ReadFile>
 #include <osgDB/WriteFile>
 #include <osgSim/Sector>
@@ -23,7 +24,6 @@
 #include <osgSim/LightPointNode>
 #include <osgSim/BlinkSequence>
 #include <iostream>
-#include <fstream>
 
 #if defined(linux)
 #  include <unistd.h>
@@ -391,10 +391,10 @@ bool TXPArchive::loadTexture(int i)
             path += _PATHD ;
         
         std::string theFile = path + filename ;
-        osg::Image* image = osgDB::readImageFile(theFile);
-        if (image)
+        osg::ref_ptr<osg::Image> image = osgDB::readRefImageFile(theFile);
+        if (image.valid())
         {
-            osg_texture->setImage(image);
+            osg_texture->setImage(image.get());
         }
         else
         {
@@ -458,7 +458,7 @@ bool TXPArchive::loadModel(int ix)
 
 bool TXPArchive::loadModels()
 {
-    osg::notify(osg::NOTICE) << "txp:: Loading models ..." << std::endl;
+    osg::notify(osg::INFO) << "txp:: Loading models ..." << std::endl;
 
     int numModel;
     modelTable.GetNumModels(numModel);
@@ -470,13 +470,13 @@ bool TXPArchive::loadModels()
     {
     loadModel(itr->first);
     }
-    osg::notify(osg::NOTICE) << "txp:: ... done." << std::endl;
+    osg::notify(osg::INFO) << "txp:: ... done." << std::endl;
     return true;
 }
 
 bool TXPArchive::loadLightAttributes()
 {
-    osg::notify(osg::NOTICE) << "txp:: Loading light attributes ..." << std::endl;
+    osg::notify(osg::INFO) << "txp:: Loading light attributes ..." << std::endl;
 
     trpgLightTable::LightMapType *lm  = lightTable.getLightMap();
     trpgLightTable::LightMapType::iterator itr = lm->begin();
@@ -576,7 +576,7 @@ bool TXPArchive::loadLightAttributes()
     addLightAttribute(osgLight, stateSet, osg::Vec3(normal.x,normal.y,normal.z),itr->first);
     }
 
-    osg::notify(osg::NOTICE) << "txp:: ... done." << std::endl;
+    osg::notify(osg::INFO) << "txp:: ... done." << std::endl;
     return true;
 }
 
@@ -599,12 +599,12 @@ bool TXPArchive::loadTextStyles()
     std::map< std::string, std::string > fontmap;
 
     std::string fmapfname = std::string(getDir())+"\\fontmap.txt";
-    std::ifstream fmapfile;
+    osgDB::ifstream fmapfile;
     fmapfile.open(fmapfname.c_str(),std::ios::in);
 
     if (fmapfile.is_open())
     {
-    osg::notify(osg::NOTICE) << "txp:: Font map file found: " << fmapfname << std::endl;
+    osg::notify(osg::INFO) << "txp:: Font map file found: " << fmapfname << std::endl;
     std::string line;
     while (std::getline(fmapfile,line))
     {
