@@ -8,6 +8,7 @@
 #include <iostream>
 #include <vector>
 #include <algorithm>
+#include <stdio.h>
 
 #include "TileMapper.h"
 #include "TXPNode.h"
@@ -26,7 +27,7 @@ class RetestCallback : public osg::NodeCallback
 public:
     RetestCallback()
     {
-	timer = osg::Timer::instance(); // get static timer
+        timer = osg::Timer::instance(); // get static timer
         prevTime = 0; // should this be instantiated with current time?
     }
 
@@ -38,7 +39,7 @@ public:
             (n = (osg::Group *) pLOD->getChild(0)) &&
             (n->getNumChildren() == 0))
         {
-	    osg::Timer_t curTime = timer->tick();
+            osg::Timer_t curTime = timer->tick();
             if ((prevTime + 2.0/timer->getSecondsPerTick() ) < curTime)
             {
                 prevTime = curTime;
@@ -64,7 +65,7 @@ _originX(0.0),
 _originY(0.0)
 {
     setNumChildrenRequiringUpdateTraversal(1);
-	setCullingActive(false);
+    setCullingActive(false);
 }
             
 TXPNode::TXPNode(const TXPNode& txpNode,const osg::CopyOp& copyop):
@@ -90,6 +91,8 @@ void TXPNode::traverse(osg::NodeVisitor& nv)
     {
     case osg::NodeVisitor::CULL_VISITOR:
     {
+
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
                 
         osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(&nv);
         if (cv)
@@ -132,8 +135,12 @@ void TXPNode::traverse(osg::NodeVisitor& nv)
         break;
     }
     case osg::NodeVisitor::UPDATE_VISITOR:
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+
         updateSceneGraph();
         break;
+    }
     default:
         break;
     }
@@ -179,40 +186,32 @@ const std::string& TXPNode::getArchiveName() const
     return _archiveName;
 }
 
-bool TXPNode::loadArchive()
+bool TXPNode::loadArchive(TXPArchive* archive)
 {
-    if (_archive.get())
-    {
-        TXPNodeERROR("loadArchive()") << "archive already open" << std::endl;
-        return false;
-    }
+    //if (_archive.get())
+    //{
+    //    TXPNodeERROR("loadArchive()") << "archive already open" << std::endl;
+    //    return false;
+    //}
 
-    _archive = new TXPArchive;
-    if (_archive->openFile(_archiveName) == false)
-    {
-        TXPNodeERROR("loadArchive()") << "failed to load archive: \"" << _archiveName << "\"" << std::endl;
-        return false;
-    }
 
-    /*
-    if (_archive->loadMaterials() == false)
-    {
-        TXPNodeERROR("loadArchive()") << "failed to load materials from archive: \"" << _archiveName << "\"" << std::endl;
-        return false;
-    }
-
-    if (_archive->loadModels() == false)
-    {
-        TXPNodeERROR("loadArchive()") << "failed to load models from archive: \"" << _archiveName << "\"" << std::endl;
-        return false;
-    }
-
-    if (_archive->loadLightAttributes() == false)
-    {
-        TXPNodeERROR("loadArchive()") << "failed to load light attributes from archive: \"" << _archiveName << "\"" << std::endl;
-        return false;
-    }
-    */
+   //modified by Brad Anderegg on May-27-08
+   //if NULL is passed in we will create a new archive and open the database
+   //otherwise we will use the archive provided which should have already been loaded 
+   //by ReaderWriterTXP::getArchive(). See line 57-77 of ReaderWriterTXP.cpp.
+   if(archive == NULL)
+   {
+       _archive = new TXPArchive;
+       if (_archive->openFile(_archiveName) == false)
+       {
+           TXPNodeERROR("loadArchive()") << "failed to load archive: \"" << _archiveName << "\"" << std::endl;
+           return false;
+       }
+   }
+   else
+   {
+      _archive = archive;
+   } 
 
     _archive->getOrigin(_originX,_originY);
     _archive->getExtents(_extents);
