@@ -1074,21 +1074,15 @@ bool trpgTexture::isValid() const
     switch (mode) {
     case External:
 	return (name != NULL);
-	break;
     case Local:
 	return (type != trpg_Unknown && sizeX != -1 && sizeY != -1);
-	break;
     case Global:
 	return (type != trpg_Unknown);
-	break;
     case Template:
 	return (type != trpg_Unknown && sizeX != -1 && sizeY != -1);
-	break;
     default:
 	return false;
     }
-
-    return false;
 }
 
 
@@ -1417,11 +1411,15 @@ bool trpgTexture::Read(trpgReadBuffer &buf)
 	buf.Get(ival);  
 	// Read the handle if we can..
 	try {
-	    if(!buf.Get((int32 &)handle)) {
-		handle = -1;
-	    }
-	    else {
+            int32 tempHandle;                
+	    if(buf.Get(tempHandle)) 
+            {
 		writeHandle = true;
+                handle = tempHandle;
+            }
+            else
+            {
+		handle = -1;
 	    }
 	}
 	catch (...) {
