@@ -25,6 +25,7 @@
 #include <osg/TexEnvCombine>
 #include <osg/Program>
 #include <osg/Math>
+#include <osg/Timer>
 
 using namespace osgTerrain;
 
@@ -42,8 +43,13 @@ GeometryTechnique::GeometryTechnique():
 }
 
 GeometryTechnique::GeometryTechnique(const GeometryTechnique& gt,const osg::CopyOp& copyop):
-    TerrainTechnique(gt,copyop)
+    TerrainTechnique(gt,copyop),
+    _currentReadOnlyBuffer(1),
+    _currentWriteBuffer(0)
 {
+    setFilterBias(gt._filterBias);
+    setFilterWidth(gt._filterWidth);
+    setFilterMatrix(gt._filterMatrix);
 }
 
 GeometryTechnique::~GeometryTechnique()
@@ -101,7 +107,7 @@ void GeometryTechnique::setFilterMatrixAs(FilterType filterType)
 
 void GeometryTechnique::init()
 {
-    // osg::notify(osg::NOTICE)<<"Doing GeometryTechnique::init()"<<std::endl;
+    osg::notify(osg::INFO)<<"Doing GeometryTechnique::init()"<<std::endl;
     
     if (!_terrainTile) return;
 
@@ -120,8 +126,6 @@ void GeometryTechnique::init()
 
     if (buffer._transform.valid()) buffer._transform->setThreadSafeRefUnref(true);
 
-    _dirty = false;    
-
     swapBuffers();
 }
 
@@ -234,6 +238,8 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
     double i_sampleFactor = 1.0;
     double j_sampleFactor = 1.0;
 
+    // osg::notify(osg::NOTICE)<<"Sample ratio="<<sampleRatio<<std::endl;
+
     if (sampleRatio!=1.0f)
     {
     
@@ -277,8 +283,8 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
     geometry->setNormalBinding(osg::Geometry::BIND_PER_VERTEX);
     
 
-    float minHeight = 0.0;
-    float scaleHeight = 1.0;
+    //float minHeight = 0.0;
+    float scaleHeight = _terrainTile->getTerrain() ? _terrainTile->getTerrain()->getVerticalScale() : 1.0f;
 
     // allocate and assign tex coords
     typedef std::pair< osg::ref_ptr<osg::Vec2Array>, Locator* > TexCoordLocatorPair;
@@ -297,10 +303,26 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
             }
             else
             {
+
+                Locator* locator = colorLayer->getLocator();
+                if (!locator)
+                {            
+                    osgTerrain::SwitchLayer* switchLayer = dynamic_cast<osgTerrain::SwitchLayer*>(colorLayer);
+                    if (switchLayer)
+                    {
+                        if (switchLayer->getActiveLayer()>=0 &&
+                            static_cast<unsigned int>(switchLayer->getActiveLayer())<switchLayer->getNumLayers() &&
+                            switchLayer->getLayer(switchLayer->getActiveLayer()))
+                        {
+                            locator = switchLayer->getLayer(switchLayer->getActiveLayer())->getLocator();
+                        }
+                    }
+                }            
+            
                 TexCoordLocatorPair& tclp = layerToTexCoordMap[colorLayer];
                 tclp.first = new osg::Vec2Array;
                 tclp.first->reserve(numVertices);
-                tclp.second = colorLayer->getLocator() ? colorLayer->getLocator() : masterLocator;
+                tclp.second = locator ? locator : masterLocator;
                 geometry->setTexCoordArray(layerNum, tclp.first.get());
             }
         }
@@ -341,7 +363,7 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
                 float value = 0.0f;
                 validValue = elevationLayer->getValidValue(i_equiv,j_equiv, value);
                 // osg::notify(osg::INFO)<<"i="<<i<<" j="<<j<<" z="<<value<<std::endl;
-                ndc.z() = value;
+                ndc.z() = value*scaleHeight;
             }
             
             if (validValue)
@@ -373,7 +395,7 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
 
                 if (elevations.valid())
                 {
-                    (*elevations).push_back((ndc.z()-minHeight)*scaleHeight);
+                    (*elevations).push_back(ndc.z());
                 }
 
                 // compute the local normal
@@ -394,9 +416,16 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
     // populate primitive sets
 //    bool optimizeOrientations = elevations!=0;
     bool swapOrientation = !(masterLocator->orientationOpenGL());
+
+    bool smallTile = numVertices <= 16384;
+
+    // osg::notify(osg::NOTICE)<<"smallTile = "<<smallTile<<std::endl;
     
-    osg::ref_ptr<osg::DrawElementsUInt> elements = new osg::DrawElementsUInt(GL_TRIANGLES);
-    elements->reserve((numRows-1) * (numColumns-1) * 6);
+    osg::ref_ptr<osg::DrawElements> elements = smallTile ? 
+        static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_TRIANGLES)) :
+        static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_TRIANGLES));
+        
+    elements->reserveElements((numRows-1) * (numColumns-1) * 6);
 
     geometry->addPrimitiveSet(elements.get());
 
@@ -441,31 +470,31 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
 
                 if (fabsf(e00-e11)<fabsf(e01-e10))
                 {
-                    elements->push_back(i01);
-                    elements->push_back(i00);
-                    elements->push_back(i11);
+                    elements->addElement(i01);
+                    elements->addElement(i00);
+                    elements->addElement(i11);
 
-                    elements->push_back(i00);
-                    elements->push_back(i10);
-                    elements->push_back(i11);
+                    elements->addElement(i00);
+                    elements->addElement(i10);
+                    elements->addElement(i11);
                 }
                 else
                 {
-                    elements->push_back(i01);
-                    elements->push_back(i00);
-                    elements->push_back(i10);
+                    elements->addElement(i01);
+                    elements->addElement(i00);
+                    elements->addElement(i10);
 
-                    elements->push_back(i01);
-                    elements->push_back(i10);
-                    elements->push_back(i11);
+                    elements->addElement(i01);
+                    elements->addElement(i10);
+                    elements->addElement(i11);
                 }
             }
             else if (numValid==3)
             {
-                if (i00>=0) elements->push_back(i00);
-                if (i01>=0) elements->push_back(i01);
-                if (i11>=0) elements->push_back(i11);
-                if (i10>=0) elements->push_back(i10);
+                if (i00>=0) elements->addElement(i00);
+                if (i01>=0) elements->addElement(i01);
+                if (i11>=0) elements->addElement(i11);
+                if (i10>=0) elements->addElement(i10);
             }
             
         }
@@ -484,7 +513,9 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
 
     if (createSkirt)
     {
-        osg::ref_ptr<osg::DrawElementsUShort> skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
+        osg::ref_ptr<osg::DrawElements> skirtDrawElements = smallTile ? 
+            static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_QUAD_STRIP)) :
+            static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_QUAD_STRIP));
 
         // create bottom skirt vertices
         int r,c;
@@ -506,24 +537,28 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
                     itr->second.first->push_back((*itr->second.first)[orig_i]);
                 }
                 
-                skirtDrawElements->push_back(orig_i);
-                skirtDrawElements->push_back(new_i);
+                skirtDrawElements->addElement(orig_i);
+                skirtDrawElements->addElement(new_i);
             }
             else
             {
-                if (!skirtDrawElements->empty())
+                if (skirtDrawElements->getNumIndices()!=0)
                 {
                     geometry->addPrimitiveSet(skirtDrawElements.get());
-                    skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
+                    skirtDrawElements = smallTile ? 
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_QUAD_STRIP)) :
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_QUAD_STRIP));
                 }
                 
             }
         }
 
-        if (!skirtDrawElements->empty())
+        if (skirtDrawElements->getNumIndices()!=0)
         {
             geometry->addPrimitiveSet(skirtDrawElements.get());
-            skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
+            skirtDrawElements = smallTile ? 
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_QUAD_STRIP)) :
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_QUAD_STRIP));
         }
 
         // create right skirt vertices
@@ -544,24 +579,28 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
                     itr->second.first->push_back((*itr->second.first)[orig_i]);
                 }
                 
-                skirtDrawElements->push_back(orig_i);
-                skirtDrawElements->push_back(new_i);
+                skirtDrawElements->addElement(orig_i);
+                skirtDrawElements->addElement(new_i);
             }
             else
             {
-                if (!skirtDrawElements->empty())
+                if (skirtDrawElements->getNumIndices()!=0)
                 {
                     geometry->addPrimitiveSet(skirtDrawElements.get());
-                    skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
+                    skirtDrawElements = smallTile ? 
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_QUAD_STRIP)) :
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_QUAD_STRIP));
                 }
                 
             }
         }
 
-        if (!skirtDrawElements->empty())
+        if (skirtDrawElements->getNumIndices()!=0)
         {
             geometry->addPrimitiveSet(skirtDrawElements.get());
-            skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
+            skirtDrawElements = smallTile ? 
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_QUAD_STRIP)) :
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_QUAD_STRIP));
         }
 
         // create top skirt vertices
@@ -582,24 +621,28 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
                     itr->second.first->push_back((*itr->second.first)[orig_i]);
                 }
                 
-                skirtDrawElements->push_back(orig_i);
-                skirtDrawElements->push_back(new_i);
+                skirtDrawElements->addElement(orig_i);
+                skirtDrawElements->addElement(new_i);
             }
             else
             {
-                if (!skirtDrawElements->empty())
+                if (skirtDrawElements->getNumIndices()!=0)
                 {
                     geometry->addPrimitiveSet(skirtDrawElements.get());
-                    skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
+                    skirtDrawElements = smallTile ? 
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_QUAD_STRIP)) :
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_QUAD_STRIP));
                 }
                 
             }
         }
 
-        if (!skirtDrawElements->empty())
+        if (skirtDrawElements->getNumIndices()!=0)
         {
             geometry->addPrimitiveSet(skirtDrawElements.get());
-            skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
+            skirtDrawElements = smallTile ? 
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_QUAD_STRIP)) :
+                        static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_QUAD_STRIP));
         }
 
         // create left skirt vertices
@@ -620,12 +663,12 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
                     itr->second.first->push_back((*itr->second.first)[orig_i]);
                 }
                 
-                skirtDrawElements->push_back(orig_i);
-                skirtDrawElements->push_back(new_i);
+                skirtDrawElements->addElement(orig_i);
+                skirtDrawElements->addElement(new_i);
             }
             else
             {
-                if (!skirtDrawElements->empty())
+                if (skirtDrawElements->getNumIndices()!=0)
                 {
                     geometry->addPrimitiveSet(skirtDrawElements.get());
                     skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
@@ -634,16 +677,32 @@ void GeometryTechnique::generateGeometry(Locator* masterLocator, const osg::Vec3
             }
         }
 
-        if (!skirtDrawElements->empty())
+        if (skirtDrawElements->getNumIndices()!=0)
         {
             geometry->addPrimitiveSet(skirtDrawElements.get());
-            skirtDrawElements = new osg::DrawElementsUShort(GL_QUAD_STRIP);
+            smallTile ? 
+                static_cast<osg::DrawElements*>(new osg::DrawElementsUShort(GL_QUAD_STRIP)) :
+                static_cast<osg::DrawElements*>(new osg::DrawElementsUInt(GL_QUAD_STRIP));
         }
     }
 
 
-    //geometry->setUseDisplayList(false);
+    geometry->setUseDisplayList(false);
     geometry->setUseVertexBufferObjects(true);
+    
+    
+    if (osgDB::Registry::instance()->getBuildKdTreesHint()==osgDB::ReaderWriter::Options::BUILD_KDTREES &&
+        osgDB::Registry::instance()->getKdTreeBuilder())
+    {
+    
+        
+        //osg::Timer_t before = osg::Timer::instance()->tick();
+        //osg::notify(osg::NOTICE)<<"osgTerrain::GeometryTechnique::build kd tree"<<std::endl;
+        osg::ref_ptr<osg::KdTreeBuilder> builder = osgDB::Registry::instance()->getKdTreeBuilder()->clone();
+        buffer._geode->accept(*builder);
+        //osg::Timer_t after = osg::Timer::instance()->tick();
+        //osg::notify(osg::NOTICE)<<"KdTree build time "<<osg::Timer::instance()->delta_m(before, after)<<std::endl;
+    }
 }
 
 void GeometryTechnique::applyColorLayers()
@@ -658,6 +717,19 @@ void GeometryTechnique::applyColorLayers()
         osgTerrain::Layer* colorLayer = _terrainTile->getColorLayer(layerNum);
         if (!colorLayer) continue;
 
+        osgTerrain::SwitchLayer* switchLayer = dynamic_cast<osgTerrain::SwitchLayer*>(colorLayer);
+        if (switchLayer)
+        {
+            if (switchLayer->getActiveLayer()<0 || 
+                static_cast<unsigned int>(switchLayer->getActiveLayer())>=switchLayer->getNumLayers())
+            {
+                continue;
+            }
+
+            colorLayer = switchLayer->getLayer(switchLayer->getActiveLayer());
+            if (!colorLayer) continue;
+        }
+
         osg::Image* image = colorLayer->getImage();
         if (!image) continue;
 
@@ -674,14 +746,27 @@ void GeometryTechnique::applyColorLayers()
                 texture2D->setImage(image);
                 texture2D->setMaxAnisotropy(16.0f);
                 texture2D->setResizeNonPowerOfTwoHint(false);
-                texture2D->setFilter(osg::Texture::MIN_FILTER, osg::Texture::LINEAR_MIPMAP_LINEAR);
-                texture2D->setFilter(osg::Texture::MAG_FILTER, colorLayer->getFilter()==Layer::LINEAR ? osg::Texture::LINEAR :  osg::Texture::NEAREST);
+
+                texture2D->setFilter(osg::Texture::MIN_FILTER, colorLayer->getMinFilter());
+                texture2D->setFilter(osg::Texture::MAG_FILTER, colorLayer->getMagFilter());
+
                 texture2D->setWrap(osg::Texture::WRAP_S,osg::Texture::CLAMP_TO_EDGE);
                 texture2D->setWrap(osg::Texture::WRAP_T,osg::Texture::CLAMP_TO_EDGE);
 
+                bool mipMapping = !(texture2D->getFilter(osg::Texture::MIN_FILTER)==osg::Texture::LINEAR || texture2D->getFilter(osg::Texture::MIN_FILTER)==osg::Texture::NEAREST);
+                bool s_NotPowerOfTwo = image->s()==0 || (image->s() & (image->s() - 1));
+                bool t_NotPowerOfTwo = image->t()==0 || (image->t() & (image->t() - 1));
+
+                if (mipMapping && (s_NotPowerOfTwo || t_NotPowerOfTwo))
+                {
+                    osg::notify(osg::INFO)<<"Disabling mipmapping for non power of two tile size("<<image->s()<<", "<<image->t()<<")"<<std::endl;
+                    texture2D->setFilter(osg::Texture::MIN_FILTER, osg::Texture::LINEAR);
+                }
+
+
                 layerToTextureMap[colorLayer] = texture2D;
 
-                // osg::notify(osg::NOTICE)<<"Creating new ImageLayer texture "<<layerNum<<std::endl;
+                // osg::notify(osg::NOTICE)<<"Creating new ImageLayer texture "<<layerNum<<" image->s()="<<image->s()<<"  image->t()="<<image->t()<<std::endl;
 
             }
             else
@@ -703,7 +788,7 @@ void GeometryTechnique::applyColorLayers()
                 texture1D->setImage(image);
                 texture1D->setResizeNonPowerOfTwoHint(false);
                 texture1D->setFilter(osg::Texture::MIN_FILTER, osg::Texture::NEAREST);
-                texture1D->setFilter(osg::Texture::MAG_FILTER, colorLayer->getFilter()==Layer::LINEAR ? osg::Texture::LINEAR :  osg::Texture::NEAREST);
+                texture1D->setFilter(osg::Texture::MAG_FILTER, colorLayer->getMagFilter());
 
                 layerToTextureMap[colorLayer] = texture1D;
             }
@@ -721,7 +806,7 @@ void GeometryTechnique::applyTransparency()
     bool containsTransparency = false;
     for(unsigned int i=0; i<_terrainTile->getNumColorLayers(); ++i)
     {
-        osg::Image* image = _terrainTile->getColorLayer(i)->getImage();
+        osg::Image* image = (_terrainTile->getColorLayer(i)!=0) ? _terrainTile->getColorLayer(i)->getImage() : 0;
         if (image)
         {
             containsTransparency = image->isImageTranslucent();
@@ -777,7 +862,7 @@ void GeometryTechnique::traverse(osg::NodeVisitor& nv)
     // if app traversal update the frame count.
     if (nv.getVisitorType()==osg::NodeVisitor::UPDATE_VISITOR)
     {
-        if (_dirty) _terrainTile->init();
+        if (_terrainTile->getDirty()) _terrainTile->init();
 
         osgUtil::UpdateVisitor* uv = dynamic_cast<osgUtil::UpdateVisitor*>(&nv);
         if (uv)
@@ -798,7 +883,7 @@ void GeometryTechnique::traverse(osg::NodeVisitor& nv)
     }
 
 
-    if (_dirty) 
+    if (_terrainTile->getDirty()) 
     {
         osg::notify(osg::INFO)<<"******* Doing init ***********"<<std::endl;
         _terrainTile->init();
@@ -813,8 +898,3 @@ void GeometryTechnique::cleanSceneGraph()
 {
 }
 
-void GeometryTechnique::dirty()
-{
-    TerrainTechnique::dirty();
-}
-
