@@ -12,19 +12,25 @@
 */
 
 #include <osgTerrain/Terrain>
+#include <osgTerrain/GeometryTechnique>
+
 #include <OpenThreads/ScopedLock>
 
 using namespace osg;
 using namespace osgTerrain;
 
 Terrain::Terrain():
-    _sampleRatio(1.0)
+    _sampleRatio(1.0),
+    _verticalScale(1.0)
 {
+    _terrainTechnique = new GeometryTechnique;
 }
 
 Terrain::Terrain(const Terrain& ts, const osg::CopyOp& copyop):
     osg::Group(ts,copyop),
-    _sampleRatio(ts._sampleRatio)
+    _sampleRatio(ts._sampleRatio),
+    _verticalScale(ts._verticalScale),
+    _terrainTechnique(ts._terrainTechnique)
 {
 }
 
@@ -54,7 +60,7 @@ TerrainTile* Terrain::getTile(const TileID& tileID)
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
 
     TerrainTileMap::iterator itr = _terrainTileMap.find(tileID);
-    if (itr != _terrainTileMap.end()) return 0;
+    if (itr == _terrainTileMap.end()) return 0;
     
     return itr->second;
 }
@@ -64,11 +70,23 @@ const TerrainTile* Terrain::getTile(const TileID& tileID) const
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
 
     TerrainTileMap::const_iterator itr = _terrainTileMap.find(tileID);
-    if (itr != _terrainTileMap.end()) return 0;
+    if (itr == _terrainTileMap.end()) return 0;
     
     return itr->second;
 }
 
+void Terrain::dirtyRegisteredTiles()
+{
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+
+    for(TerrainTileSet::iterator itr = _terrainTileSet.begin();
+        itr != _terrainTileSet.end();
+        ++itr)
+    {
+        (const_cast<TerrainTile*>(*itr))->setDirty(true);
+    }
+}
+
 static unsigned int s_maxNumTiles = 0;
 void Terrain::registerTerrainTile(TerrainTile* tile)
 {
