@@ -228,7 +228,7 @@ public:
                 return false;
             }
 
-            osg::ref_ptr<Point> pNew = edge->_proposedPoint.valid()? edge->_proposedPoint : computeInterpolatedPoint(edge,0.5f);
+            osg::ref_ptr<Point> pNew = edge->_proposedPoint.valid()? edge->_proposedPoint.get() : computeInterpolatedPoint(edge,0.5f);
             return (collapseEdge(edge,pNew.get()));
         }
         osg::notify(osg::INFO)<<"collapseMinimumErrorEdge() return false due to _edgeSet.empty()"<<std::endl;
@@ -248,7 +248,7 @@ public:
                 return false;
             }
 
-            osg::ref_ptr<Point> pNew = edge->_proposedPoint.valid()? edge->_proposedPoint : computeInterpolatedPoint(edge,0.5f);
+            osg::ref_ptr<Point> pNew = edge->_proposedPoint.valid()? edge->_proposedPoint.get() : computeInterpolatedPoint(edge,0.5f);
             return (divideEdge(edge,pNew.get()));
         }
         osg::notify(osg::INFO)<<"divideLongestEdge() return false due to _edgeSet.empty()"<<std::endl;
@@ -1348,6 +1348,11 @@ class CopyArrayToPointsVisitor : public osg::ArrayVisitor
         }
         
         EdgeCollapse::PointList& _pointList;
+        
+        
+    protected:
+    
+        CopyArrayToPointsVisitor& operator = (const CopyArrayToPointsVisitor&) { return *this; }
 };
 
 class CopyVertexArrayToPointsVisitor : public osg::ArrayVisitor
@@ -1400,6 +1405,11 @@ class CopyVertexArrayToPointsVisitor : public osg::ArrayVisitor
         }
         
         EdgeCollapse::PointList& _pointList;
+
+    protected:
+    
+        CopyVertexArrayToPointsVisitor& operator = (const CopyVertexArrayToPointsVisitor&) { return *this; }
+
 };
 
 void EdgeCollapse::setGeometry(osg::Geometry* geometry, const Simplifier::IndexList& protectedPoints)
@@ -1413,6 +1423,14 @@ void EdgeCollapse::setGeometry(osg::Geometry* geometry, const Simplifier::IndexL
         osg::notify(osg::INFO)<<"EdgeCollapse::setGeometry(..): Removing attribute indices"<<std::endl;
         _geometry->copyToAndOptimize(*_geometry);
     }
+    
+    // check to see if vertex attributes indices exists, if so expand them to remove them
+    if (_geometry->containsSharedArrays())
+    {
+        // removing coord indices
+        osg::notify(osg::INFO)<<"EdgeCollapse::setGeometry(..): Duplicate shared arrays"<<std::endl;
+        _geometry->duplicateSharedArrays();
+    }
 
     unsigned int numVertices = geometry->getVertexArray()->getNumElements();
         
@@ -1559,6 +1577,10 @@ class CopyPointsToArrayVisitor : public osg::ArrayVisitor
         
         EdgeCollapse::PointList& _pointList;
         unsigned int _index;
+        
+    protected:
+    
+        CopyPointsToArrayVisitor& operator = (CopyPointsToArrayVisitor&) { return *this; }
 };
 
 class NormalizeArrayVisitor : public osg::ArrayVisitor
@@ -1625,6 +1647,10 @@ class CopyPointsToVertexArrayVisitor : public osg::ArrayVisitor
         }
         
         EdgeCollapse::PointList& _pointList;
+        
+    protected:
+    
+        CopyPointsToVertexArrayVisitor& operator = (const CopyPointsToVertexArrayVisitor&) { return *this; }
 };
 
 
