@@ -72,7 +72,7 @@ void CompositeViewer::constructorInit()
     _updateVisitor = new osgUtil::UpdateVisitor;
     _updateVisitor->setFrameStamp(_frameStamp.get());
 
-    setStats(new osg::Stats("CompsiteViewer"));
+    setViewerStats(new osg::Stats("CompsiteViewer"));
 }
 
 CompositeViewer::~CompositeViewer()
@@ -119,6 +119,10 @@ bool CompositeViewer::readConfiguration(const std::string& filename)
 
 void CompositeViewer::addView(osgViewer::View* view)
 {
+    if (!view) return;
+
+    bool alreadyRealized = isRealized();
+    
     bool threadsWereRuinning = _threadsRunning;
     if (threadsWereRuinning) stopThreading();
 
@@ -126,8 +130,47 @@ void CompositeViewer::addView(osgViewer::View* view)
     
     view->_viewerBase = this;
     
+    if (view->getSceneData())
+    {        
+        // make sure that existing scene graph objects are allocated with thread safe ref/unref
+        if (getThreadingModel()!=ViewerBase::SingleThreaded) 
+        {
+            view->getSceneData()->setThreadSafeRefUnref(true);
+        }
+        
+        // update the scene graph so that it has enough GL object buffer memory for the graphics contexts that will be using it.
+        view->getSceneData()->resizeGLObjectBuffers(osg::DisplaySettings::instance()->getMaxNumberOfGraphicsContexts());
+    }
+
     view->setFrameStamp(_frameStamp.get());
     
+    if (alreadyRealized)
+    {
+        Contexts contexts;
+        if (view->getCamera()->getGraphicsContext())
+        {
+            contexts.push_back(view->getCamera()->getGraphicsContext());
+        }
+        for(unsigned int i=0; i<view->getNumSlaves(); ++i)
+        {
+            if (view->getSlave(i)._camera->getGraphicsContext())
+            {
+                contexts.push_back(view->getSlave(i)._camera->getGraphicsContext());
+            }
+        }
+
+        for(Contexts::iterator itr = contexts.begin();
+            itr != contexts.end();
+            ++itr)
+        {
+            if (!((*itr)->isRealized()))
+            {
+                (*itr)->realize();
+            }
+        }
+
+    }
+    
     if (threadsWereRuinning) startThreading();
 }
 
@@ -184,6 +227,8 @@ int CompositeViewer::run()
         }
     }
         
+    setReleaseContextAtEndOfFrameHint(false);
+
     return ViewerBase::run();
 }
 
@@ -245,39 +290,37 @@ void CompositeViewer::getContexts(Contexts& contexts, bool onlyValid)
     typedef std::set<osg::GraphicsContext*> ContextSet;
     ContextSet contextSet;
 
+    contexts.clear();
+
     for(RefViews::iterator vitr = _views.begin();
         vitr != _views.end();
         ++vitr)
     {
         osgViewer::View* view = vitr->get();
-        if (view->getCamera() && 
-            view->getCamera()->getGraphicsContext() && 
-            (view->getCamera()->getGraphicsContext()->valid() || !onlyValid))
+        osg::GraphicsContext* gc = view->getCamera() ? view->getCamera()->getGraphicsContext() : 0;
+        if (gc && (gc->valid() || !onlyValid))
         {
-            contextSet.insert(view->getCamera()->getGraphicsContext());
+            if (contextSet.count(gc)==0)
+            {
+                contextSet.insert(gc);
+                contexts.push_back(gc);
+            }
         }
 
         for(unsigned int i=0; i<view->getNumSlaves(); ++i)
         {
             View::Slave& slave = view->getSlave(i);
-            if (slave._camera.valid() && 
-                slave._camera->getGraphicsContext() && 
-                (slave._camera->getGraphicsContext()->valid() || !onlyValid))
+            osg::GraphicsContext* sgc = slave._camera.valid() ? slave._camera->getGraphicsContext() : 0;
+            if (sgc && (sgc->valid() || !onlyValid))
             {
-                contextSet.insert(slave._camera->getGraphicsContext());
+                if (contextSet.count(sgc)==0)
+                {
+                    contextSet.insert(sgc);
+                    contexts.push_back(sgc);
+                }
             }
         }
     }
-    
-    contexts.clear();
-    contexts.reserve(contextSet.size());
-
-    for(ContextSet::iterator itr = contextSet.begin();
-        itr != contextSet.end();
-        ++itr)
-    {
-        contexts.push_back(const_cast<osg::GraphicsContext*>(*itr));
-    }
 }
 
 void CompositeViewer::getCameras(Cameras& cameras, bool onlyActive)
@@ -303,24 +346,10 @@ void CompositeViewer::getCameras(Cameras& cameras, bool onlyActive)
     }
 }
  
-void CompositeViewer::getWindows(Windows& windows, bool onlyValid)
-{
-    windows.clear();
-
-    Contexts contexts;
-    getContexts(contexts, onlyValid);
-    
-    for(Contexts::iterator itr = contexts.begin();
-        itr != contexts.end();
-        ++itr)
-    {
-        osgViewer::GraphicsWindow* gw = dynamic_cast<osgViewer::GraphicsWindow*>(*itr);
-        if (gw) windows.push_back(gw);
-    }
-}
-
 void CompositeViewer::getScenes(Scenes& scenes, bool onlyValid)
 {
+    scenes.clear();
+
     typedef std::set<osgViewer::Scene*> SceneSet;
     SceneSet sceneSet;
 
@@ -331,20 +360,19 @@ void CompositeViewer::getScenes(Scenes& scenes, bool onlyValid)
         osgViewer::View* view = vitr->get();
         if (view->getScene() && (!onlyValid || view->getScene()->getSceneData()))
         {
-            sceneSet.insert(view->getScene());
+            if (sceneSet.count(view->getScene())==0)
+            {
+                sceneSet.insert(view->getScene());
+                scenes.push_back(view->getScene());
+            }
         }
     }
-
-    for(SceneSet::iterator sitr = sceneSet.begin();
-        sitr != sceneSet.end();
-        ++sitr)
-    {
-        scenes.push_back(const_cast<osgViewer::Scene*>(*sitr));
-    }
 }
 
 void CompositeViewer::getViews(Views& views, bool onlyValid)
 {
+    views.clear();
+
     for(RefViews::iterator vitr = _views.begin();
         vitr != _views.end();
         ++vitr)
@@ -355,6 +383,8 @@ void CompositeViewer::getViews(Views& views, bool onlyValid)
 
 void CompositeViewer::getAllThreads(Threads& threads, bool onlyActive)
 {
+    threads.clear();
+
     OperationThreads operationThreads;
     getOperationThreads(operationThreads);
     
@@ -373,10 +403,17 @@ void CompositeViewer::getAllThreads(Threads& threads, bool onlyActive)
         ++sitr)
     {
         Scene* scene = *sitr;
-        if (scene->getDatabasePager() &&
-           (!onlyActive || scene->getDatabasePager()->isRunning())) 
+        osgDB::DatabasePager* dp = scene->getDatabasePager();
+        if (dp)
         {
-            threads.push_back(scene->getDatabasePager());
+            for(unsigned int i=0; i<dp->getNumDatabaseThreads(); ++i)
+            {
+                osgDB::DatabasePager::DatabaseThread* dt = dp->getDatabaseThread(i);
+                if (!onlyActive || dt->isRunning())
+                {
+                    threads.push_back(dt);
+                }
+            }
         }
     }
 }
@@ -533,15 +570,15 @@ void CompositeViewer::advance(double simulationTime)
         _frameStamp->setSimulationTime(simulationTime);
     }
     
-    if (getStats() && getStats()->collectStats("frame_rate"))
+    if (getViewerStats() && getViewerStats()->collectStats("frame_rate"))
     {
         // update previous frame stats
         double deltaFrameTime = _frameStamp->getReferenceTime() - prevousReferenceTime;
-        getStats()->setAttribute(previousFrameNumber, "Frame duration", deltaFrameTime);
-        getStats()->setAttribute(previousFrameNumber, "Frame rate", 1.0/deltaFrameTime);
+        getViewerStats()->setAttribute(previousFrameNumber, "Frame duration", deltaFrameTime);
+        getViewerStats()->setAttribute(previousFrameNumber, "Frame rate", 1.0/deltaFrameTime);
 
         // update current frames stats
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Reference time", _frameStamp->getReferenceTime());
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Reference time", _frameStamp->getReferenceTime());
     }
 
 }
@@ -619,6 +656,8 @@ void CompositeViewer::eventTraversal()
             {
                 osgGA::GUIEventAdapter* event = itr->get();
                 
+                //osg::notify(osg::NOTICE)<<"event->getGraphicsContext()="<<event->getGraphicsContext()<<std::endl;
+                
                 bool pointerEvent = false;
 
                 float x = event->getX();
@@ -661,15 +700,11 @@ void CompositeViewer::eventTraversal()
                                         // If this camera is not a slave camera
                                         if (camera->getView()->getCamera() == camera)
                                         {
-                                            const osg::GraphicsContext::Traits* traits = gw ? gw->getTraits() : 0;
-                                            if (traits) 
-                                            {
-                                                eventState->setInputRange( 0, 0, traits->width, traits->height);
-                                            }
-                                            else
-                                            {
-                                                eventState->setInputRange(-1.0, -1.0, 1.0, 1.0);
-                                            }
+                                            eventState->setGraphicsContext(gw);
+                                            eventState->setInputRange( viewport->x(), viewport->y(), 
+                                                                       viewport->x()+viewport->width(),
+                                                                       viewport->y()+viewport->height());
+
                                         }
                                         else
                                         {
@@ -690,6 +725,20 @@ void CompositeViewer::eventTraversal()
                                                 masterCameraVPW *= viewport->computeWindowMatrix();
                                             }
                                         }
+
+                                        // If this camera is not a slave camera
+                                        if (camera->getView()->getCamera() == camera)
+                                        {
+                                            eventState->setGraphicsContext(gw);
+                                            eventState->setInputRange( viewport->x(), viewport->y(), 
+                                                                       viewport->x()+viewport->width(),
+                                                                       viewport->y()+viewport->height());
+
+                                        }
+                                        else
+                                        {
+                                            eventState->setInputRange(-1.0, -1.0, 1.0, 1.0);
+                                        }
                                     }
                                 }
                             }
@@ -737,8 +786,6 @@ void CompositeViewer::eventTraversal()
                     event->setButtonMask(eventState->getButtonMask());
                     event->setMouseYOrientation(eventState->getMouseYOrientation());
                 }
-                //osg::notify(osg::NOTICE)<<"   mouse x = "<<event->getX()<<" y="<<event->getY()<<std::endl;
-                // osg::notify(osg::NOTICE)<<"   mouse Xmin = "<<event->getXmin()<<" Ymin="<<event->getYmin()<<" xMax="<<event->getXmax()<<" Ymax="<<event->getYmax()<<std::endl;
             }
 
             for(itr = gw_events.begin();
@@ -782,17 +829,6 @@ void CompositeViewer::eventTraversal()
         view->getEventQueue()->takeEvents(viewEventsMap[view]);
     }
     
-#if 0
-    _eventQueue->getCurrentEventState()->setInputRange(eventState->getXmin(), eventState->getYmin(), eventState->getXmax(), eventState->getYmax());
-    _eventQueue->getCurrentEventState()->setX(eventState->getX());
-    _eventQueue->getCurrentEventState()->setY(eventState->getY());
-    _eventQueue->getCurrentEventState()->setButtonMask(eventState->getButtonMask());
-    _eventQueue->getCurrentEventState()->setMouseYOrientation(eventState->getMouseYOrientation());
-
-    _eventQueue->frame( getFrameStamp()->getReferenceTime() );
-    _eventQueue->takeEvents(events);
-#endif
-
 
     // osg::notify(osg::NOTICE)<<"Events "<<events.size()<<std::endl;
     
@@ -826,33 +862,6 @@ void CompositeViewer::eventTraversal()
         
     if (_done) return;
 
-    for(ViewEventsMap::iterator veitr = viewEventsMap.begin();
-        veitr != viewEventsMap.end();
-        ++veitr)
-    {
-        View* view = veitr->first;
-        _eventVisitor->setActionAdapter(view);
-        
-        for(osgGA::EventQueue::Events::iterator itr = veitr->second.begin();
-            itr != veitr->second.end();
-            ++itr)
-        {
-            osgGA::GUIEventAdapter* event = itr->get();
-
-            for(View::EventHandlers::iterator hitr = view->getEventHandlers().begin();
-                hitr != view->getEventHandlers().end();
-                ++hitr)
-            {
-                (*hitr)->handleWithCheckAgainstIgnoreHandledEventsMask( *event, *view, 0, 0);
-            }
-
-            if (view->getCameraManipulator())
-            {
-                view->getCameraManipulator()->handleWithCheckAgainstIgnoreHandledEventsMask( *event, *view);
-            }
-        }
-    }
-
     if (_eventVisitor.valid())
     {
         _eventVisitor->setFrameStamp(getFrameStamp());
@@ -863,6 +872,8 @@ void CompositeViewer::eventTraversal()
             ++veitr)
         {
             View* view = veitr->first;
+            _eventVisitor->setActionAdapter(view);
+            
             if (view->getSceneData())
             {            
                 for(osgGA::EventQueue::Events::iterator itr = veitr->second.begin();
@@ -896,16 +907,57 @@ void CompositeViewer::eventTraversal()
         }
         
     }
+
+    for(ViewEventsMap::iterator veitr = viewEventsMap.begin();
+        veitr != viewEventsMap.end();
+        ++veitr)
+    {
+        View* view = veitr->first;
+        
+        for(osgGA::EventQueue::Events::iterator itr = veitr->second.begin();
+            itr != veitr->second.end();
+            ++itr)
+        {
+            osgGA::GUIEventAdapter* event = itr->get();
+
+            for(View::EventHandlers::iterator hitr = view->getEventHandlers().begin();
+                hitr != view->getEventHandlers().end();
+                ++hitr)
+            {
+                (*hitr)->handleWithCheckAgainstIgnoreHandledEventsMask( *event, *view, 0, 0);
+            }
+        }
+    }
+
+    for(ViewEventsMap::iterator veitr = viewEventsMap.begin();
+        veitr != viewEventsMap.end();
+        ++veitr)
+    {
+        View* view = veitr->first;
+        
+        for(osgGA::EventQueue::Events::iterator itr = veitr->second.begin();
+            itr != veitr->second.end();
+            ++itr)
+        {
+            osgGA::GUIEventAdapter* event = itr->get();
+
+            if (view->getCameraManipulator())
+            {
+                view->getCameraManipulator()->handleWithCheckAgainstIgnoreHandledEventsMask( *event, *view);
+            }
+        }
+    }
+
     
 
-    if (getStats() && getStats()->collectStats("event"))
+    if (getViewerStats() && getViewerStats()->collectStats("event"))
     {
         double endEventTraversal = osg::Timer::instance()->delta_s(_startTick, osg::Timer::instance()->tick());
 
         // update current frames stats
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal begin time", beginEventTraversal);
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal end time", endEventTraversal);
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal time taken", endEventTraversal-beginEventTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal begin time", beginEventTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal end time", endEventTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal time taken", endEventTraversal-beginEventTraversal);
     }
 }
 
@@ -927,19 +979,18 @@ void CompositeViewer::updateTraversal()
         ++sitr)
     {
         Scene* scene = *sitr;
-        if (scene->getSceneData())
-        {
-            scene->getSceneData()->accept(*_updateVisitor);
-        }
-
-        if (scene->getDatabasePager())
-        {    
-            // synchronize changes required by the DatabasePager thread to the scene graph
-            scene->getDatabasePager()->updateSceneGraph(_frameStamp->getReferenceTime());
-        }
-
+        scene->updateSceneGraph(*_updateVisitor);
     }
 
+    // if we have a shared state manager prune any unused entries
+    if (osgDB::Registry::instance()->getSharedStateManager())
+        osgDB::Registry::instance()->getSharedStateManager()->prune();
+
+    // update the Registry object cache.
+    osgDB::Registry::instance()->updateTimeStampOfObjectsInCacheWithExternalReferences(*getFrameStamp());
+    osgDB::Registry::instance()->removeExpiredObjectsInCache(*getFrameStamp());
+
+
     if (_updateOperations.valid())
     {
         _updateOperations->runOperations(this);
@@ -951,8 +1002,6 @@ void CompositeViewer::updateTraversal()
     {
         View* view = vitr->get();
 
-        Scene* scene = view->getScene();
-
         {
             // call any camera update callbacks, but only traverse that callback, don't traverse its subgraph
             // leave that to the scene update traversal.
@@ -982,14 +1031,14 @@ void CompositeViewer::updateTraversal()
 
     }
     
-    if (getStats() && getStats()->collectStats("update"))
+    if (getViewerStats() && getViewerStats()->collectStats("update"))
     {
         double endUpdateTraversal = osg::Timer::instance()->delta_s(_startTick, osg::Timer::instance()->tick());
 
         // update current frames stats
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal begin time", beginUpdateTraversal);
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal end time", endUpdateTraversal);
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal time taken", endUpdateTraversal-beginUpdateTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal begin time", beginUpdateTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal end time", endUpdateTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal time taken", endUpdateTraversal-beginUpdateTraversal);
     }
 
 }
