@@ -17,11 +17,13 @@
 
 #include <osgUtil/Optimizer>
 #include <osgUtil/GLObjectsVisitor>
+#include <osgUtil/Statistics>
 
 #include <osgViewer/Renderer>
 #include <osgViewer/View>
 
 #include <osgDB/DatabasePager>
+#include <osgDB/ImagePager>
 
 #include <osg/io_utils>
 
@@ -119,18 +121,18 @@ void OpenGLQuerySupport::initialize(osg::State* state)
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //
-//  TheadSafeQueue
+//  ThreadSafeQueue
 
-Renderer::TheadSafeQueue::TheadSafeQueue()
+Renderer::ThreadSafeQueue::ThreadSafeQueue()
 {
     _block.set(false);
 }
 
-Renderer::TheadSafeQueue::~TheadSafeQueue()
+Renderer::ThreadSafeQueue::~ThreadSafeQueue()
 {
 }
 
-osgUtil::SceneView* Renderer::TheadSafeQueue::takeFront()
+osgUtil::SceneView* Renderer::ThreadSafeQueue::takeFront()
 {
     if (_queue.empty()) _block.block();
 
@@ -145,7 +147,7 @@ osgUtil::SceneView* Renderer::TheadSafeQueue::takeFront()
     return front;
 }
 
-void Renderer::TheadSafeQueue::add(osgUtil::SceneView* sv)
+void Renderer::ThreadSafeQueue::add(osgUtil::SceneView* sv)
 {
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
     _queue.push_back(sv);
@@ -167,7 +169,8 @@ Renderer::Renderer(osg::Camera* camera):
     _conservativeTimeRatio(0.5),
     _camera(camera),
     _done(false),
-    _graphicsThreadDoesCull(true)
+    _graphicsThreadDoesCull(true),
+    _compileOnNextDraw(true)
 {
 
     DEBUG_MESSAGE<<"Render::Render() "<<this<<std::endl;
@@ -175,7 +178,6 @@ Renderer::Renderer(osg::Camera* camera):
     _sceneView[0] = new osgUtil::SceneView;
     _sceneView[1] = new osgUtil::SceneView;
 
-    unsigned int sceneViewOptions = osgUtil::SceneView::HEADLIGHT;
 
     osg::Camera* masterCamera = _camera->getView() ? _camera->getView()->getCamera() : camera;
     osg::StateSet* stateset = masterCamera->getOrCreateStateSet();
@@ -184,6 +186,17 @@ Renderer::Renderer(osg::Camera* camera):
     osg::DisplaySettings* ds = _camera->getDisplaySettings() ?  _camera->getDisplaySettings() :
                                ((view && view->getDisplaySettings()) ?  view->getDisplaySettings() :  osg::DisplaySettings::instance());
 
+    unsigned int sceneViewOptions = osgUtil::SceneView::HEADLIGHT;
+    if (view)
+    {
+        switch(view->getLightingMode())
+        {
+            case(osg::View::NO_LIGHT): sceneViewOptions = 0; break;
+            case(osg::View::SKY_LIGHT): sceneViewOptions = osgUtil::SceneView::SKY_LIGHT; break;
+            case(osg::View::HEADLIGHT): sceneViewOptions = osgUtil::SceneView::HEADLIGHT; break;
+        }
+    }
+
     _sceneView[0]->setGlobalStateSet(stateset);
     _sceneView[1]->setGlobalStateSet(stateset);
     
@@ -236,8 +249,12 @@ void Renderer::updateSceneView(osgUtil::SceneView* sceneView)
     }
 
     osgViewer::View* view = dynamic_cast<osgViewer::View*>(_camera->getView());
+
     osgDB::DatabasePager* databasePager = view ? view->getDatabasePager() : 0;
     sceneView->getCullVisitor()->setDatabaseRequestHandler(databasePager);
+
+    osgDB::ImagePager* imagePager = view ? view->getImagePager() : 0;
+    sceneView->getCullVisitor()->setImageRequestHandler(imagePager);
     
     sceneView->setFrameStamp(view ? view->getFrameStamp() : state->getFrameStamp());
     
@@ -251,6 +268,22 @@ void Renderer::updateSceneView(osgUtil::SceneView* sceneView)
     if (view) _startTick = view->getStartTick();
 }
 
+void Renderer::compile()
+{
+    DEBUG_MESSAGE<<"Renderer::compile()"<<std::endl;
+
+    _compileOnNextDraw = false;
+    
+    osgUtil::SceneView* sceneView = _sceneView[0].get();
+    if (!sceneView || _done) return;
+
+    if (sceneView->getSceneData()) 
+    {
+        osgUtil::GLObjectsVisitor glov;
+        glov.setState(sceneView->getState());
+        sceneView->getSceneData()->accept(glov);
+    }
+}
 
 void Renderer::cull()
 {
@@ -302,6 +335,33 @@ void Renderer::cull()
             stats->setAttribute(frameNumber, "Cull traversal time taken", osg::Timer::instance()->delta_s(beforeCullTick, afterCullTick));
         }
 
+        if (stats && stats->collectStats("scene"))
+        {
+            osgUtil::Statistics sceneStats;
+            sceneView->getStats(sceneStats);
+            
+            stats->setAttribute(frameNumber, "Visible vertex count", static_cast<double>(sceneStats._vertexCount));
+            stats->setAttribute(frameNumber, "Visible number of drawables", static_cast<double>(sceneStats.numDrawables));
+            stats->setAttribute(frameNumber, "Visible number of lights", static_cast<double>(sceneStats.nlights));
+            stats->setAttribute(frameNumber, "Visible number of render bins", static_cast<double>(sceneStats.nbins));
+            stats->setAttribute(frameNumber, "Visible depth", static_cast<double>(sceneStats.depth));
+            stats->setAttribute(frameNumber, "Visible number of materials", static_cast<double>(sceneStats.nummat));
+            stats->setAttribute(frameNumber, "Visible number of impostors", static_cast<double>(sceneStats.nimpostor));
+
+            osgUtil::Statistics::PrimitiveCountMap& pcm = sceneStats.getPrimitiveCountMap();
+            stats->setAttribute(frameNumber, "Visible number of GL_POINTS", static_cast<double>(pcm[GL_POINTS]));
+            stats->setAttribute(frameNumber, "Visible number of GL_LINES", static_cast<double>(pcm[GL_LINES]));
+            stats->setAttribute(frameNumber, "Visible number of GL_LINE_STRIP", static_cast<double>(pcm[GL_LINE_STRIP]));
+            stats->setAttribute(frameNumber, "Visible number of GL_LINE_LOOP", static_cast<double>(pcm[GL_LINE_LOOP]));
+            stats->setAttribute(frameNumber, "Visible number of GL_TRIANGLES", static_cast<double>(pcm[GL_TRIANGLES]));
+            stats->setAttribute(frameNumber, "Visible number of GL_TRIANGLE_STRIP", static_cast<double>(pcm[GL_TRIANGLE_STRIP]));
+            stats->setAttribute(frameNumber, "Visible number of GL_TRIANGLE_FAN", static_cast<double>(pcm[GL_TRIANGLE_FAN]));
+            stats->setAttribute(frameNumber, "Visible number of GL_QUADS", static_cast<double>(pcm[GL_QUADS]));
+            stats->setAttribute(frameNumber, "Visible number of GL_QUAD_STRIP", static_cast<double>(pcm[GL_QUAD_STRIP]));
+            stats->setAttribute(frameNumber, "Visible number of GL_POLYGON", static_cast<double>(pcm[GL_POLYGON]));
+            
+        }
+
         _drawQueue.add(sceneView);
 
     }
@@ -313,7 +373,7 @@ void Renderer::draw()
 {
     DEBUG_MESSAGE<<"draw() "<<this<<std::endl;
 
-    osg::Timer_t startDrawTick = osg::Timer::instance()->tick();
+    // osg::Timer_t startDrawTick = osg::Timer::instance()->tick();
 
     osgUtil::SceneView* sceneView = _drawQueue.takeFront();
 
@@ -322,8 +382,13 @@ void Renderer::draw()
     osg::GraphicsContext* compileContext = sceneView ? osg::GraphicsContext::getCompileContext(sceneView->getState()->getContextID()) : 0;
     osg::GraphicsThread* compileThread = compileContext ? compileContext->getGraphicsThread() : 0;
 
-    if (sceneView || _done)
+    if (sceneView && !_done)
     {
+        if (_compileOnNextDraw)
+        {
+            compile();
+        }
+    
         osgViewer::View* view = dynamic_cast<osgViewer::View*>(_camera->getView());
         osgDB::DatabasePager* databasePager = view ? view->getDatabasePager() : 0;
 
@@ -419,6 +484,7 @@ void Renderer::draw()
             stats->setAttribute(frameNumber, "Draw traversal end time", osg::Timer::instance()->delta_s(_startTick, afterDrawTick));
             stats->setAttribute(frameNumber, "Draw traversal time taken", osg::Timer::instance()->delta_s(beforeDrawTick, afterDrawTick));
         }
+
     }
 
     DEBUG_MESSAGE<<"end draw() "<<this<<std::endl;
@@ -431,19 +497,25 @@ void Renderer::cull_draw()
     osgUtil::SceneView* sceneView = _sceneView[0].get();
     if (!sceneView || _done) return;
 
+    if (_done)
+    {
+        osg::notify(osg::INFO)<<"Render::release() causing cull_draw to exit"<<std::endl;
+        return;
+    }
+    
     updateSceneView(sceneView);
 
+    if (_compileOnNextDraw)
+    {
+        compile();
+    }
+
     osgViewer::View* view = dynamic_cast<osgViewer::View*>(_camera->getView());
     osgDB::DatabasePager* databasePager = view ? view->getDatabasePager() : 0;
 
     osg::GraphicsContext* compileContext = osg::GraphicsContext::getCompileContext(sceneView->getState()->getContextID());
     osg::GraphicsThread* compileThread = compileContext ? compileContext->getGraphicsThread() : 0;
 
-    if (_done)
-    {
-        osg::notify(osg::INFO)<<"Render::release() causing cull_draw to exit"<<std::endl;
-        return;
-    }
 
     // osg::notify(osg::NOTICE)<<"RenderingOperation"<<std::endl;
 
@@ -475,6 +547,20 @@ void Renderer::cull_draw()
 
     osg::Timer_t afterCullTick = osg::Timer::instance()->tick();
 
+    if (stats && stats->collectStats("scene"))
+    {
+        osgUtil::Statistics sceneStats;
+        sceneView->getStats(sceneStats);
+
+        stats->setAttribute(frameNumber, "Visible vertex count", static_cast<double>(sceneStats._vertexCount));
+        stats->setAttribute(frameNumber, "Visible number of drawables", static_cast<double>(sceneStats.numDrawables));
+        stats->setAttribute(frameNumber, "Visible number of lights", static_cast<double>(sceneStats.nlights));
+        stats->setAttribute(frameNumber, "Visible number of render bins", static_cast<double>(sceneStats.nbins));
+        stats->setAttribute(frameNumber, "Visible depth", static_cast<double>(sceneStats.depth));
+        stats->setAttribute(frameNumber, "Visible number of materials", static_cast<double>(sceneStats.nummat));
+        stats->setAttribute(frameNumber, "Visible number of impostors", static_cast<double>(sceneStats.nimpostor));
+    }
+
 #if 0
     if (state->getDynamicObjectCount()==0 && state->getDynamicObjectRenderingCompletedCallback())
     {
