@@ -12,6 +12,7 @@
 */
 
 #include <stdio.h>
+#include <stdlib.h>
 
 #include <osg/DeleteHandler>
 #include <osg/io_utils>
@@ -25,8 +26,8 @@
 #include <osgViewer/Renderer>
 #include <osgViewer/CompositeViewer>
 
-
 #include <sstream>
+#include <string.h>
 
 using namespace osgViewer;
 
@@ -43,6 +44,21 @@ Viewer::Viewer(osg::ArgumentParser& arguments)
     _viewerBase = this;
 
     constructorInit();
+
+    // Add help for command-line options read here
+    arguments.getApplicationUsage()->addCommandLineOption("--SingleThreaded","Select SingleThreaded threading model for viewer.");
+    arguments.getApplicationUsage()->addCommandLineOption("--CullDrawThreadPerContext","Select CullDrawThreadPerContext threading model for viewer.");
+    arguments.getApplicationUsage()->addCommandLineOption("--DrawThreadPerContext","Select DrawThreadPerContext threading model for viewer.");
+    arguments.getApplicationUsage()->addCommandLineOption("--CullThreadPerCameraDrawThreadPerContext","Select CullThreadPerCameraDrawThreadPerContext threading model for viewer.");
+    arguments.getApplicationUsage()->addCommandLineOption("--clear-color <color>","Set the background color of the viewer in the form \"r,g,b[,a]\".");
+    arguments.getApplicationUsage()->addCommandLineOption("--screen <num>","Set the screen to use when multiple screens are present.");
+    arguments.getApplicationUsage()->addCommandLineOption("--window <x y w h>","Set the position (x,y) and size (w,h) of the viewer window.");
+    // FIXME: Uncomment these lines when the options have been documented properly
+    //arguments.getApplicationUsage()->addCommandLineOption("--3d-sd","");
+    //arguments.getApplicationUsage()->addCommandLineOption("--panoramic-sd","");
+    //arguments.getApplicationUsage()->addCommandLineOption("--radius","");
+    //arguments.getApplicationUsage()->addCommandLineOption("--collar","");
+    //arguments.getApplicationUsage()->addCommandLineOption("--im","");
     
     std::string filename;
     bool readConfig = false;
@@ -76,7 +92,38 @@ Viewer::Viewer(osg::ArgumentParser& arguments)
     while (arguments.read("--window",x,y,width,height)) {}
     
     bool ss3d = false;
-    if ((ss3d=arguments.read("--3d-sd")) || arguments.read("--panoramic-sd"))
+    bool wowvx20 = false;
+    bool wowvx42 = false;
+    if ((wowvx20=arguments.read("--wowvx-20")) || (wowvx42=arguments.read("--wowvx-42")) || arguments.read("--wowvx"))
+    {
+        int wow_content=0x02, wow_factor=0x40, wow_offset=0x80;
+        float wow_Zd, wow_vz, wow_M, wow_C;
+        if (wowvx20){
+            wow_Zd = 0.459813f;
+            wow_vz = 6.180772f;
+            wow_M = -1586.34f;
+            wow_C = 127.5f;
+        }
+        else if (wowvx42){
+            wow_Zd = 0.467481f;
+            wow_vz = 7.655192f;
+            wow_M = -1960.37f;
+            wow_C = 127.5f;
+        }
+
+        while (arguments.read("--wow-content",wow_content)) {}
+        while (arguments.read("--wow-factor",wow_factor)) {}
+        while (arguments.read("--wow-offset",wow_offset)) {}
+        while (arguments.read("--wow-zd",wow_Zd)) {}
+        while (arguments.read("--wow-vz",wow_vz)) {}
+        while (arguments.read("--wow-M",wow_M)) {}
+        while (arguments.read("--wow-C",wow_C)) {}
+            
+        if (screenNum<0) screenNum = 0;
+        
+        setUpViewForWoWVxDisplay( screenNum, wow_content, wow_factor, wow_offset, wow_Zd, wow_vz, wow_M, wow_C );
+    }
+    else if ((ss3d=arguments.read("--3d-sd")) || arguments.read("--panoramic-sd"))
     {
         double radius = 1.0;
         while (arguments.read("--radius",radius)) {}
@@ -130,7 +177,7 @@ void Viewer::constructorInit()
     _updateVisitor = new osgUtil::UpdateVisitor;
     _updateVisitor->setFrameStamp(_frameStamp.get());
 
-    setStats(new osg::Stats("Viewer"));
+    setViewerStats(new osg::Stats("Viewer"));
 }
 
 Viewer::~Viewer()
@@ -242,8 +289,6 @@ bool Viewer::readConfiguration(const std::string& filename)
         osg::notify(osg::NOTICE)<<"Error: Config file \""<<filename<<"\" does not contain a valid Viewer configuration."<<std::endl;
         return false;
     }
-    
-    return false;
 }
 
 bool Viewer::isRealized() const
@@ -270,6 +315,8 @@ int Viewer::run()
     {
         setCameraManipulator(new osgGA::TrackballManipulator());
     }
+    
+    setReleaseContextAtEndOfFrameHint(false);
             
     return ViewerBase::run();
 }
@@ -312,16 +359,6 @@ void Viewer::setSceneData(osg::Node* node)
     setReferenceTime(0.0);
 
     View::setSceneData(node);
-
-    if (_threadingModel!=SingleThreaded && getSceneData())
-    {
-        // make sure that existing scene graph objects are allocated with thread safe ref/unref
-        getSceneData()->setThreadSafeRefUnref(true);
-        
-        // update the scene graph so that it has enough GL object buffer memory for the graphics contexts that will be using it.
-        getSceneData()->resizeGLObjectBuffers(osg::DisplaySettings::instance()->getMaxNumberOfGraphicsContexts());
-    }
-
 }
 
 GraphicsWindowEmbedded* Viewer::setUpViewerAsEmbeddedInWindow(int x, int y, int width, int height)
@@ -479,15 +516,15 @@ void Viewer::advance(double simulationTime)
         _frameStamp->setSimulationTime(simulationTime);
     }
     
-    if (getStats() && getStats()->collectStats("frame_rate"))
+    if (getViewerStats() && getViewerStats()->collectStats("frame_rate"))
     {
         // update previous frame stats
         double deltaFrameTime = _frameStamp->getReferenceTime() - prevousReferenceTime;
-        getStats()->setAttribute(previousFrameNumber, "Frame duration", deltaFrameTime);
-        getStats()->setAttribute(previousFrameNumber, "Frame rate", 1.0/deltaFrameTime);
+        getViewerStats()->setAttribute(previousFrameNumber, "Frame duration", deltaFrameTime);
+        getViewerStats()->setAttribute(previousFrameNumber, "Frame rate", 1.0/deltaFrameTime);
 
         // update current frames stats
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Reference time", _frameStamp->getReferenceTime());
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Reference time", _frameStamp->getReferenceTime());
     }
 
     if (osg::Referenced::getDeleteHandler())
@@ -756,26 +793,6 @@ void Viewer::eventTraversal()
         
     if (_done) return;
 
-    for(osgGA::EventQueue::Events::iterator itr = events.begin();
-        itr != events.end();
-        ++itr)
-    {
-        osgGA::GUIEventAdapter* event = itr->get();
-
-        for(EventHandlers::iterator hitr = _eventHandlers.begin();
-            hitr != _eventHandlers.end();
-            ++hitr)
-        {
-            (*hitr)->handleWithCheckAgainstIgnoreHandledEventsMask( *event, *this, 0, 0);
-        }
-
-        if (_cameraManipulator.valid())
-        {
-            _cameraManipulator->handleWithCheckAgainstIgnoreHandledEventsMask( *event, *this);
-        }
-        
-    }
-
     if (_eventVisitor.valid() && getSceneData())
     {
         _eventVisitor->setFrameStamp(getFrameStamp());
@@ -810,14 +827,41 @@ void Viewer::eventTraversal()
         }
     }
 
-    if (getStats() && getStats()->collectStats("event"))
+
+    for(osgGA::EventQueue::Events::iterator itr = events.begin();
+        itr != events.end();
+        ++itr)
+    {
+        osgGA::GUIEventAdapter* event = itr->get();
+
+        for(EventHandlers::iterator hitr = _eventHandlers.begin();
+            hitr != _eventHandlers.end();
+            ++hitr)
+        {
+            (*hitr)->handleWithCheckAgainstIgnoreHandledEventsMask( *event, *this, 0, 0);
+        }
+        
+    }
+
+    for(osgGA::EventQueue::Events::iterator itr = events.begin();
+        itr != events.end();
+        ++itr)
+    {
+        osgGA::GUIEventAdapter* event = itr->get();
+        if (_cameraManipulator.valid())
+        {
+            _cameraManipulator->handleWithCheckAgainstIgnoreHandledEventsMask( *event, *this);
+        }
+    }
+    
+    if (getViewerStats() && getViewerStats()->collectStats("event"))
     {
         double endEventTraversal = osg::Timer::instance()->delta_s(_startTick, osg::Timer::instance()->tick());
 
         // update current frames stats
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal begin time", beginEventTraversal);
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal end time", endEventTraversal);
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal time taken", endEventTraversal-beginEventTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal begin time", beginEventTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal end time", endEventTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Event traversal time taken", endEventTraversal-beginEventTraversal);
     }
 
 }
@@ -832,16 +876,16 @@ void Viewer::updateTraversal()
     _updateVisitor->setFrameStamp(getFrameStamp());
     _updateVisitor->setTraversalNumber(getFrameStamp()->getFrameNumber());
 
-    if (getSceneData())
-    {
-        getSceneData()->accept(*_updateVisitor);
-    }
-    
-    if (_scene->getDatabasePager())
-    {    
-        // synchronize changes required by the DatabasePager thread to the scene graph
-        _scene->getDatabasePager()->updateSceneGraph(_frameStamp->getReferenceTime());
-    }
+    _scene->updateSceneGraph(*_updateVisitor);
+
+    // if we have a shared state manager prune any unused entries
+    if (osgDB::Registry::instance()->getSharedStateManager())
+        osgDB::Registry::instance()->getSharedStateManager()->prune();
+
+    // update the Registry object cache.
+    osgDB::Registry::instance()->updateTimeStampOfObjectsInCacheWithExternalReferences(*getFrameStamp());
+    osgDB::Registry::instance()->removeExpiredObjectsInCache(*getFrameStamp());
+
 
     if (_updateOperations.valid())
     {
@@ -875,46 +919,33 @@ void Viewer::updateTraversal()
 
     updateSlaves();
 
-    if (getStats() && getStats()->collectStats("update"))
+    if (getViewerStats() && getViewerStats()->collectStats("update"))
     {
         double endUpdateTraversal = osg::Timer::instance()->delta_s(_startTick, osg::Timer::instance()->tick());
 
         // update current frames stats
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal begin time", beginUpdateTraversal);
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal end time", endUpdateTraversal);
-        getStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal time taken", endUpdateTraversal-beginUpdateTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal begin time", beginUpdateTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal end time", endUpdateTraversal);
+        getViewerStats()->setAttribute(_frameStamp->getFrameNumber(), "Update traversal time taken", endUpdateTraversal-beginUpdateTraversal);
     }
 }
 
 void Viewer::getScenes(Scenes& scenes, bool onlyValid)
 {
+    scenes.clear();
     scenes.push_back(_scene.get());
 }
 
 void Viewer::getViews(Views& views, bool onlyValid)
 {
+    views.clear();
     views.push_back(this);
 }
 
-void Viewer::getWindows(Windows& windows, bool onlyValid)
-{
-    windows.clear();
-
-    Contexts contexts;
-    getContexts(contexts, onlyValid);
-    
-    for(Contexts::iterator itr = contexts.begin();
-        itr != contexts.end();
-        ++itr)
-    {
-        osgViewer::GraphicsWindow* gw = dynamic_cast<osgViewer::GraphicsWindow*>(*itr);
-        if (gw) windows.push_back(gw);
-    }
-}
-
-
 void Viewer::getAllThreads(Threads& threads, bool onlyActive)
 {
+    threads.clear();
+
     OperationThreads operationThreads;
     getOperationThreads(operationThreads);
     
@@ -925,11 +956,21 @@ void Viewer::getAllThreads(Threads& threads, bool onlyActive)
         threads.push_back(*itr);
     }
     
-    if (_scene.valid() && 
-        _scene->getDatabasePager() &&
-       (!onlyActive || _scene->getDatabasePager()->isRunning())) 
+
+    if (_scene.valid())
     {
-        threads.push_back(_scene->getDatabasePager());
+        osgDB::DatabasePager* dp = _scene->getDatabasePager();
+        if (dp)
+        {
+            for(unsigned int i=0; i<dp->getNumDatabaseThreads(); ++i)
+            {
+                osgDB::DatabasePager::DatabaseThread* dt = dp->getDatabaseThread(i);
+                if (!onlyActive || dt->isRunning())
+                {
+                    threads.push_back(dt);
+                }
+            }
+        }
     }
 }
 
@@ -968,68 +1009,34 @@ void Viewer::getOperationThreads(OperationThreads& threads, bool onlyActive)
     
 }
 
-struct LessGraphicsContext
-{
-    bool operator () (const osg::GraphicsContext* lhs, const osg::GraphicsContext* rhs) const
-    {
-        int screenLeft = lhs->getTraits()? lhs->getTraits()->screenNum : 0;
-        int screenRight = rhs->getTraits()? rhs->getTraits()->screenNum : 0;
-        if (screenLeft < screenRight) return true;
-        if (screenLeft > screenRight) return false;
-
-        screenLeft = lhs->getTraits()? lhs->getTraits()->x : 0;
-        screenRight = rhs->getTraits()? rhs->getTraits()->x : 0;
-        if (screenLeft < screenRight) return true;
-        if (screenLeft > screenRight) return false;
-
-        screenLeft = lhs->getTraits()? lhs->getTraits()->y : 0;
-        screenRight = rhs->getTraits()? rhs->getTraits()->y : 0;
-        if (screenLeft < screenRight) return true;
-        if (screenLeft > screenRight) return false;
-        
-        return lhs < rhs;
-    } 
-};
-
-
-
 void Viewer::getContexts(Contexts& contexts, bool onlyValid)
 {
     typedef std::set<osg::GraphicsContext*> ContextSet;
     ContextSet contextSet;
 
+    contexts.clear();
+
     if (_camera.valid() && 
         _camera->getGraphicsContext() && 
         (_camera->getGraphicsContext()->valid() || !onlyValid))
     {
         contextSet.insert(_camera->getGraphicsContext());
+        contexts.push_back(_camera->getGraphicsContext());
     }
     
     for(unsigned int i=0; i<getNumSlaves(); ++i)
     {
         Slave& slave = getSlave(i);
-        if (slave._camera.valid() && 
-            slave._camera->getGraphicsContext() && 
-            (slave._camera->getGraphicsContext()->valid() || !onlyValid))
+        osg::GraphicsContext* sgc = slave._camera.valid() ? slave._camera->getGraphicsContext() : 0;
+        if (sgc && (sgc->valid() || !onlyValid))
         {
-            contextSet.insert(slave._camera->getGraphicsContext());
+            if (contextSet.count(sgc)==0)
+            {
+                contextSet.insert(sgc);
+                contexts.push_back(sgc);
+            }
         }
     }
-
-    contexts.clear();
-    contexts.reserve(contextSet.size());
-
-    for(ContextSet::iterator itr = contextSet.begin();
-        itr != contextSet.end();
-        ++itr)
-    {
-        contexts.push_back(const_cast<osg::GraphicsContext*>(*itr));
-    }
-
-    if (contexts.size()>=2)
-    {
-        std::sort(contexts.begin(), contexts.end(), LessGraphicsContext()); 
-    }
 }
 
 void Viewer::getCameras(Cameras& cameras, bool onlyActive)
