@@ -15,6 +15,7 @@
 #include <string.h>
 
 #include <osgViewer/ViewerBase>
+#include <osgViewer/View>
 #include <osgViewer/Renderer>
 
 #include <osg/io_utils>
@@ -26,8 +27,7 @@
 
 #include <osgUtil/Optimizer>
 #include <osgUtil/IntersectionVisitor>
-#include <osgUtil/GLObjectsVisitor>
-#include <osgUtil/RenderLeaf>
+#include <osgUtil/Statistics>
 
 static osg::ApplicationUsageProxy ViewerBase_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_CONFIG_FILE <filename>","Specify a viewer configuration file to load by default.");
 static osg::ApplicationUsageProxy ViewerBase_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_THREADING <value>","Set the threading model using by Viewer, <value> can be SingleThreaded, CullDrawThreadPerContext, DrawThreadPerContext or CullThreadPerCameraDrawThreadPerContext.");
@@ -44,6 +44,7 @@ ViewerBase::ViewerBase():
     _done = false;
     _keyEventSetsDone = osgGA::GUIEventAdapter::KEY_Escape;
     _quitEventSetsDone = true;
+    _releaseContextAtEndOfFrameHint = true;
     _threadingModel = AutomaticSelection;
     _threadsRunning = false;
     _endBarrierPosition = AfterSwapBuffers;
@@ -56,6 +57,7 @@ ViewerBase::ViewerBase(const ViewerBase& base):
     _done = false;
     _keyEventSetsDone = osgGA::GUIEventAdapter::KEY_Escape;
     _quitEventSetsDone = true;
+    _releaseContextAtEndOfFrameHint = true;
     _threadingModel = AutomaticSelection;
     _threadsRunning = false;
     _endBarrierPosition = AfterSwapBuffers;
@@ -144,9 +146,14 @@ void ViewerBase::setUpThreading()
                     ++itr)
                 {
                     Scene* scene = *itr;
-                    if (scene && scene->getDatabasePager())
+                    osgDB::DatabasePager* dp = scene->getDatabasePager();
+                    if (dp)
                     {
-                        scene->getDatabasePager()->setProcessorAffinity(1);
+                        for(unsigned int i=0; i<dp->getNumDatabaseThreads(); ++i)
+                        {
+                            osgDB::DatabasePager::DatabaseThread* dt = dp->getDatabaseThread(i);
+                            dt->setProcessorAffinity(1);
+                        }
                     }
                 }
             
@@ -281,7 +288,6 @@ void ViewerBase::startThreading()
     // using multi-threading so make sure that new objects are allocated with thread safe ref/unref
     osg::Referenced::setThreadSafeReferenceCounting(true);
 
-
     Scenes scenes;
     getScenes(scenes);
     for(Scenes::iterator scitr = scenes.begin();
@@ -367,6 +373,12 @@ void ViewerBase::startThreading()
     {
         osg::GraphicsContext* gc = (*citr);
         
+        if (!gc->isRealized())
+        {
+            osg::notify(osg::INFO)<<"ViewerBase::startThreading() : Realizng window "<<gc<<std::endl;
+            gc->realize();
+        }
+        
         gc->getState()->setDynamicObjectRenderingCompletedCallback(_endDynamicDrawBlock.get());
 
         // create the a graphics thread for this context
@@ -375,8 +387,6 @@ void ViewerBase::startThreading()
         if (affinity) gc->getGraphicsThread()->setProcessorAffinity(processNum % numProcessors);
         threadAffinityMap[gc->getGraphicsThread()] = processNum % numProcessors;
 
-        gc->getGraphicsThread()->add(new osgUtil::GLObjectsOperation());
-
         // add the startRenderingBarrier
         if (_threadingModel==CullDrawThreadPerContext && _startRenderingBarrier.valid()) gc->getGraphicsThread()->add(_startRenderingBarrier.get());
 
@@ -492,6 +502,22 @@ void ViewerBase::startThreading()
     osg::notify(osg::INFO)<<"Set up threading"<<std::endl;
 }
 
+void ViewerBase::getWindows(Windows& windows, bool onlyValid)
+{
+    windows.clear();
+
+    Contexts contexts;
+    getContexts(contexts, onlyValid);
+    
+    for(Contexts::iterator itr = contexts.begin();
+        itr != contexts.end();
+        ++itr)
+    {
+        osgViewer::GraphicsWindow* gw = dynamic_cast<osgViewer::GraphicsWindow*>(*itr);
+        if (gw) windows.push_back(gw);
+    }
+}
+
 void ViewerBase::checkWindowStatus()
 {
     Contexts contexts;
@@ -586,6 +612,25 @@ void ViewerBase::frame(double simulationTime)
 
 void ViewerBase::renderingTraversals()
 {
+    bool _outputMasterCameraLocation = false;
+    if (_outputMasterCameraLocation)
+    {
+        Views views;
+        getViews(views);
+
+        for(Views::iterator itr = views.begin();
+            itr != views.end();
+            ++itr)
+        {
+            osgViewer::View* view = *itr;
+            if (view)
+            {
+                const osg::Matrixd& m = view->getCamera()->getInverseViewMatrix();
+                osg::notify(osg::NOTICE)<<"View "<<view<<", Master Camera position("<<m.getTrans()<<"), rotation("<<m.getRotate()<<")"<<std::endl;
+            }
+        }
+    }
+        
     // check to see if windows are still valid
     checkWindowStatus();
 
@@ -595,6 +640,67 @@ void ViewerBase::renderingTraversals()
 
     osg::FrameStamp* frameStamp = getViewerFrameStamp();
 
+    if (getViewerStats() && getViewerStats()->collectStats("scene"))
+    {
+        int frameNumber = frameStamp ? frameStamp->getFrameNumber() : 0;
+    
+        Views views;
+        getViews(views);
+        for(Views::iterator vitr = views.begin();
+            vitr != views.end();
+            ++vitr)
+        {
+            View* view = *vitr;
+            osg::Stats* stats = view->getStats();            
+            osg::Node* sceneRoot = view->getSceneData();
+            if (sceneRoot && stats)
+            {
+                osgUtil::StatsVisitor statsVisitor;
+                sceneRoot->accept(statsVisitor);
+                statsVisitor.totalUpStats();
+                
+                unsigned int unique_primitives = 0;
+                osgUtil::Statistics::PrimitiveCountMap::iterator pcmitr;
+                for(pcmitr = statsVisitor._uniqueStats.GetPrimitivesBegin();
+                    pcmitr != statsVisitor._uniqueStats.GetPrimitivesEnd();
+                    ++pcmitr)
+                {
+                    unique_primitives += pcmitr->second;
+                }
+
+                stats->setAttribute(frameNumber, "Number of unique StateSet", static_cast<double>(statsVisitor._statesetSet.size()));
+                stats->setAttribute(frameNumber, "Number of unique Group", static_cast<double>(statsVisitor._groupSet.size()));
+                stats->setAttribute(frameNumber, "Number of unique Transform", static_cast<double>(statsVisitor._transformSet.size()));
+                stats->setAttribute(frameNumber, "Number of unique LOD", static_cast<double>(statsVisitor._lodSet.size()));
+                stats->setAttribute(frameNumber, "Number of unique Switch", static_cast<double>(statsVisitor._switchSet.size()));
+                stats->setAttribute(frameNumber, "Number of unique Geode", static_cast<double>(statsVisitor._geodeSet.size()));
+                stats->setAttribute(frameNumber, "Number of unique Drawable", static_cast<double>(statsVisitor._drawableSet.size()));
+                stats->setAttribute(frameNumber, "Number of unique Geometry", static_cast<double>(statsVisitor._geometrySet.size()));
+                stats->setAttribute(frameNumber, "Number of unique Vertices", static_cast<double>(statsVisitor._uniqueStats._vertexCount));
+                stats->setAttribute(frameNumber, "Number of unique Primitives", static_cast<double>(unique_primitives));
+
+                unsigned int instanced_primitives = 0;
+                for(pcmitr = statsVisitor._instancedStats.GetPrimitivesBegin();
+                    pcmitr != statsVisitor._instancedStats.GetPrimitivesEnd();
+                    ++pcmitr)
+                {
+                    instanced_primitives += pcmitr->second;
+                }
+
+                stats->setAttribute(frameNumber, "Number of instanced Stateset", static_cast<double>(statsVisitor._numInstancedStateSet));
+                stats->setAttribute(frameNumber, "Number of instanced Group", static_cast<double>(statsVisitor._numInstancedGroup));
+                stats->setAttribute(frameNumber, "Number of instanced Transform", static_cast<double>(statsVisitor._numInstancedTransform));
+                stats->setAttribute(frameNumber, "Number of instanced LOD", static_cast<double>(statsVisitor._numInstancedLOD));
+                stats->setAttribute(frameNumber, "Number of instanced Switch", static_cast<double>(statsVisitor._numInstancedSwitch));
+                stats->setAttribute(frameNumber, "Number of instanced Geode", static_cast<double>(statsVisitor._numInstancedGeode));
+                stats->setAttribute(frameNumber, "Number of instanced Drawable", static_cast<double>(statsVisitor._numInstancedDrawable));
+                stats->setAttribute(frameNumber, "Number of instanced Geometry", static_cast<double>(statsVisitor._numInstancedGeometry));
+                stats->setAttribute(frameNumber, "Number of instanced Vertices", static_cast<double>(statsVisitor._instancedStats._vertexCount));
+                stats->setAttribute(frameNumber, "Number of instanced Primitives", static_cast<double>(instanced_primitives));
+           }
+        }
+    }
+
     Scenes scenes;
     getScenes(scenes);
     
@@ -608,6 +714,13 @@ void ViewerBase::renderingTraversals()
         {
             dp->signalBeginFrame(frameStamp);
         }
+        
+        if (scene->getSceneData())
+        {
+            // fire off a build of the bounding volumes while we 
+            // are still running single threaded.
+            scene->getSceneData()->getBound();
+        }
     }
 
     // osg::notify(osg::NOTICE)<<std::endl<<"Start frame"<<std::endl;
@@ -620,15 +733,17 @@ void ViewerBase::renderingTraversals()
     getCameras(cameras);
     
     Contexts::iterator itr;
-
-    // dispatch the the rendering threads
-    if (_startRenderingBarrier.valid()) _startRenderingBarrier->block();
+    
+    bool doneMakeCurrentInThisThread = false;
 
     if (_endDynamicDrawBlock.valid())
     {
         _endDynamicDrawBlock->reset();
     }
     
+    // dispatch the rendering threads
+    if (_startRenderingBarrier.valid()) _startRenderingBarrier->block();
+
     // reset any double buffer graphics objects
     for(Cameras::iterator camItr = cameras.begin();
         camItr != cameras.end();
@@ -651,7 +766,8 @@ void ViewerBase::renderingTraversals()
     {
         if (_done) return;
         if (!((*itr)->getGraphicsThread()) && (*itr)->valid())
-        { 
+        {
+            doneMakeCurrentInThisThread = true; 
             makeCurrent(*itr);
             (*itr)->runOperations();
         }
@@ -670,6 +786,7 @@ void ViewerBase::renderingTraversals()
 
         if (!((*itr)->getGraphicsThread()) && (*itr)->valid())
         { 
+            doneMakeCurrentInThisThread = true; 
             makeCurrent(*itr);
             (*itr)->swapBuffers();
         }
@@ -694,17 +811,21 @@ void ViewerBase::renderingTraversals()
         _endDynamicDrawBlock->block();
         // osg::notify(osg::NOTICE)<<"Time waiting "<<osg::Timer::instance()->delta_m(startTick, osg::Timer::instance()->tick())<<std::endl;;
     }
+    
+    if (_releaseContextAtEndOfFrameHint && doneMakeCurrentInThisThread)
+    {
+        //osg::notify(osg::NOTICE)<<"Doing release context"<<std::endl;
+        releaseContext();
+    }
 
-
-
-    if (getStats() && getStats()->collectStats("update"))
+    if (getViewerStats() && getViewerStats()->collectStats("update"))
     {
         double endRenderingTraversals = elapsedTime();
 
         // update current frames stats
-        getStats()->setAttribute(frameStamp->getFrameNumber(), "Rendering traversals begin time ", beginRenderingTraversals);
-        getStats()->setAttribute(frameStamp->getFrameNumber(), "Rendering traversals end time ", endRenderingTraversals);
-        getStats()->setAttribute(frameStamp->getFrameNumber(), "Rendering traversals time taken", endRenderingTraversals-beginRenderingTraversals);
+        getViewerStats()->setAttribute(frameStamp->getFrameNumber(), "Rendering traversals begin time ", beginRenderingTraversals);
+        getViewerStats()->setAttribute(frameStamp->getFrameNumber(), "Rendering traversals end time ", endRenderingTraversals);
+        getViewerStats()->setAttribute(frameStamp->getFrameNumber(), "Rendering traversals time taken", endRenderingTraversals-beginRenderingTraversals);
     }
 }
 
